!   Make_Rch.f90 
!
!   Program 4 of the COHYST SUITE of programs
!  
!   Purpose:
!   Compile all sources of recharge in the model region and properly format the
!   results into annual output files.
!
!   Development log:
!   February 2014, Isaac Mortensen - TFG
!
!****************************************************************************
!  
!
!****************************************************************************
MODULE PARAM

    DATA MasterDIR /'W:\COHYST\COHYST\Run028\Run028b2\Inputs\'/
    DATA MasterFile /'MasterInput_v27.1.txt'/
    
    CHARACTER*12, ALLOCATABLE ::    Crops(:)
    
    CHARACTER*1000  INDIR, OUTDIR, WBPDIR, LUDIR, CNLDIR, MasterDIR
    CHARACTER*1000  MISCDIR, MIDIR
    CHARACTER*250   GWLUfld, SWLUfld, COLUfld, DryLUfld, GWCfld
    CHARACTER*250   GWLU, SWLU, COLU, DryLU, cRun, MODname
    CHARACTER*250   CellLoc, Coeffile, AEfile, ROZfile, GWCfile, MasterFile
    CHARACTER*250   PctRchfile, SWidxfile, CountID, Cell_county
    CHARACTER*250   cnlmastfile, miscrmastfile, miscpmastfile, mirmastfile, mipmastfile
    CHARACTER*250   ActCellFile
    !Added 1/7/2016
    CHARACTER*250   Callyrfile
    
    INTEGER     ncells, nczones, nrzones, nsoils, nirrdist, ncrops
    INTEGER     startyr, endyr, DPonly, SWonly
    INTEGER     ncols, nrows, cnlrflg, miscrflg, miscpflg, mirflg, mipflg
    INTEGER     Rch0flg
    !Added 1/7/2016
    INTEGER     ctrlyrs
    
    REAL    csize
    
    CHARACTER*250   rchfld, rchfile
    CHARACTER*4     YRT
    !Added 1/7/2016
    CHARACTER*4     YRTcnl, YRTmisc, YRTmi
    
    INTEGER     i, fcol, pcol, izone        ! IM 11-25 added izone
    INTEGER     cell, iyear, startmon, endmon, DayIM(12)
    INTEGER     yr, pflag, swdist, imon, irow, iline
    INTEGER     icell, col, row, zone, idx
    
    REAL    acs, annval, monval(12)
    REAL    LF, SF, RO2DP, RO2ET, totrch
    
    INTEGER, ALLOCATABLE :: soil(:), CoefZ(:), ROZ(:)
    INTEGER, ALLOCATABLE :: ZeroRCH(:)
    
    !Added 1/7/2016
    INTEGER, ALLOCATABLE :: yrctrl(:)
    INTEGER, ALLOCATABLE :: CNLyear(:), MISCyear(:), MIyear(:)
    
    REAL, ALLOCATABLE :: M2G(:)
    REAL, ALLOCATABLE :: IrrPrecDP(:,:), IrrPrecRO(:,:), LatSeep(:,:)
    REAL, ALLOCATABLE :: CanalRch(:, :, :), miscrch(:, :), MunIrch(:, :)
    REAL, ALLOCATABLE :: CellRCH(:, :)
    REAL, ALLOCATABLE :: LPM(:), PctRCH(:)
    REAL, ALLOCATABLE :: ROZsf(:), ROZro2dp(:), ROZro2et(:)       ! IM 11-25 added ROZsf
    
    DATA DayIM /31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31/
    
END MODULE PARAM
!****************************************************************************
PROGRAM Make_Rch
    USE PARAM
    IMPLICIT NONE
    
    OPEN (1, FILE = TRIM(MasterDIR)//TRIM(MasterFile), STATUS = 'OLD')
    READ (1, *)

    READ (1, *) cRun
    READ (1, *) MODname
    READ (1, *) INDIR
    READ (1, *) OUTDIR
    READ (1, *) WBPDIR
    READ (1, *) LUDIR
    READ (1, *) CNLDIR
    READ (1, *) MiscDIR
    READ (1, *) MIDIR
    READ (1, *) GWLUfld
    READ (1, *) SWLUfld
    READ (1, *) COLUfld
    READ (1, *) DryLUfld
    READ (1, *) GWLU
    READ (1, *) SWLU
    READ (1, *) COLU
    READ (1, *) DryLU
    READ (1, *) GWCfld
    READ (1, *) GWCfile
    READ (1, *) CellLoc
    READ (1, *) Coeffile
    READ (1, *) AEfile
    READ (1, *) ROZfile
    READ (1, *) PctRchfile
    READ (1, *) SWidxfile
    READ (1, *) CountID
    READ (1, *) Cell_county
    READ (1, *) CNLmastfile
    READ (1, *) MISCrmastfile
    READ (1, *) MISCpmastfile
    READ (1, *) MIrmastfile
    READ (1, *) MIpmastfile
    READ (1, *) ActCellfile
    !Added 1/7/2016
    READ (1, *) CallYrfile
    READ (1, *) ncells
    READ (1, *) startyr
    READ (1, *) endyr
    READ (1, *) RCH0flg
    READ (1, *) DPonly
    READ (1, *) SWonly
    READ (1, *) cnlrflg
    READ (1, *) miscrflg
    READ (1, *) miscpflg
    READ (1, *) MIrflg
    READ (1, *) MIpflg
    !Added 1/7/2016
    READ (1, *) ctrlyrs
    READ (1, *) nczones
    READ (1, *) nrzones
    READ (1, *) nsoils
    READ (1, *) nirrdist
    READ (1, *) csize
    READ (1, *) ncols
    READ (1, *) nrows
    READ (1, *) ncrops
    
    ALLOCATE (Crops(ncrops))
    
    DO i = 1, ncrops
        READ (1, *) crops(i)
    END DO

    CLOSE (1)

    ALLOCATE (soil(ncells), CoefZ(ncells), ROZ(ncells), M2G(ncells))
    ALLOCATE (IrrPrecDP(ncells, 12), IrrPrecRO(ncells, 12), LatSeep(ncells, 12))
    ALLOCATE (CanalRch(ncells, 2, 12), miscrch(ncells, 12), Munirch(ncells, 12))
    ALLOCATE (CellRch(nrows, ncols))
    ALLOCATE (LPM(0 : nrzones))
    ALLOCATE (PctRch(nczones))
    ALLOCATE (ROZsf(0 : nrzones), ROZro2dp(0 : nrzones), ROZro2et(0 : nrzones))       ! IM 11-25 added line ROZsf
    ALLOCATE (ZeroRCH(ncells))
    
    !Added 1/7/2016
    ALLOCATE (yrctrl(ctrlyrs))
    ALLOCATE (CNLyear(startyr : endyr), MISCyear(startyr : endyr), MIyear(startyr : endyr))
    
    OUTDIR = TRIM(OUTDIR)//TRIM(cRun)//'\Results\'
    
    fcol = ncols / 10
    pcol = MOD(ncols, 10)
    
    !Populate cell soils, zones, and location
    WRITE (6, *) 'Reading Cell Location'
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CellLoc), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, Soil(cell), CoefZ(cell), ROZ(cell), M2G(cell)
    END DO
    CLOSE (1)
    
    !Populate Runoff Zone Coefficients
    OPEN (1, FILE = TRIM(INDIR)//TRIM(ROZfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) zone, LPM(zone)
    END DO
    CLOSE (1)
    
    !Populate PctRch
    OPEN (1, FILE = TRIM(INDIR)//TRIM(PctRchfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) zone, pctrch(zone)
    END DO
    CLOSE (1)    
    
    ZeroRCH = 0
    IF (RCH0flg .EQ. 1) THEN
        !Determine Nulled Cells 
        OPEN (1, FILE = TRIM(INDIR)//TRIM(ActCellfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, idx, ZeroRCH(cell)
        END DO
        CLOSE (1)
    END IF
    
    !Read in Call year
    !Added 1/7/2016 to control the annual input data and allow the same data to be applied to different years
    !ex: applying a fixed land use to the historical climate
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CallYrFile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) yr, yrctrl
        IF ((yr .GE. startyr) .AND. (yr .LE. endyr)) THEN
            CNLyear(yr) = yrctrl(4)
            Miscyear(yr) = yrctrl(5)
            MIyear(yr) = yrctrl(6)
        END IF
    END DO
    CLOSE (1)
    
    OPEN (10, FILE = TRIM(OUTDIR)//'Rch_values.txt', STATUS = 'UNKNOWN')
    WRITE (10, 100)
    
    ! IM 11-25 added monthly stream flow summary
    OPEN (11, FILE = TRIM(OUTDIR)//TRIM(MODname)//'_'//TRIM(cRun)//'SF.txt', STATUS = 'UNKNOWN')
    WRITE (11, 110)
    
    !Added 1/7/2016
    WRITE (6, 600)
    
    DO iyear = startyr, endyr
        WRITE (YRT, '(I4)') iyear
        !Altered 1/7/2016
        WRITE (6, 601) iyear, CNLyear(iyear), MISCyear(iyear), MIyear(iyear)
        
        !Added 1/7/2016
        WRITE (YRTcnl, '(I4)') CNLyear(iyear)
        WRITE (YRTmisc, '(I4)') MISCyear(iyear)
        WRITE (YRTmi, '(I4)') MIyear(iyear)
        
        IF (iyear .LT. 1985) THEN
            startmon = 10
            endmon = 12
        ELSE
            startmon = 1
            endmon = 12
        END IF
        
        IF (MOD(iyear, 4) .EQ. 0) THEN
            DayIM(2) = 29
        ELSE
            DayIM(2) = 28
        END IF
        
        IrrPrecDP = 0.
        IrrPrecRO = 0.
        LatSeep = 0.
                
        OPEN (1, FILE = TRIM(OUTDIR)//'WSPP_out\WSPPDP'//TRIM(YRT)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, yr, pflag, swdist, acs, Annval, monval
            IrrPrecDP(cell, :) = IrrPrecDP(cell, :) + monval
            IF (ZeroRch(cell) .EQ. 1) IrrPrecDP(cell, :) = 0.
        END DO
        CLOSE (1)
        
        OPEN (1, FILE = TRIM(OUTDIR)//'WSPP_out\WSPPRO'//TRIM(YRT)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, yr, pflag, swdist, acs, Annval, monval
            IrrPrecRO(cell, :) = IrrPrecRO(cell, :) + monval
            IF (ZeroRch(cell) .EQ. 1) IrrPrecRO(cell, :) = 0.
        END DO
        CLOSE (1)
        
        OPEN (1, FILE = TRIM(OUTDIR)//'LatLoss\LatLoss'//TRIM(YRT)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, yr, pflag, swdist, acs, Annval, monval
            LatSeep(cell, :) = LatSeep(cell, :) + monval
            IF (ZeroRch(cell) .EQ. 1) LatSeep(cell, :) = 0.
        END DO
    
        CanalRch = 0.
        IF (Cnlrflg .EQ. 1) THEN
            OPEN (2, FILE = TRIM(INDIR)//TRIM(CNLmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2)) 
                READ (2, *) idx, rchfld, rchfile
                
                OPEN (1, FILE = TRIM(CNLDIR)//TRIM(rchfld)//'\'//TRIM(rchfile)//TRIM(YRTcnl)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    canalrch(cell, idx, :) = canalrch(cell, idx, :) + monval
                    IF (ZeroRch(cell) .EQ. 1) canalrch(cell, idx, :) = 0.
                END DO
                CLOSE (1)
                
            END DO
            CLOSE (2)
        END IF
        
        miscrch = 0.
        IF (Miscrflg .EQ. 1) THEN
            OPEN (2, FILE = TRIM(INDIR)//TRIM(MISCrmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2))
                READ (2, *) rchfld, rchfile
                
                OPEN (1, FILE = TRIM(MiscDIR)//TRIM(rchfld)//'\'//TRIM(rchfile)//TRIM(YRTmisc)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    miscrch(cell, :) = miscrch(cell, :) + monval
                    IF (ZeroRch(cell) .EQ. 1) miscrch(cell, :) = 0.
                END DO
                CLOSE (1)
                
            END DO
            CLOSE (2)
        END IF
        
        MunIrch = 0.
        IF (MIrflg .EQ. 1) THEN
            OPEN (2, FILE = TRIM(INDIR)//TRIM(MIrmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2))
                READ (2, *) Rchfld, rchfile
                
                OPEN (1, FILE = TRIM(MIDIR)//TRIM(RCHfld)//'\'//TRIM(rchfile)//TRIM(YRTmi)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    Munirch(cell, :) = MunIrch(cell, :) + monval
                    IF (ZeroRch(cell) .EQ. 1) Munirch(cell, :) = 0.
                END DO
                CLOSE (1)
            
            END DO
            CLOSE (2)
        END IF
        
        OPEN (20, FILE = TRIM(OUTDIR)//'RCH\'//TRIM(MODname)//TRIM(YRT)//'.RCH', STATUS = 'UNKNOWN')
        OPEN (30, FILE = TRIM(OUTDIR)//'SF\SF'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        OPEN (40, FILE = TRIM(OUTDIR)//'RObal\RObal'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        OPEN (50, FILE = TRIM(OUTDIR)//'AnnRch\AnnRCH'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        OPEN (51, FILE = TRIM(OUTDIR)//'AnnRch\MonRCH'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        OPEN (60, FILE = TRIM(OUTDIR)//'Cnlchk\cnlchk'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
    
        WRITE (30, 130)
        WRITE (40, 140)
        WRITE (50, 150)
        WRITE (51, 150)
        WRITE (60, 160)
        
        DO imon = startmon, endmon
            CellRch = 0.
            ROZsf = 0.          ! IM 11-25 added ROZsf initialize
            ROZro2dp = 0.
            ROZro2et = 0.
            !WRITE (6, *) imon
            CALL popcellrch
            
            ! IM 11-25 added SF summary writeout
            DO izone = 1, nrzones
                WRITE (11, 111) iyear, imon, izone, ROZsf(izone)
            END DO
        
            WRITE (20, 120)
            WRITE (20, 121)
        
            DO irow = 1, nrows
                DO iline = 1, fcol
                    DO i = 1, 9
                        IF (Cellrch(irow, (iline - 1) * 10 + i) .GT. 0.) THEN
                            WRITE (20, 122) Cellrch(irow, (iline - 1) * 10 + i)
                        ELSE
                            WRITE (20, 123) Cellrch(irow, (iline - 1) * 10 + i)
                        END IF
                    END DO
                    
                    IF (Cellrch(irow, (iline) * 10) .GT. 0.) THEN
                        WRITE (20, 124) Cellrch(irow, (iline) * 10)
                    ELSE
                        WRITE (20, 125) Cellrch(irow, (iline) * 10)
                    END IF
                END DO
                
                DO i = 1, pcol - 1
                    IF(Cellrch(irow, fcol * 10 + i) .GT. 0.) THEN
                        WRITE (20, 122) Cellrch(irow, fcol * 10 + i)
                    ELSE
                        WRITE (20, 123) Cellrch(irow, fcol * 10 + i)
                    END IF
                END DO
                
                IF (Cellrch(irow, fcol * 10 + pcol) .GT. 0.) THEN
                    WRITE (20, 124) Cellrch(irow, fcol * 10 + pcol)
                ELSE
                    WRITE (20, 125) Cellrch(irow, fcol * 10 + pcol)
                END IF
                
            END DO
        END DO
    END DO
    

100 FORMAT ('Cell, Year, Recharge_(in)')
! IM 11-25 added SF summary formats
110 FORMAT ('Year, Month, RO_Zone, SF_AF')
111 FORMAT (I7, ',', I5, ',', I3, ',', F11.2)
    
120 FORMAT ('         1         1')	!Stress Period Header Line 1 (File 20)
121	FORMAT ('INTERNAL 1.0 (free) -1')	!Stress Period Header Line 2 (File 20)
122 FORMAT (ES11.5, ' ', \)
123 FORMAT (F3.1, ' ', \)
124 FORMAT (ES11.5)
125 FORMAT (F3.1)
    
130 FORMAT ('Cell, ROZone, Row, Column, Year, Month, NumberOfDaysInMonth, SF_AF, IrrPrecRO_AF, LPM, MiToGauge, MonthlySF_AFperDAY')
140 FORMAT ('Cell, Year, Month, IrrPrec_RO, RO2DP, Stream_Flow, RO2ET')
150 FORMAT ('Cell, Year, Month, Tot_Rch, IrrPrec_Rch, IrrPrec_RO, Canal_Rch_NP, Canal_Rch_RR, Lateral_Seepage, Misc_Rch, M&I_Rch, RO2DP, RO2ET, Stream_Flow') 
160 FORMAT ('Cell, Year, Month, Totrch, Canal_Rch_NP, Canal_Rch_RR')    

!Added 1/7/2016
600 FORMAT (' Sim Year, CNL Year,Misc Year,  MI Year')
601 FORMAT (I9, 3(',', I9))
    
END PROGRAM Make_Rch
!****************************************************************************
SUBROUTINE popcellrch
    USE PARAM
    IMPLICIT NONE
    
    DO icell = 1, ncells
        col = MOD(icell, ncols)
        IF (col .EQ. 0) col = ncols
        row = (icell - col) / ncols + 1
        
        IF (M2G(icell) .EQ. 0.) THEN
            LF = 0.5
        ELSE
            LF = MIN(1 - EXP(-LPM(ROZ(icell)) * M2G(icell)), 1.0)
        END IF
        
        SF = IrrPrecRO(icell, imon) * (1 - LF)
        RO2DP = IrrPrecRO(icell, imon) * LF * PctRch(CoefZ(icell))
        RO2ET = IrrPrecRO(icell, imon) * LF * (1 - PctRch(CoefZ(icell)))
        
        ! IM 11-25 added summing of SF per runoff zone
        !IF (ROZ(icell) .GT. 0) THEN
            ROZsf(ROZ(icell)) = ROZsf(ROZ(icell)) + SF
            ROZro2dp(ROZ(icell)) = ROZro2dp(ROZ(icell)) + RO2DP
            ROZro2ET(ROZ(icell)) = ROZro2et(ROZ(icell)) + RO2ET
        !END IF
        
        DO i = 1, 2
            IF (Canalrch(icell, i, imon) .LT. 0.) THEN
                IF (IrrPrecDP(icell, imon) + LatSeep(icell, imon) + RO2DP + Munirch(icell, imon) + Miscrch(icell, imon) .LT. - Canalrch(icell, i, imon)) THEN
                    Canalrch(icell, i, imon) = -(IrrPrecDP(icell, imon) + LatSeep(icell, imon) + RO2DP + Munirch(icell, imon) + Miscrch(icell, imon))
                END IF
            END IF
        END DO
        
        Totrch = IrrPrecDP(icell, imon) + SUM(Canalrch(icell, :, imon)) + Latseep(icell, imon) + RO2DP + MunIrch(icell, imon) + Miscrch(icell, imon)
        Cellrch(row, col) = MAX(TotRch / csize / DayIM(imon), 0.)
        
        IF ((Totrch .GT. 0.) .OR. (SUM(Canalrch(icell, :, imon)) .LT. 0.)) THEN
            WRITE (51, 151) icell, iyear, imon, Totrch, IrrPrecDP(icell, imon), IrrPrecRO(icell, imon), Canalrch(icell, :,  imon), Latseep(icell, imon), &
                                Miscrch(icell, imon), Munirch(icell, imon), RO2DP, RO2ET, SF
            
            WRITE (30, 131) icell, ROZ(icell), row, col, iyear, imon, DayIM(imon), SF, IrrPrecRO(icell, imon), LPM(ROZ(icell)), M2G(icell), SF / DayIM(imon)
        END IF
        
        IF(ABS(SUM(Canalrch(icell, :, imon))) .NE. 0.) THEN
            WRITE (60, 161) icell, iyear, imon, Totrch, Canalrch(icell, :,  imon)
        END IF
        
        IF (IrrPrecRO(icell, imon) .GT. 0.) THEN
            WRITE (40, 141) icell, iyear, imon, IrrPrecRO(icell, imon), RO2DP, SF, RO2ET
        END IF
        
        IF (imon .EQ. 12) THEN
            SF = SUM(IrrPrecRO(icell, :)) * (1 - LF)
            RO2DP = SUM(IrrPrecRO(icell, :)) * LF * PctRch(CoefZ(icell))
            RO2ET = SUM(IrrPrecRO(icell, :)) * LF * (1 - PctRch(COefZ(icell)))
            
            TotRch = SUM(IrrPrecDP(icell, :)) + SUM(Canalrch(icell, :, :)) + SUM(Latseep(icell, :)) + RO2DP + SUM(MunIrch(icell, :)) + SUM(Miscrch(icell, :))
            
            IF (Totrch .GT. 0.) THEN
                WRITE (50, 151) icell, iyear, imon - 12, Totrch, SUM(IrrPrecDP(icell, :)), SUM(IrrPrecRO(icell, :)), (SUM(Canalrch(icell, i, :)), i = 1, 2), SUM(Latseep(icell, :)), &
                                    SUM(Miscrch(icell, :)), SUM(Munirch(icell, :)), RO2DP, RO2ET, SF
                
            END IF
            WRITE (10, 101) icell, iyear, MAX(TotRch, 0.) / csize *12
        END IF
        
        
    END DO
    
101 FORMAT (I7, ',', I5, ',', F6.2)
    
131 FORMAT (I7, 6(',', I5), 2(',', F14.8), ',', F5.2, ',', F8.2, ',', F16.10)
141 FORMAT (I7, 2(',', I5), 4(',', F8.2))
151 FORMAT (I7, 2(',', I5), 11(',', F9.4))
161 FORMAT (I7, 2(',', I5), 3(',', F8.4))

END SUBROUTINE popcellrch
!****************************************************************************