!   WSPP.f90 
!
!   Program 3 of the COHYST SUITE of programs  
!
!   Purpose:
!   Employ irrigation supplies to determine the water balance within each cell
!   of the COHYST model domain.
!
!   Development log:
!   February 2014, Isaac Mortensen - TFG
!
!****************************************************************************
!
!****************************************************************************
MODULE PARAM

    DATA MasterDIR /'W:\COHYST\COHYST\Run028\Run028b2\Inputs\'/
    DATA MasterFile /'MasterInput_v27.1.txt'/

    CHARACTER*12, ALLOCATABLE ::    Crops(:)
    
    CHARACTER*1000  INDIR, OUTDIR, WBPDIR, LUDIR, CNLDIR, MasterDIR
    CHARACTER*1000  MISCDIR, MIDIR
    CHARACTER*250   GWLUfld, SWLUfld, COLUfld, DryLUfld, GWCfld
    CHARACTER*250   GWLU, SWLU, COLU, DryLU, cRun, MODname
    CHARACTER*250   CellLoc, Coeffile, AEfile, ROZfile, GWCfile, MasterFile
    CHARACTER*250   PctRchfile, SWidxfile, CountID, Cell_county
    CHARACTER*250   cnlmastfile, miscrmastfile, miscpmastfile, mirmastfile, mipmastfile
    CHARACTER*250   ActCellFile
    !Added 1/7/2016
    CHARACTER*250   Callyrfile
    
    INTEGER     ncells, nczones, nrzones, nsoils, nirrdist, ncrops
    INTEGER     startyr, endyr, DPonly, SWonly
    INTEGER     ncols, nrows, cnlrflg, miscrflg, miscpflg, mirflg, mipflg
    INTEGER     Rch0flg
    !Added 1/7/2016
    INTEGER     ctrlyrs
            
    REAL    csize
    
    CHARACTER*4     YRT
    CHARACTER*3     Mon(0 : 12)
    !Added 1/7/2016
    CHARACTER*4     YRTlu, YRTwbp, YRTgwc

    INTEGER     cell, iyear, swdist, pflag, swidx, idx
    INTEGER     sidx(56)
    
    
    REAL    Dummy(0 : 12), Pump(0 : 12), Sdel(0 : 12)
    REAL    cellDP(0 : 12), cellRO(0 : 12), cellPump(0 : 12), cellLatloss(0 : 12)
    REAL    cellAcs
    
    REAL    ETMaxadjdry(0 : 12), ETMaxdry(0 : 12)
    REAL    NIRmET(0 : 12), ETtrans(0 : 12)
    REAL    RO1dry(0 : 12), RO2dry(0 : 12), DP1dry(0 : 12), DP2dry(0 : 12)
    REAL    DPdrytot(0 : 12), DP2RO(0 : 12)
    
    REAL    ET(0 : 12), RO1irr(0 : 12), RO2(0 : 12)
    REAL    SL(0 : 12), ETgain(0 : 12), ETbase(0 : 12), DET1(0 : 12), DET2(0 : 12)
    REAL    DP1irr(0 : 12), DP2(0 : 12), DPirrtot(0 : 12), Ewat(0 : 12)
    REAL    Latseep(0 : 12)
    
    INTEGER, ALLOCATABLE :: soil(:), CoefZ(:), ROZ(:)
    INTEGER, ALLOCATABLE :: ntsw(:), ntco(:)
    INTEGER, ALLOCATABLE :: SWDt(:,:,:), COPt(:,:,:), CODt(:,:,:)
    !Added 1/7/2016
    INTEGER, ALLOCATABLE :: LUyear(:), WBPyear(:), GWCyear(:)
    INTEGER, ALLOCATABLE :: yrctrl(:)
    
    REAL, ALLOCATABLE :: M2G(:)
    REAL, ALLOCATABLE :: ADJetdry(:,:,:), ADJetirr(:,:,:), ADJnir(:,:,:)
    REAL, ALLOCATABLE :: Fslgw(:,:,:), Fslsw(:,:,:), DryET2RO(:,:,:)
    REAL, ALLOCATABLE :: ADJdp(:,:,:), ADJro(:,:,:), ROmax(:,:,:), ROmin(:,:,:)
    REAL, ALLOCATABLE :: DPLL(:,:,:), DPcap(:,:,:), ROfDP(:,:,:)
    REAL, ALLOCATABLE :: AEgwb(:,:), AEgyr(:,:), AEswb(:,:), AEsyr(:,:)
    REAL, ALLOCATABLE :: AEgw(:), AEsw(:)
    REAL, ALLOCATABLE :: GWC(:), LatLoss(:)
    
    REAL, ALLOCATABLE :: DryAc(:), GWac(:), SWac(:), COac(:)
    REAL, ALLOCATABLE :: CSdryDP(:,:,:), CSdryET(:,:,:), CSdryRO(:,:,:)
    REAL, ALLOCATABLE :: CSirrDP(:,:,:), CSirrET(:,:,:), CSirrRO(:,:,:), CSNIR(:,:,:)
    
    REAL, ALLOCATABLE :: SWD(:,:), COP(:,:), COD(:,:)
    REAL, ALLOCATABLE :: GWP(:,:,:)
    
    DATA Mon /'Ann', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'/
    
END MODULE PARAM
!****************************************************************************
PROGRAM WSPP
    USE PARAM
    IMPLICIT NONE
    
    INTEGER     c, i, j, k, s, z
    INTEGER     zone, yr, irrdist, crp, icrop, run1
    INTEGER     ngwt, nswt, ncot
    
    REAL    SWDc(0 : 12), GWPc(0 : 12), COPc(0 : 12), CODc(0 : 12)
    REAL    totacs

    REAL    wAP, wSW, wET, wRO, WDP, WSL, wETgain, wETb, wDET1, wDET2
    REAL    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wEwat, wNIRmET, wLatseep
    
    OPEN (1, FILE = TRIM(MasterDIR)//TRIM(MasterFile), STATUS = 'OLD')
    READ (1, *)

    READ (1, *) cRun
    READ (1, *) MODname
    READ (1, *) INDIR
    READ (1, *) OUTDIR
    READ (1, *) WBPDIR
    READ (1, *) LUDIR
    READ (1, *) CNLDIR
    READ (1, *) MiscDIR
    READ (1, *) MIDIR
    READ (1, *) GWLUfld
    READ (1, *) SWLUfld
    READ (1, *) COLUfld
    READ (1, *) DryLUfld
    READ (1, *) GWLU
    READ (1, *) SWLU
    READ (1, *) COLU
    READ (1, *) DryLU
    READ (1, *) GWCfld
    READ (1, *) GWCfile
    READ (1, *) CellLoc
    READ (1, *) Coeffile
    READ (1, *) AEfile
    READ (1, *) ROZfile
    READ (1, *) PctRchfile
    READ (1, *) SWidxfile
    READ (1, *) CountID
    READ (1, *) Cell_county
    READ (1, *) CNLmastfile
    READ (1, *) MISCrmastfile
    READ (1, *) MISCpmastfile
    READ (1, *) MIrmastfile
    READ (1, *) MIpmastfile
    READ (1, *) ActCellfile
    !Added 1/7/2016
    READ (1, *) CallYrfile
    READ (1, *) ncells
    READ (1, *) startyr
    READ (1, *) endyr
    READ (1, *) RCH0flg
    READ (1, *) DPonly
    READ (1, *) SWonly
    READ (1, *) cnlrflg
    READ (1, *) miscrflg
    READ (1, *) miscpflg
    READ (1, *) MIrflg
    READ (1, *) MIpflg
    !Added 1/7/2016
    READ (1, *) ctrlyrs
    READ (1, *) nczones
    READ (1, *) nrzones
    READ (1, *) nsoils
    READ (1, *) nirrdist
    READ (1, *) csize
    READ (1, *) ncols
    READ (1, *) nrows
    READ (1, *) ncrops
    
    ALLOCATE (Crops(ncrops))
    
    DO i = 1, ncrops
        READ (1, *) crops(i)
    END DO
    CLOSE (1)
    
    ALLOCATE (Soil(ncells), CoefZ(ncells), ROZ(ncells), M2g(ncells))
    ALLOCATE (ADJetdry(nczones, ncrops, nsoils), ADJetirr(nczones, ncrops, nsoils), ADJnir(nczones, ncrops, nsoils))
    ALLOCATE (Fslgw(nczones, ncrops, nsoils), Fslsw(nczones, ncrops, nsoils), DryET2RO(nczones, ncrops, nsoils))
    ALLOCATE (ADJdp(nczones, ncrops, nsoils), ADJro(nczones, ncrops, nsoils), ROmax(nczones, ncrops, nsoils), ROmin(nczones, ncrops, nsoils))
    ALLOCATE (DPLL(nczones, ncrops, nsoils), DPcap(nczones, ncrops, nsoils), ROfDP(nczones, ncrops, nsoils))
    ALLOCATE (AEgwb(nczones, 2), AEgyr(nczones, 2), AEswb(nczones, 2), AEsyr(nczones, 2))
    ALLOCATE (AEgw(nczones), AEsw(nczones))
    ALLOCATE (GWC(nirrdist), LatLoss(nirrdist))
    ALLOCATE (DryAc(ncrops), GWac(ncrops), SWac(ncrops), COac(ncrops))
    ALLOCATE (CSdryDP(ncells, ncrops, 0 : 12), CSdryET(ncells, ncrops, 0 : 12), CSdryRO(ncells, ncrops, 0 : 12))
    ALLOCATE (CSirrDP(ncells, ncrops, 0 : 12), CSirrET(ncells, ncrops, 0 : 12), CSirrRO(ncells, ncrops, 0 : 12))
    ALLOCATE (CSNIR(ncells, ncrops, 0 : 12))
    
    ALLOCATE (ntsw(startyr : endyr), ntco(startyr : endyr))
    ALLOCATE (SWDt(ncells, nirrdist, ncrops - 2), COPt(ncells, nirrdist, ncrops - 2), CODt(ncells, nirrdist, ncrops - 2))
    ALLOCATE (GWP(ncells, ncrops - 2, 12))
    
    !Added 1/7/2016
    ALLOCATE (yrctrl(ctrlyrs))
    ALLOCATE (LUyear(startyr : endyr), WBPyear(startyr : endyr), GWCyear(startyr : endyr))
    
    OUTDIR = TRIM(OUTDIR)//TRIM(cRun)//'\Results\'
    Dummy = 0.
    
    !sw irrigation transactions
    OPEN (1, FILE = TRIM(OUTDIR)//'AppIrr\Irrtrans.txt', STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF (1))
        READ (1, *) yr, ngwt, nswt, ncot
        IF ((yr .GE. startyr) .AND. (yr .LE. endyr)) THEN
            ntsw(yr) = nswt
            ntco(yr) = ncot
        END IF
    END DO
    CLOSE (1)
    
    !Populate cell soils, zones, and location
    WRITE (6, *) 'Reading Cell Location'
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CellLoc), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, Soil(cell), CoefZ(cell), ROZ(cell), M2G(cell)
    END DO
    CLOSE (1)  
    
    !Load coefficient zone coefficients
    !z - zone
    !c - crop
    !s - soil
    WRITE (6, *) 'Reading Coefficient Zone Parameters'
    OPEN (1, FILE = TRIM(INDIR)//TRIM(Coeffile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) z, c, s, ADJnir(z, c, s), ADJetdry(z, c, s), ADJetirr(z, c, s), &
                        Fslgw(z, c, s), Fslsw(z, c, s), DryET2RO(z, c, s), &
                        ADJdp(z, c, s), ADJro(z, c, s), ROmax(z, c, s), ROmin(z, c, s), &
                        DPLL(z, c, s), DPcap(z, c, s), ROfDP(z, c, s)
    END DO
    CLOSE (1)    
    
    !Load application efficiencies
    WRITE (6, *) 'Loading Application Efficiencies'
    OPEN (1, FILE = TRIM(INDIR)//TRIM(AEfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) zone, AEgwb(zone, :), AEgyr(zone, :), AEswb(zone, :), AEsyr(zone, :)
    END DO
    CLOSE (1)
    
    sidx = 0
    !Read SWIndex
    OPEN (1, FILE = TRIM(INDIR)//TRIM(SWidxfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) idx, swdist
        sidx(swdist) = idx    
    END DO
    CLOSE (1)    
    
    !Read in Call year
    !Added 1/7/2016 to control the annual input data and allow the same data to be applied to different years
    !ex: applying a fixed land use to the historical climate
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CallYrFile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) yr, yrctrl
        IF ((yr .GE. startyr) .AND. (yr .LE. endyr)) THEN
            LUyear(yr) = yrctrl(1)
            WBPyear(yr) = yrctrl(2)
            GWCyear(yr) = yrctrl(3)
        END IF
    END DO
    CLOSE (1)    
    
    run1 = 1
    
    WRITE (6, 600) 
    DO iyear = startyr, endyr
        !Altered 1/7/2016
        WRITE (6, 601) iyear, LUyear(iyear), WBPyear(iyear), GWCyear(iyear)
        WRITE (YRT, '(I4)') iyear
        
        !Added 1/7/2016
        WRITE (YRTlu, '(I4)') LUyear(iyear)
        WRITE (YRTwbp, '(I4)') WBPyear(iyear)
        WRITE (YRTgwc, '(I4)') GWCyear(iyear)
        
        IF (run1 .EQ. 1) THEN
            Run1 = 0
        ELSE
            DEALLOCATE (SWD, COP, COD)
        END IF
        
        ALLOCATE (SWD(ntsw(iyear), 12), COP(ntco(iyear), 12), COD(ntco(iyear), 12))
        
        CALL GetAE
        
        GWC = 0.
        LatLoss = 0.
        !Load ground water concentration and lateral loss parameters
        !Updated YRTgwc 1/7/2016
        OPEN (1, FILE = TRIM(INDIR)//TRIM(GWCfld)//'\'//TRIM(GWCfile)//Trim(YRTgwc)//'.txt', STATUS = 'OLD')
        READ (1, *) 
        DO WHILE (.NOT. EOF(1))
            READ (1, *) yr, irrdist, GWC(sidx(irrdist)), LatLoss(sidx(irrdist))
        END DO
        CLOSE (1)
        
        CALL CSFiles
        
        OPEN (10, FILE = TRIM(OUTDIR)//'RAW\RAW_WSPP'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        WRITE (10, 100)
        
        OPEN (11, FILE = TRIM(OUTDIR)//'WSPP_OUT\WSPPDP'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        WRITE (11, 110) (Mon(i), i = 0, 12)
        
        OPEN (12, FILE = TRIM(OUTDIR)//'WSPP_OUT\WSPPRO'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        WRITE (12, 120) (Mon(i), i = 0, 12)
        
        OPEN (13, FILE = TRIM(OUTDIR)//'Pump\Pump'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        WRITE (13, 130) (Mon(i), i = 0, 12)
        
        OPEN (14, FILE = TRIM(OUTDIR)//'LatLoss\LatLoss'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        WRITE (14, 140) (Mon(i), i = 0, 12)
        
        SWD = 0.
        GWP = 0.
        COD = 0.
        COP = 0.
        
        IF (DPonly .EQ. 0) THEN
            
            !Get Surface Water Deliveries
            OPEN (1, FILE = TRIM(OUTDIR)//'AppIrr/SWD/SWD'//TRIM(YRT)//'.txt', STATUS = 'OLD')
            READ (1, *)
            
            DO WHILE (.NOT. EOF(1))
                READ (1, *) idx, cell, yr, swdist, swidx, crp, SWDc
                SWDt(cell, swidx, crp) = idx
                SWD(idx, :) = SWDc(1 : 12)
            END DO
            CLOSE (1)
            
            !Get Ground water pumping
            OPEN (1, FILE = TRIM(OUTDIR)//'AppIrr/GWP/GWP'//TRIM(YRT)//'.txt', STATUS = 'OLD')
            READ (1, *)
            
            DO WHILE (.NOT. EOF(1))
                READ (1, *) idx, cell, yr, crp, GWPc
                GWP(cell, crp, :) = GWPc(1 : 12)
            END DO
            CLOSE (1)
            
            !Get Co-mingled surface water deliveries
            OPEN (1, FILE = TRIM(OUTDIR)//'AppIrr/COD/COD'//TRIM(YRT)//'.txt', STATUS = 'OLD')
            READ (1, *)
            
            DO WHILE (.NOT. EOF(1))
                READ (1, *) idx, cell, yr, swdist, swidx, crp, CODc
                CODt(cell, swidx, crp) = idx
                COD(idx, :) = CODc(1 : 12)
            END DO
            CLOSE (1)
            
            !Get Co-mingled ground water pumping
            OPEN (1, FILE = TRIM(OUTDIR)//'AppIrr/COP/COP'//TRIM(YRT)//'.txt', STATUS = 'OLD')
            READ (1, *)
            
            DO WHILE (.NOT. EOF(1))
                READ (1, *) idx, cell, yr, swdist, swidx, crp, COPc
                COPt(cell, swidx, crp) = idx
                COP(idx, :) = COPc(1 : 12)
            END DO
            CLOSE (1)
        END IF
        
        IF (SWonly .EQ. 1) THEN
            COP = 0.
            GWP = 0.
        END IF
        
        
        
        !Dryland Loop
        swdist = 0
        swidx = 0
        Pflag = 1
        !Updated YRTlu 1/7/2016
        OPEN (1, FILE = TRIM(LUDIR)//TRIM(DryLUfld)//'\'//TRIM(DryLU)//TRIM(YRTlu)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, yr, totacs, DryAc
            
            cellDP = 0.
            cellRO = 0.
            cellAcs = 0.
            
            DO j = 1, ncrops
                IF (DryAc(j) .GT. 0.) THEN
                    IF (j .EQ. ncrops) THEN
                        icrop = 6
                    ELSE
                        icrop = j
                    END IF
                
                    CALL CalcDryCrop(CoefZ(cell), icrop, Soil(cell))
                                       
                    DO k = 0, 12
                    
                        wAP = 0.
                        wSW = 0.
                        
                        wET = (ETMaxadjdry(k) + ETtrans(k)) / 12 * DryAc(j)
                        wRO = (RO2dry(k) + RO1dry(k) + DP2RO(k)) / 12 * DryAc(j)
                        wDP = DPdrytot(k) / 12 * DryAc(j)
                        wSL = 0.
                        
                        wETgain = 0.
                        wETb = ETMaxadjdry(k) / 12 * DryAc(j)
                        wDET1 = 0.
                        wDET2 = (ETMaxdry(k) - ETMaxadjdry(k)) / 12 * DryAc(j)
                        
                        wDP1 = DP1dry(k) / 12 * DryAc(j)
                        wDP2 = DP2dry(k) / 12 * DryAc(j)
                        wRO1 = RO1dry(k) / 12 * DryAc(j)
                        wRO2 = RO2dry(k) / 12 * DryAc(j)
                        wDP2RO = DP2RO(k) / 12 * DryAc(j)
                        
                        wETtrans = ETtrans(k) / 12 * DryAc(j)
                        wEwat = (ETMaxdry(k) - ETMaxadjdry(k)) / 12 * DryAc(j)
                        wNIRmET = NIRmET(k) / 12 * DryAc(j)
                        wLatseep = 0.
                        
                        
                        WRITE (10, 101) Cell, iyear, k, j, swdist, swidx, Pflag, Soil(cell), CoefZ(cell), ROZ(cell), Dryac(j), &
                                        wAP, wSW, wET, wRO, wDP, wSL, wETgain, wETb, wDET1, wDET2, wDP1, wDP2, &
                                        wRO1, wRO2, wDP2RO, wEwat, wETtrans, wNIRmET, wLatseep
                        
                    END DO
                    
                    cellacs = cellacs + Dryac(j)
                    cellDP = cellDP + DPdrytot / 12 * DryAc(j)
                    cellRO = cellRO + (RO1dry + RO2dry + DP2RO) / 12 * DryAc(j)
                    
                END IF 
            END DO
            
            cellDP(0) = SUM(cellDP(1 : 12))
            cellRO(0) = SUM(cellRO(1 : 12))
            
            WRITE (11, 111) cell, iyear, pflag, swdist, cellacs, cellDP
            WRITE (12, 111) cell, iyear, pflag, swdist, cellacs, cellRO
            
        END DO
        CLOSE (1)
        
        
        !Ground water only Loop
        swdist = 0
        swidx = 0
        Pflag = 2
        !Updated YRTlu 1/7/2016
        OPEN (1, FILE = TRIM(LUDIR)//TRIM(GWLUfld)//'\'//TRIM(GWLU)//TRIM(YRTlu)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF (1)) 
            READ (1, *) cell, yr, totacs, GWac
            
            cellDP = 0.
            cellRO = 0.
            cellPump = 0.
            cellacs = 0.
            
            DO j = 1, ncrops - 2
                IF (GWac(j) .GT. 0.) THEN
                    
                    Pump(1 : 12) = GWP(cell, j, :) * 12 / GWac(j)
                    Pump(0) = SUM(Pump(1 : 12))
                    
                    CALL CalcIrrCrop (CoefZ(cell), j, soil(cell), Dummy, Pump)
                    
                    DO k = 0, 12
                        wAP = Pump(k) / 12 * GWAc(j)
                        wSW = 0.
                        
                        wET = ET(k) / 12 * GWAc(j)
                        wRO = (RO1irr(k) + RO2(k) + DP2RO(k)) / 12 * GWAc(j)
                        wDP = DPirrtot(k) / 12 * GWAc(j)
                        wSL = SL(k) / 12 * GWAc(j)
                        
                        wETgain = ETgain(k) / 12 * GWAc(j)
                        wETb = ETbase(k) / 12 * GWAc(j)
                        wDET1 = DET1(k) / 12 * GWAc(j)
                        wDET2 = DET2(k) / 12 * GWAc(j)
                        
                        wDP1 = DP1irr(k) / 12 * GWAc(j)
                        wDP2 = DP2(k) / 12 * GWAc(j)
                        wRO1 = RO1irr(k) / 12 * GWAc(j)
                        wRO2 = RO2(k) / 12 * GWAc(j)
                        wDP2RO = DP2RO(k) / 12 * GWAc(j)
                        
                        wETtrans = ETtrans(k) / 12 * GWAc(j)
                        wEwat = Ewat(k) / 12 * GWAc(j)
                        wNIRmET = NIRmET(k) / 12 * GWAc(j)
                        wLatseep = 0.
                        
                        
                        WRITE (10, 101) Cell, iyear, k, j, swdist, swidx, Pflag, Soil(cell), CoefZ(cell), ROZ(cell), GWac(j), &
                                        wAP, wSW, wET, wRO, wDP, wSL, wETgain, wETb, wDET1, wDET2, wDP1, wDP2, &
                                        wRO1, wRO2, wDP2RO, wEwat, wETtrans, wNIRmET, wLatseep
                                
                    END DO
                    
                    cellDP = cellDP + DPirrtot / 12 * GWAc(j)
                    cellRO = cellRO + (RO1irr + RO2 + DP2RO) / 12 * GWAc(j)
                    cellPump = cellPump + Pump / 12 * GWAc(j)
                    cellacs = cellacs + GWac(j)

                END IF
            END DO
            
            cellDP(0) = SUM(cellDP(1 : 12))
            cellRO(0) = SUM(cellRO(1 : 12))
            cellPump(0) = SUM(cellPump(1 : 12))
            
            WRITE (11, 111) cell, iyear, pflag, swdist, cellacs, cellDP
            WRITE (12, 111) cell, iyear, pflag, swdist, cellacs, cellRO
            WRITE (13, 111) cell, iyear, pflag, swdist, cellacs, cellPump
            
        END DO
        CLOSE (1)
        
        
        !Surface water only Loop
        Pflag = 3
        !Updated YRTlu 1/7/2016
        OPEN (1, FILE = TRIM(LUDIR)//TRIM(SWLUfld)//'\'//TRIM(SWLU)//TRIM(YRTlu)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) Cell, yr, swdist, swidx, totacs, SWac
            
            cellDP = 0.
            cellRO = 0.
            cellLatLoss = 0.
            cellAcs = 0.
            
            DO j = 1, ncrops - 2
                IF (SWac(j) .GT. 0.) THEN
                    
                    Latseep(1 : 12) = SWD(SWDt(cell, swidx, j), :) * LatLoss(swidx) / (1 + LatLoss(swidx))
                    Latseep(0) = SUM(Latseep(1 : 12))
                    
                    Sdel(1 : 12) = SWD(SWDt(cell, swidx, j), :) / (1 + LatLoss(swidx)) * 12 / SWac(j)
                    Sdel(0) = SUM(Sdel(1 : 12))
                    
                    CALL CalcIrrCrop (CoefZ(cell), j, soil(cell), Sdel, Dummy)
                    
!                    WRITE (11, 111) cell, iyear, pflag, swdist, DPirrtot
!                    WRITE (12, 111) cell, iyear, pflag, swdist, (RO1irr(k) + RO2(k) + DP2RO(k), k = 0, 12)
                    
                    DO k = 0, 12
                        wAP = 0.
                        wSW = Sdel(k) / 12 * SWAc(j)
                        
                        wET = ET(k) / 12 * SWAc(j)
                        wRO = (RO1irr(k) + RO2(k) + DP2RO(k)) / 12 * SWAc(j)
                        wDP = DPirrtot(k) / 12 * SWAc(j)
                        wSL = SL(k) / 12 * SWAc(j)
                        
                        wETgain = ETgain(k) / 12 * SWAc(j)
                        wETb = ETbase(k) / 12 * SWAc(j)
                        wDET1 = DET1(k) / 12 * SWAc(j)
                        wDET2 = DET2(k) / 12 * SWAc(j)
                        
                        wDP1 = DP1irr(k) / 12 * SWAc(j)
                        wDP2 = DP2(k) / 12 * SWAc(j)
                        wRO1 = RO1irr(k) / 12 * SWAc(j)
                        wRO2 = RO2(k) / 12 * SWAc(j)
                        wDP2RO = DP2RO(k) / 12 * SWAc(j)
                        
                        wETtrans = ETtrans(k) / 12 * SWAc(j)
                        wEwat = Ewat(k) / 12 * SWAc(j)
                        wNIRmET = NIRmET(k) / 12 * SWAc(j)
                        wLatseep = Latseep(k)
                        
                        WRITE (10, 101) Cell, iyear, k, j, swdist, swidx, Pflag, Soil(cell), CoefZ(cell), ROZ(cell), SWac(j), &
                                        wAP, wSW, wET, wRO, wDP, wSL, wETgain, wETb, wDET1, wDET2, wDP1, wDP2, &
                                        wRO1, wRO2, wDP2RO, wEwat, wETtrans, wNIRmET, wLatseep
                        
                    END DO
                    
                    cellDP = cellDP + DPirrtot / 12 * SWAc(j)
                    cellRO = cellRO + (RO1irr + RO2 + DP2RO) / 12 * SWAc(j)
                    cellLatLoss = cellLatLoss + Latseep
                    cellAcs = cellAcs + SWac(j)
                    
                END IF
            END DO
            
            cellDP(0) = SUM(cellDP(1 : 12))
            cellRO(0) = SUM(cellRO(1 : 12))
            cellLatLoss(0) = SUM(cellLatLoss(1 : 12))
            
            WRITE (11, 111) cell, iyear, pflag, swdist, cellacs, cellDP
            WRITE (12, 111) cell, iyear, pflag, swdist, cellacs, cellRO
            WRITE (14, 111) cell, iyear, pflag, swdist, cellacs, cellLatLoss
            
        END DO
        CLOSE (1)
        
        
        !Co-mingled lands loop
        Pflag = 4
        !Updated YRTlu 1/7/2016
        OPEN (1, FILE = TRIM(LUDIR)//TRIM(COLUfld)//'\'//TRIM(COLU)//TRIM(YRTlu)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) Cell, yr, swdist, swidx, totacs, COac
            
            !IF (cell .EQ. 72297) THEN
            !    CONTINUE
            !END IF  
            
            cellDP = 0.
            cellRO = 0.
            cellPump = 0.
            cellLatLoss = 0.
            cellAcs = 0.
            
            DO j = 1, ncrops - 2
                IF (COac(j) .GT. 0.) THEN
                    
                    Latseep(1 : 12) = COD(CODt(cell, swidx, j), :) * LatLoss(swidx) / (1 + LatLoss(swidx))
                    Latseep(0) = SUM(Latseep(1 : 12))
                    
                    Sdel(1 : 12) = COD(CODt(cell, swidx, j), :) / (1 + LatLoss(swidx)) * 12 / COac(j)
                    Pump(1 : 12) = COP(COPt(cell, swidx, j), :) * 12 / COac(j)
                    
                    Sdel(0) = SUM(Sdel(1 : 12))
                    Pump(0) = SUM(Pump(1 : 12))
                    
                    CALL CalcIrrCrop (CoefZ(cell), j, soil(cell), Sdel, Pump)
                    
                    DO k = 0, 12
                        wAP = Pump(k) / 12 * COAc(j)
                        wSW = Sdel(k) / 12 * COAc(j)
                        
                        wET = ET(k) / 12 * COAc(j)
                        wRO = (RO1irr(k) + RO2(k) + DP2RO(k)) / 12 * COAc(j)
                        wDP = DPirrtot(k) / 12 * COAc(j)
                        wSL = SL(k) / 12 * COAc(j)
                        
                        wETgain = ETgain(k) / 12 * COAc(j)
                        wETb = ETbase(k) / 12 * COAc(j)
                        wDET1 = DET1(k) / 12 * COAc(j)
                        wDET2 = DET2(k) / 12 * COAc(j)
                        
                        wDP1 = DP1irr(k) / 12 * COAc(j)
                        wDP2 = DP2(k) / 12 * COAc(j)
                        wRO1 = RO1irr(k) / 12 * COAc(j)
                        wRO2 = RO2(k) / 12 * COAc(j)
                        wDP2RO = DP2RO(k) / 12 * COAc(j)
                        
                        wETtrans = ETtrans(k) / 12 * COAc(j)
                        wEwat = Ewat(k) / 12 * COAc(j)
                        wNIRmET = NIRmET(k) / 12 * COAc(j)
                        wLatseep = Latseep(k) 
                        
                        
                        WRITE (10, 101) Cell, iyear, k, j, swdist, swidx, Pflag, Soil(cell), CoefZ(cell), ROZ(cell), COac(j), &
                                        wAP, wSW, wET, wRO, wDP, wSL, wETgain, wETb, wDET1, wDET2, wDP1, wDP2, &
                                        wRO1, wRO2, wDP2RO, wEwat, wETtrans, wNIRmET, wLatseep
                        
                    END DO
                    
                    cellDP = cellDP + DPirrtot / 12 * COAc(j)
                    cellRO = cellRO + (RO1irr + RO2 + DP2RO) / 12 * COAc(j)
                    cellPump = cellPump + Pump / 12 * COAc(j)
                    cellLatLoss = cellLatLoss + Latseep
                    cellAcs = cellAcs + COac(j)
                    
                END IF
            END DO
            
            cellDP(0) = SUM(cellDP(1 : 12))
            cellRO(0) = SUM(cellRO(1 : 12))
            cellPump(0) = SUM(cellPump(1 : 12))
            cellLatLoss(0) = SUM(cellLatLoss(1 : 12))
            
            WRITE (11, 111) cell, iyear, pflag, swdist, cellacs, cellDP
            WRITE (12, 111) cell, iyear, pflag, swdist, cellacs, cellRO
            WRITE (13, 111) cell, iyear, pflag, swdist, cellacs, cellPump
            WRITE (14, 111) cell, iyear, pflag, swdist, cellacs, cellLatLoss
            
        END DO
        CLOSE (1)
        
        CLOSE (10)
        CLOSE (11)
        CLOSE (12)
        CLOSE (13)
        CLOSE (14)
        
    END DO
    
100 FORMAT( 'Cell, Year, Month, Crop, SW_District, SW_index, Irr_Source, Soil, Coef_Zone, RO_zone, Acres, ' &
            'Pumping, SW_Delivery, ET, RO_tot, DP_tot, Surface_Loss, ET_gain, ET_base, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, Ewat, ETtrans, NIRmET, Lateral_Loss')
    
101 FORMAT(I7, ',', I5, 8(',', I3), 20(',', F14.6))
    
110 FORMAT('Cell, Year, Irr_Source, SWdist, Acres' 13(', DP_', A3))
120 FORMAT('Cell, Year, Irr_Source, SWdist, Acres' 13(', RO_', A3))
130 FORMAT('Cell, Year, Irr_Source, SWdist, Acres' 13(', Pump_', A3))
140 FORMAT('Cell, Year, Irr_Source, SWdist, Acres' 13(', LatLoss_', A3))
111 FORMAT(I7, ',', I5, 2(',', I3), 14(',', F14.6))   

!Added 1/7/2016
600 FORMAT (' Sim Year,  LU Year, WBP Year, GWC Year')
601 FORMAT (I9, 3(',', I9))
    
END PROGRAM WSPP
!****************************************************************************
SUBROUTINE CSFiles
    USE PARAM
    IMPLICIT NONE
    
    !Variables
    INTEGER     nn, crp, yr, i
    
    CSdryDP = 0.
    CSdryRO = 0.
    CSdryET = 0.
    CSirrDP = 0.
    CSirrRO = 0.
    CSirrET = 0.
    CSNIR = 0.
    
    DO nn = 1, ncrops - 1
        
        !Updated YRTwbp 1/7/2016
        OPEN (100, FILE = TRIM(WBPDIR)//'DP\'//TRIM(Crops(nn))//'-DryDP'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
        READ (100, *)
        
        DO WHILE (.NOT. EOF(100))
            READ (100, *) cell, yr, crp, (CSdryDP(cell, nn, i), i = 0, 12)
        END DO
        CLOSE (100)
        
        !Updated YRTwbp 1/7/2016
        OPEN (100, FILE = TRIM(WBPDIR)//'RO\'//TRIM(Crops(nn))//'-DryRO'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
        READ (100, *)
        
        DO WHILE (.NOT. EOF(100))
            READ (100, *) cell, yr, crp, (CSdryRO(cell, nn, i), i = 0, 12)
        END DO
        CLOSE (100)
        
        !Updated YRTwbp 1/7/2016
        OPEN (100, FILE = TRIM(WBPDIR)//'ET\'//TRIM(Crops(nn))//'-DryET'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
        READ (100, *)
        
        DO WHILE (.NOT. EOF(100))
            READ (100, *) cell, yr, crp, (CSdryET(cell, nn, i), i = 0, 12)
        END DO
        CLOSE (100)
        
        IF (nn .NE. ncrops - 1) THEN
            
            !Updated YRTwbp 1/7/2016
            OPEN (100, FILE = TRIM(WBPDIR)//'DP\'//TRIM(Crops(nn))//'-IrrDP'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
            READ (100, *)
        
            DO WHILE (.NOT. EOF(100))
                READ (100, *) cell, yr, crp, (CSirrDP(cell, nn, i), i = 0, 12)
            END DO
            CLOSE (100)
            
            !Updated YRTwbp 1/7/2016
            OPEN (100, FILE = TRIM(WBPDIR)//'RO\'//TRIM(Crops(nn))//'-IrrRO'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
            READ (100, *)
        
            DO WHILE (.NOT. EOF(100))
                READ (100, *) cell, yr, crp, (CSirrRO(cell, nn, i), i = 0, 12)
            END DO
            CLOSE (100)
            
            !Updated YRTwbp 1/7/2016
            OPEN (100, FILE = TRIM(WBPDIR)//'ET\'//TRIM(Crops(nn))//'-IrrET'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
            READ (100, *)
        
            DO WHILE (.NOT. EOF(100))
                READ (100, *) cell, yr, crp, (CSirrET(cell, nn, i), i = 0, 12)
            END DO
            CLOSE (100)
            
            !Updated YRTwbp 1/7/2016
            OPEN (100, FILE = TRIM(WBPDIR)//'NIR\'//TRIM(Crops(nn))//'-NIR'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
            READ (100, *)
        
            DO WHILE (.NOT. EOF(100))
                READ (100, *) cell, yr, crp, (CSNIR(cell, nn, i), i = 0, 12)
            END DO
            CLOSE (100)
        END IF
    END DO
    
END SUBROUTINE CSFILES
!****************************************************************************
SUBROUTINE GetAE
    USE PARAM

    !Variables
    INTEGER izone
    
    DO izone = 1, nczones
        IF (iyear .LE. AEgyr(izone, 1)) THEN
            AEgw(izone) = AEgwb(izone, 1)
        ELSE IF (iyear .GE. AEgyr(izone, 2)) THEN
            AEgw(izone) = AEgwb(izone, 2)
        ELSE
            AEgw(izone) = AEgwb(izone, 1) + (AEgwb(izone, 2) - AEgwb(izone, 1)) * ((iyear - AEgyr(izone, 1)) / (AEgyr(izone, 2) - AEgyr(izone, 1)))
        END IF
    
        IF (iyear .LE. AEsyr(izone, 1)) THEN
            AEsw(izone) = AEswb(izone, 1)
        ELSE IF (iyear .GE. AEsyr(izone, 2)) THEN
            AEsw(izone) = AEswb(izone, 2)
        ELSE
            AEsw(izone) = AEswb(izone, 1) + (AEswb(izone, 2) - AEswb(izone, 1)) * ((iyear - AEsyr(izone, 1)) / (AEsyr(izone, 2) - AEsyr(izone, 1)))
        END IF
    END DO
    
END SUBROUTINE GetAE
!****************************************************************************
SUBROUTINE CalcDryCrop(izone, CropCS, isoil)

    USE PARAM
    IMPLICIT NONE
    
    INTEGER     CropCS, n, izone, isoil
    
    REAL    DPUL, alpha, DPsum
    
    ETMAXdry = CSdryET(cell, CropCS, :)
    ETMAXadjdry = ETMAXdry * ADJetdry(izone, CropCS, isoil)
    
    NIRmET = 0. - ETMAXdry
    
    RO1dry = CSdryRO(cell, CropCS, :) * ADJro(izone, CropCS, isoil)
    RO2dry = (ETMAXdry - ETMAXadjdry) * DryET2RO(izone, CropCS, isoil)
    
    DP1dry = CSdryDP(cell, CropCS, :) * ADJdp(izone, CropCS, isoil)
    DP2dry = (ETMAXdry - ETMAXadjdry) * (1 - DryET2RO(izone, CropCS, isoil))
    
    ETTrans = CSdryDP(cell, CropCS, :) * (1 - ADJdp(izone, CropCS, isoil)) + CSdryRO(cell, CropCS, :) * (1 - ADJro(izone, CropCS, isoil))
    
    ETMAXdry(0) = SUM(ETMAXdry(1 : 12))
    ETMAXadjdry(0) = SUM(ETMAXadjdry(1 : 12))
    RO1dry(0) = SUM(RO1dry(1 : 12))
    RO2dry(0) = SUM(RO2dry(1 : 12))
    DP1dry(0) = SUM(DP1dry(1 : 12))
    DP2dry(0) = SUM(DP2dry(1 : 12))
    ETtrans(0) = SUM(ETtrans(1 : 12))
    NIRmET(0) = SUM(NIRmET(1 : 12))
    
    DPUL = DPLL(izone, CropCS, isoil) + 100
    alpha = (DPcap(izone, CropCS, isoil) - DPLL(izone, CropCS, isoil)) / (DPUL - DPLL(izone, CropCS, isoil))

    IF (DP1dry(0) + DP2dry(0) .GT. DPLL(izone, CropCS, isoil)) THEN
        IF (DP1dry(0) + DP2dry(0) .GT. DPUL) THEN
            DPdrytot(0) = DPcap(izone, CropCS, isoil)
        ELSE
            DPdrytot(0) = DPLL(izone, CropCS, isoil) + (DPcap(izone, CropCS, isoil) - DPLL(izone, CropCS, isoil)) * &
                        (1 - (1 - (DP1dry(0) + DP2dry(0) - DPLL(izone, CropCS, isoil)) / (DPUL - DPLL(izone, CropCS, isoil)))**(1 / alpha))
        END IF
        
        DPsum = DP1dry(0) + DP2dry(0)
        DO n = 1, 12
            DPdrytot(n) = DPdrytot(0) * (DP1dry(n) + DP2dry(n)) / DPsum
            DP2RO(n) = DP1dry(n) + DP2dry(n) - DPdrytot(n)
        END DO
        DP2RO(0) = SUM(DP2RO(1 : 12))
        DPdrytot(0) = SUM(DPdrytot(1 : 12))
    ELSE
        DPdrytot = DP1dry + DP2dry
        DPdrytot(0) = SUM(DPdrytot(1 : 12))
        DP2RO = 0.
    END IF
    
END SUBROUTINE CalcDryCrop
!****************************************************************************
SUBROUTINE CalcIrrCrop (izone, CropCS, isoil, AppSW, AppGW)

    USE PARAM
    IMPLICIT NONE
    
    INTEGER     n, izone, cropcs, isoil
    
    REAL    AppGW(0 : 12), AppSW(0 : 12), AppWat(0 : 12)
    REAL    RODPwt(12)
    REAL    SEAetirr, SEAetdry, SEAappWat
    REAL    AEcomp
    
    REAL    ETMax(0 : 12), NIR(0 : 12), ETdry(0 : 12)
    REAL    ETgain1(0 : 12), ETbase1(0 : 12), ET1(0 : 12)
    REAL    CIR1, GIR1, Beta1
    REAL    DPUL, alpha, DPsum
    
    SEAetirr = 0.
    SEAetdry = 0.
    SEAappWat = 0.
    
    AppWat = AppGW + AppSW
    
    RO1irr = CSirrRO(cell, CropCS, :) * ADJro(izone, CropCS, isoil)
    DP1irr = CSirrDP(cell, CropCS, :) * ADJdp(izone, CropCS, isoil)
    ETtrans = CSirrRO(cell, CropCS, :) * (1 - ADJro(izone, CropCS, isoil)) + CSirrDP(cell, CropCS, :) * (1 - ADJdp(izone, CropCS, isoil))
    
    ETMax = CSirrET(cell, CropCS, :)
    ETdry = CSdryET(cell, CropCS, :)
    NIR = CSNIR(cell, CropCS, :)
    
    AppWat(0) = SUM(AppWat(1 : 12))
    RO1irr(0) = SUM(RO1irr(1 : 12))
    DP1irr(0) = SUM(DP1irr(1 : 12))
    ETMax(0) = SUM(ETMax(1 : 12))
    ETdry(0) = SUM(ETdry(1 : 12))
    NIR(0) = SUM(NIR(1 : 12))
    ETtrans(0) = SUM(ETtrans(1 : 12))
    
    NIRmET = NIR - ETMAX
    
    NIRmET(0) = SUM(NIRmET(1 : 12))
    
    IF (AppSW(0) + AppGW(0) .GT. 0.) THEN
        AEcomp = AEgw(izone) * AppGW(0) / (AppGW(0) + AppSW(0)) + AEsw(izone) * AppSW(0) / (AppGW(0) + AppSw(0))
    ELSE
        AEcomp = AEgw(izone)
    END IF
    
    DO n = 1, 12
        IF (RO1irr(n) + DP1irr(n) .LE. 0.) THEN
            RODPwt(n) = DryET2RO(izone, CropCS, isoil)
        ELSE
            RODPwt(n) = MIN(MAX(ROfDP(izone, CropCS, isoil) * RO1irr(n) / (ROfDP(izone, CropCS, isoil) * RO1irr(n) + DP1irr(n)), ROmin(izone, CropCS, isoil)), ROmax(izone, CropCS, isoil))
        END IF
        
        IF ((NIR(n) .LE. 0.) .AND. (AppWat(n) .LE. 0.)) THEN
            
            ETgain1(n) = 0.
            RO2(n) = 0.
            DP2(n) = 0.
        
        ELSE IF ((NIR(n) .LE. 0.) .AND. (AppWat(n) .GT. 0.)) THEN
            
            CONTINUE
            
        ELSE
            IF (ETmax(n) .GT. ETdry(n)) THEN
                SEAetirr = SEAetirr + ETMAX(n)
                SEAetdry = SEAetdry + ETdry(n)
                IF (AppWat(n) .LE. NIR(n) * ADJnir(izone, CropCS, isoil) / AEcomp) THEN
                    SEAappWAT = SEAappWat + AppWat(n)
                ELSE
                    SEAappWAT = SEAappWat + NIR(n) * ADJnir(izone, CropCS, isoil) / AEcomp
                END IF
            ELSE
                ETgain1(n) = 0.
            END IF
        END IF  
    END DO
    
    CIR1 = MAX(SEAetirr - SEAetdry, 0.0001)
    GIR1 = NIR(0) / AEcomp
    Beta1 = CIR1 / GIR1

    IF (SEAappWat .LT. GIR1) THEN
        ETGain1(0) = MAX(MIN(CIR1 * (1 - (1 - SEAappWat / GIR1)**(1 / Beta1)), SEAappWat), 0.)    
    ELSE
        ETGain1(0) = MAX(SEAetirr - SEAetdry, 0.)
    END IF
    
    IF (SEAetirr - SEAetdry .GT. 0.001) THEN
!        CALL DistETGain(RODPwt, ETGain1(0),ETMax, ETdry, AppWAt, ETGain(1 : 12), RO2, DP2, ETbase1, ET1)
        CALL DistETGain(ETGain1(0), ETMax, ETdry, AppWAt, ETGain1(1 : 12), ETbase1, ET1)
    ELSE
        ETGain1 = 0.
        ETbase1 = ETMax
        ET1 = ETMax
    END IF
    
    ETGain1(0) = SUM(ETGain1(1 : 12))
    ETbase1(0) = SUM(ETbase1(1 : 12))
    DET1 = ETMax - (ETGain1 + ETbase1)
    
    DET1(0) = SUM(DET1(1 : 12))
    ET1(0) = SUM(ET1(1 : 12))
    
    ET = ET1 * ADJetirr(izone, CropCS, isoil)
    DET2 = ET1 * (1 - ADJetirr(izone, CropCS, isoil))
    
    SL = AppGW * Fslgw(izone, CropCS, isoil) + AppSW * Fslsw(izone, CropCS, isoil)
    
    Ewat = Appwat - SL - ET - NIRmET
    
    DO n = 1, 12
        
        IF (ETGain1(n) .GT. 0.) THEN
            ETGain(n) = ETGain1(n) - ETGain1(n) / ET1(n) * DET2(n)
            ETbase(n) = ETbase1(n) - (1 - ETgain1(n) / ET1(n)) * DET2(n)
        ELSE
            ETbase(n) = ETbase1(n) - DET2(n)
            ETGain(n) = 0.
        END IF
        
        RO2(n) = MAX(EWat(n) * RODPwt(n), - RO1irr(n))
        DP2(n) = MAX(EWat(n) * (1 - RODPwt(n)), - DP1irr(n))
    END DO
    
    ET(0) = SUM(ET(1 : 12))
    DET2(0) = SUM(DET2(1 : 12))
    SL(0) = SUM(SL(1 : 12))
    Ewat(0) = SUM(Ewat(1 : 12))
    ETGain(0) = SUM(ETGain(1 : 12))
    ETbase(0) = SUM(ETbase(1 : 12))
    RO2(0) = SUM(RO2(1 : 12))
    DP2(0) = SUM(DP2(1 : 12))
    
    DPUL = DPLL(izone, CropCS, isoil) + 100
    alpha = (DPcap(izone, CropCS, isoil) - DPLL(izone, CropCS, isoil)) / (DPUL - DPLL(izone, CropCS, isoil))
    
    IF (DP1irr(0) + DP2(0) .GT. DPLL(izone, CropCS, isoil)) THEN
        IF (DP1irr(0) + DP2(0) .GT. DPUL) THEN
            DPirrtot(0) = DPcap(izone, CropCS, isoil)
        ELSE
            DPirrtot(0) = DPLL(izone, CropCS, isoil) + (DPcap(izone, CropCS, isoil) - DPLL(izone, CropCS, isoil)) * &
                (1 - (1 - (DP1irr(0) + DP2(0) - DPLL(izone, CropCS, isoil)) / (DPUL - DPLL(izone, CropCS, isoil)))**(1 / alpha))
        END IF
        
        DPsum = DP1irr(0) + DP2(0)
        DO n = 1, 12
            DPirrtot(n) = DPirrtot(0) * (DP1irr(n) + DP2(n)) / DPsum
            DP2RO(n) = DP1irr(n) + DP2(n) - DPirrtot(n)
        END DO
        DP2RO(0) = SUM(DP2RO(1 : 12))
        DPirrtot(0) = SUM(DPirrtot(1 : 12))
    ELSE
        DPirrtot = DP1irr + DP2
        DPirrtot(0) = SUM(DPirrtot(1 : 12))
        DP2RO = 0.
    END IF
    
END SUBROUTINE CalcIrrCrop
!****************************************************************************
!SUBROUTINE DistETGain(wtRODP, GainET, MaxET, DryET, WatApp, GainETm, RO2a, DP2a, ETb, ET2)
SUBROUTINE DistETGain(GainET, MaxET, DryET, WatApp, GainETm, ETb, ET2)

    USE PARAM
    IMPLICIT NONE
    
    INTEGER     flag(12), n
    
    REAL    ETb(0 : 12), ET2(0 : 12)
    REAL    GainET, GainETm(12)
    REAL    WatApp(0 : 12), MaxET(0 : 12), DryET(0 : 12)
    REAL    wtRODP(0 : 12), RO2a(0 : 12), DP2a(0 : 12)
    
    REAL    ETProp(12), ETG, irrETsum, dryETsum, AppWsum
    
    flag = 1
    ETG = GainET
    
    DO n = 1, 12
        IF (WatApp(n) .LE. 0.) flag(n) = 0
        IF (MaxET(n) .LE. DryET(n)) flag(n) = 0
    END DO
    
15  irrETsum = 0.
    dryETsum = 0.
    
    DO n = 1, 12
        IF (flag(n) .EQ. 1) THEN
            irrETsum = irrETsum + MaxET(n)
            dryETsum = dryETsum + DryET(n)
        END IF
    END DO
    
    DO n = 1, 12
        IF (flag(n) .EQ. 1) THEN
            ETprop(n) = (MaxET(n) - DryET(n)) / (irrETsum - dryETsum)
            GainETm(n) = ETG * ETprop(n)
            IF (GainETm(n) .GT. WatApp(n)) THEN
                GainETm(n) = WatApp(n)
                flag(n) = 0
                ETG = ETG - WatApp(n)
                GOTO 15
            END IF
        END IF
    END DO
    
    ETG = SUM(GainETm)
    
    IF (ETG .LT. GainET) THEN
        flag = 1
        ETG = GainET - ETG
        IF (ETG .GT. 0.00001) THEN
            DO n = 1, 12
                IF (WatApp(n) .LE. 0.) flag(n) = 0
                IF (MaxET(n) .GT. DryET(n)) flag(n) = 0
            END DO
            
16          AppWsum = 0.
            
            DO n = 1, 12
                IF (flag(n) .EQ. 1) THEN
                    AppWsum = AppWsum + WatApp(n)
                END IF
            END DO
            
            DO n = 1, 12
                IF (flag(n) .EQ. 1) THEN
                    GainETm(n) = ETG * WatApp(n) / AppWsum
                    IF (GainETm(n) .GT. WatApp(n)) THEN
                        GainETm(n) = WatApp(n)
                        flag(n) = 0
                        ETG = ETG - WatApp(n)
                        GOTO 16
                    END IF
                END IF
            END DO
        END IF
    END IF
    
    ETG = SUM(GainETm)
    
    IF (ETG .LT. GainET) THEN
        flag = 1 
        ETG = GainET - ETG
        IF (ETG .GT. 0.00001) THEN
            DO n = 1, 12
                IF (WatApp(n) .GT. 0.) flag(n) = 0
                IF (DryET(n) .LE. MaxET(n)) flag(n) = 0
            END DO
            
            irrETsum = 0.
            
            DO n = 1, 12
                IF (flag(n) .EQ. 1) THEN
                    irrETsum = irrETsum + MaxET(n)
                END IF
            END DO
            
            DO n = 1, 12
                IF (flag(n) .EQ. 1) THEN
                    GainETm(n) = ETG * MaxET(n) / irrETsum
                END IF
            END DO
        END IF
    END IF
    
    DO n = 1, 12
        IF (ISNAN(GainETm(n))) THEN
            CONTINUE
        END IF
        
        IF (WatApp(n) .LE. 0.) THEN
            ET2(n) = GainETm(n) + MaxET(n)
            ETb(n) = MaxET(n)
        ELSE
            IF (MaxET(n) .GT. DryET(n)) THEN
                ET2(n) = GainETm(n) + DryET(n)
                ETb(n) = DryET(n)
            ELSE
                ET2(n) = GainETm(n) + MaxET(n)
                ETb(n) = MaxET(n)
            END IF
        END IF
    END DO
    
END SUBROUTINE DistETGain
!****************************************************************************