!   WSPP_Report.f90 
!
!   Program 4 of the COHYST SUITE of programs
!
!   Purpose:
!   Using the results from the WSPP program, the water balance summary is created for the total area and 
!   each coefficient zone, runoff zone, county, irrigation district.
!   Summaries are also created for irrigation source, crop, and irrigation source and crop.
!   
!   Development log:
!   February 2014, Isaac Mortensen - TFG
!
!****************************************************************************
!
!  PROGRAM: WSPP_Report
!
!
!
!****************************************************************************
MODULE PARAM

    DATA MasterDIR /'W:\COHYST\COHYST\Run028\Run028b2\Inputs\'/
    DATA MasterFile /'MasterInput_v27.1.txt'/

    INTEGER, PARAMETER :: ncounty = 565
    INTEGER, PARAMETER :: subcnppid = 10
    INTEGER, PARAMETER :: nUarea = 10
    
    CHARACTER*12, ALLOCATABLE ::    Crops(:)
    
    CHARACTER*1000  INDIR, OUTDIR, WBPDIR, LUDIR, CNLDIR, MasterDIR
    CHARACTER*1000  MISCDIR, MIDIR
    CHARACTER*250   GWLUfld, SWLUfld, COLUfld, DryLUfld, GWCfld
    CHARACTER*250   GWLU, SWLU, COLU, DryLU, cRun, MODname
    CHARACTER*250   CellLoc, Coeffile, AEfile, ROZfile, GWCfile, MasterFile
    CHARACTER*250   PctRchfile, SWidxfile, CountID, Cell_county
    CHARACTER*250   cnlmastfile, miscrmastfile, miscpmastfile, mirmastfile, mipmastfile
    CHARACTER*250   ActCellFile
    !Added 1/7/2016
    CHARACTER*250   Callyrfile
    
    INTEGER     ncells, nczones, nrzones, nsoils, nirrdist, ncrops
    INTEGER     startyr, endyr, DPonly, SWonly
    INTEGER     ncols, nrows, cnlrflg, miscrflg, miscpflg, mirflg, mipflg
    INTEGER     Rch0flg
    !Added 1/7/2016
    INTEGER     ctrlyrs
    
    REAL    csize
    
    CHARACTER*250   rchfld, rchfile, pmpfld, pmpfile
    CHARACTER*25    CountyName(ncounty), cstate
    CHARACTER*4     YRT
    !Added 1/7/2016
    CHARACTER*4     YRTwbp, YRTcnl, YRTmisc, YRTmi
    
    INTEGER     idx, GEOID, cell
    INTEGER     Activecounty(ncounty)
    INTEGER     swdist, iyear
    INTEGER     yr, icrop, i
    INTEGER     mon, swidx, pflag, isoil, iczone, irzone, iuArea
    INTEGER     imon, irri, icounty, izone, irrd
    
    INTEGER     ncanals, nmiscr, nmunir, nmiscp, nmunip
    INTEGER     maxcanallen, maxmiscrlen, maxmunirlen, maxmiscplen, maxmuniplen
    
    REAL    acs, AP, SWD, ET, RO, DP, SL, ETg, ETb, DET1, DET2
    REAL    DP1, DP2, RO1, RO2, DP2RO, Ewat, ETtrans, NIRmET, Latseep
    REAL    LF, SF, RO2DP, RO2ET
    
    REAL    wPrecip, wAcs, wAP, wSWD, wET, wRO, wDP, wSL, wETg, wETb
    REAL    wDET1, wDET2, wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans
    REAL    wLatseep, wSF, wRO2DP, wRO2ET
    
    REAL    monval(12), annval
    
    CHARACTER*100, ALLOCATABLE :: Canalname(:), MiscrName(:), MunirName(:), MiscpName(:), MunipName(:)
    
    INTEGER, ALLOCATABLE :: Canalnamelen(:), MiscrNamelen(:), MunirNamelen(:), MiscpNamelen(:), MunipNamelen(:)
    
    INTEGER, ALLOCATABLE :: cntyidx(:), Soil(:), CoefZ(:), ROZ(:)
    INTEGER, ALLOCATABLE :: SWDistx(:)
    INTEGER, ALLOCATABLE :: ZeroRCH(:)
    INTEGER, ALLOCATABLE :: CNPPIDCell(:), uAreaCell(:)
    !Added 1/7/2016
    INTEGER, ALLOCATABLE :: WBPyear(:), CNLyear(:), MISCyear(:), MIyear(:)
    INTEGER, ALLOCATABLE :: yrctrl(:)
    
    REAL, ALLOCATABLE :: Precip(:,:)
    REAL, ALLOCATABLE :: LPM(:), M2G(:)
    REAL, ALLOCATABLE :: PctRch(:)
    
    DOUBLE PRECISION, ALLOCATABLE :: rCanalRch(:,:), rMiscrch(:,:), rMunirch(:,:), rMiscpmp(:,:), rMunipmp(:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cCanalRch(:,:,:), cMiscrch(:,:,:), cMunirch(:,:,:), cMiscpmp(:,:,:), cMunipmp(:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cfCanalRch(:,:,:), cfMiscrch(:,:,:), cfMunirch(:,:,:), cfMiscpmp(:,:,:), cfMunipmp(:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rzCanalRch(:,:,:), rzMiscrch(:,:,:), rzMunirch(:,:,:), rzMiscpmp(:,:,:), rzMunipmp(:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: UACanalRch(:,:,:), UAMiscrch(:,:,:), UAMunirch(:,:,:), UAMiscpmp(:,:,:), UAMunipmp(:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: rPrecip(:,:,:,:), rAcs(:,:,:,:), rAP(:,:,:,:), rSWD(:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rET(:,:,:,:), rRO(:,:,:,:), rDP(:,:,:,:), rSL(:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: rETg(:,:,:,:), rETb(:,:,:,:), rDET1(:,:,:,:), rDET2(:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rDP1(:,:,:,:), rDP2(:,:,:,:), rRO1(:,:,:,:), rRO2(:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rDP2RO(:,:,:,:), rETtrans(:,:,:,:), rLatseep(:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rSF(:,:,:,:), rRO2DP(:,:,:,:), rRO2ET(:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: cPrecip(:,:,:,:,:), cAcs(:,:,:,:,:), cAP(:,:,:,:,:), cSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cET(:,:,:,:,:), cRO(:,:,:,:,:), cDP(:,:,:,:,:), cSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: cETg(:,:,:,:,:), cETb(:,:,:,:,:), cDET1(:,:,:,:,:), cDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cDP1(:,:,:,:,:), cDP2(:,:,:,:,:), cRO1(:,:,:,:,:), cRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cDP2RO(:,:,:,:,:), cETtrans(:,:,:,:,:), cLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cSF(:,:,:,:,:), cRO2DP(:,:,:,:,:), cRO2ET(:,:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: cfPrecip(:,:,:,:,:), cfAcs(:,:,:,:,:), cfAP(:,:,:,:,:), cfSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cfET(:,:,:,:,:), cfRO(:,:,:,:,:), cfDP(:,:,:,:,:), cfSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: cfETg(:,:,:,:,:), cfETb(:,:,:,:,:), cfDET1(:,:,:,:,:), cfDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cfDP1(:,:,:,:,:), cfDP2(:,:,:,:,:), cfRO1(:,:,:,:,:), cfRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cfDP2RO(:,:,:,:,:), cfETtrans(:,:,:,:,:), cfLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cfSF(:,:,:,:,:), cfRO2DP(:,:,:,:,:), cfRO2ET(:,:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: rzPrecip(:,:,:,:,:), rzAcs(:,:,:,:,:), rzAP(:,:,:,:,:), rzSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rzET(:,:,:,:,:), rzRO(:,:,:,:,:), rzDP(:,:,:,:,:), rzSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: rzETg(:,:,:,:,:), rzETb(:,:,:,:,:), rzDET1(:,:,:,:,:), rzDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rzDP1(:,:,:,:,:), rzDP2(:,:,:,:,:), rzRO1(:,:,:,:,:), rzRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rzDP2RO(:,:,:,:,:), rzETtrans(:,:,:,:,:), rzLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: rzSF(:,:,:,:,:), rzRO2DP(:,:,:,:,:), rzRO2ET(:,:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: swdPrecip(:,:,:,:,:), swdAcs(:,:,:,:,:), swdAP(:,:,:,:,:), swdSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: swdET(:,:,:,:,:), swdRO(:,:,:,:,:), swdDP(:,:,:,:,:), swdSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: swdETg(:,:,:,:,:), swdETb(:,:,:,:,:), swdDET1(:,:,:,:,:), swdDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: swdDP1(:,:,:,:,:), swdDP2(:,:,:,:,:), swdRO1(:,:,:,:,:), swdRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: swdDP2RO(:,:,:,:,:), swdETtrans(:,:,:,:,:), swdLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: swdSF(:,:,:,:,:), swdRO2DP(:,:,:,:,:), swdRO2ET(:,:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: cnppidPrecip(:,:,:,:,:), cnppidAcs(:,:,:,:,:), cnppidAP(:,:,:,:,:), cnppidSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cnppidET(:,:,:,:,:), cnppidRO(:,:,:,:,:), cnppidDP(:,:,:,:,:), cnppidSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: cnppidETg(:,:,:,:,:), cnppidETb(:,:,:,:,:), cnppidDET1(:,:,:,:,:), cnppidDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cnppidDP1(:,:,:,:,:), cnppidDP2(:,:,:,:,:), cnppidRO1(:,:,:,:,:), cnppidRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cnppidDP2RO(:,:,:,:,:), cnppidETtrans(:,:,:,:,:), cnppidLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: cnppidSF(:,:,:,:,:), cnppidRO2DP(:,:,:,:,:), cnppidRO2ET(:,:,:,:,:)
    
    DOUBLE PRECISION, ALLOCATABLE :: uAreaPrecip(:,:,:,:,:), uAreaAcs(:,:,:,:,:), uAreaAP(:,:,:,:,:), uAreaSWD(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: uAreaET(:,:,:,:,:), uAreaRO(:,:,:,:,:), uAreaDP(:,:,:,:,:), uAreaSL(:,:,:,:,:) 
    DOUBLE PRECISION, ALLOCATABLE :: uAreaETg(:,:,:,:,:), uAreaETb(:,:,:,:,:), uAreaDET1(:,:,:,:,:), uAreaDET2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: uAreaDP1(:,:,:,:,:), uAreaDP2(:,:,:,:,:), uAreaRO1(:,:,:,:,:), uAreaRO2(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: uAreaDP2RO(:,:,:,:,:), uAreaETtrans(:,:,:,:,:), uAreaLatseep(:,:,:,:,:)
    DOUBLE PRECISION, ALLOCATABLE :: uAreaSF(:,:,:,:,:), uAreaRO2DP(:,:,:,:,:), uAreaRO2ET(:,:,:,:,:)

END MODULE PARAM
!****************************************************************************
PROGRAM WSPP_Report
    USE PARAM
    IMPLICIT NONE
    
    OPEN (1, FILE = TRIM(MasterDIR)//TRIM(MasterFile), STATUS = 'OLD')
    READ (1, *)

    READ (1, *) cRun
    READ (1, *) MODname
    READ (1, *) INDIR
    READ (1, *) OUTDIR
    READ (1, *) WBPDIR
    READ (1, *) LUDIR
    READ (1, *) CNLDIR
    READ (1, *) MiscDIR
    READ (1, *) MIDIR
    READ (1, *) GWLUfld
    READ (1, *) SWLUfld
    READ (1, *) COLUfld
    READ (1, *) DryLUfld
    READ (1, *) GWLU
    READ (1, *) SWLU
    READ (1, *) COLU
    READ (1, *) DryLU
    READ (1, *) GWCfld
    READ (1, *) GWCfile
    READ (1, *) CellLoc
    READ (1, *) Coeffile
    READ (1, *) AEfile
    READ (1, *) ROZfile
    READ (1, *) PctRchfile
    READ (1, *) SWidxfile
    READ (1, *) CountID
    READ (1, *) Cell_county
    READ (1, *) CNLmastfile
    READ (1, *) MISCrmastfile
    READ (1, *) MISCpmastfile
    READ (1, *) MIrmastfile
    READ (1, *) MIpmastfile
    READ (1, *) ActCellfile
    !Added 1/7/2016
    READ (1, *) CallYrfile    
    READ (1, *) ncells
    READ (1, *) startyr
    READ (1, *) endyr
    READ (1, *) RCH0flg
    READ (1, *) DPonly
    READ (1, *) SWonly
    READ (1, *) cnlrflg
    READ (1, *) miscrflg
    READ (1, *) miscpflg
    READ (1, *) MIrflg
    READ (1, *) MIpflg
    !Added 1/7/2016
    READ (1, *) ctrlyrs
    READ (1, *) nczones
    READ (1, *) nrzones
    READ (1, *) nsoils
    READ (1, *) nirrdist
    READ (1, *) csize
    READ (1, *) ncols
    READ (1, *) nrows
    READ (1, *) ncrops
    
    ALLOCATE (Crops(ncrops))
    
    DO i = 1, ncrops
        READ (1, *) crops(i)
    END DO
    CLOSE (1)
    
    OUTDIR = TRIM(OUTDIR)//TRIM(cRun)//'\Results\'
    
    ALLOCATE (cntyidx(ncells), Soil(ncells), CoefZ(ncells), ROZ(ncells), M2G(ncells))
    ALLOCATE (SWDistx(0 : nirrdist))
    ALLOCATE (Precip(ncells, 0 : 12))
    ALLOCATE (LPM(0 : nrzones))
    ALLOCATE (PctRch(nczones))
    ALLOCATE (ZeroRCH(ncells))
    ALLOCATE (CNPPIDcell(ncells))
    ALLOCATE (uAreaCell(ncells))
    
    ALLOCATE (rPrecip(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rAcs(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rAP(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rSWD(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rET(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rRO(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDP(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rSL(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rETg(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rETb(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDET1(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDET2(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDP1(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDP2(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rRO1(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rRO2(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rDP2RO(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rETtrans(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rLatseep(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rSF(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rRO2DP(0 : 12, ncrops, 4, nsoils))
    ALLOCATE (rRO2ET(0 : 12, ncrops, 4, nsoils))
    
    ALLOCATE (cPrecip(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cAcs(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cAP(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cSWD(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cET(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cRO(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDP(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cSL(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cETg(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cETb(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDET1(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDET2(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDP1(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDP2(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cRO1(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cRO2(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cDP2RO(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cETtrans(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cLatseep(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cSF(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cRO2DP(0 : 12, ncrops, 4, nsoils, ncounty))
    ALLOCATE (cRO2ET(0 : 12, ncrops, 4, nsoils, ncounty))
    
    ALLOCATE (cfPrecip(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfAcs(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfAP(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfSWD(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfET(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfRO(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDP(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfSL(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfETg(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfETb(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDET1(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDET2(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDP1(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDP2(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfRO1(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfRO2(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfDP2RO(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfETtrans(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfLatseep(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfSF(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfRO2DP(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    ALLOCATE (cfRO2ET(0 : 12, ncrops, 4, nsoils, 0 : nczones))
    
    ALLOCATE (rzPrecip(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzAcs(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzAP(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzSWD(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzET(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzRO(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDP(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzSL(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzETg(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzETb(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDET1(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDET2(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDP1(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDP2(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzRO1(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzRO2(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzDP2RO(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzETtrans(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzLatseep(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzSF(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzRO2DP(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    ALLOCATE (rzRO2ET(0 : 12, ncrops, 4, nsoils, 0 : nrzones))
    
    ALLOCATE (swdPrecip(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdAcs(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdAP(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdSWD(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdET(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdRO(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDP(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdSL(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdETg(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdETb(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDET1(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDET2(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDP1(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDP2(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdRO1(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdRO2(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdDP2RO(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdETtrans(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdLatseep(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdSF(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdRO2DP(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    ALLOCATE (swdRO2ET(0 : 12, ncrops, 4, nsoils, 0 : nirrdist))
    
    ALLOCATE (cnppidPrecip(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidAcs(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidAP(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidSWD(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidET(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidRO(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDP(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidSL(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidETg(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidETb(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDET1(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDET2(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDP1(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDP2(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidRO1(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidRO2(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidDP2RO(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidETtrans(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidLatseep(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidSF(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidRO2DP(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    ALLOCATE (cnppidRO2ET(0 : 12, ncrops, 4, nsoils, 0 : subcnppid))
    
    ALLOCATE (uAreaPrecip(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaAcs(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaAP(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaSWD(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaET(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaRO(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDP(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaSL(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaETg(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaETb(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDET1(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDET2(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDP1(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDP2(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaRO1(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaRO2(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaDP2RO(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaETtrans(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaLatseep(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaSF(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaRO2DP(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
    ALLOCATE (uAreaRO2ET(0 : 12, ncrops, 4, nsoils, 0 : nuArea))
 
    !Added 1/7/2016
    ALLOCATE (WBPyear(startyr : endyr), CNLyear(startyr : endyr), MISCyear(startyr : endyr), MIyear(startyr : endyr))
    ALLOCATE (yrctrl(ctrlyrs))
    
    !Populate County index
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CountID), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) idx, GEOID, CountyName(idx), cState
    END DO
    CLOSE (1)
    
    Activecounty = 0
    !Polulate County names
    OPEN (1, FILE = TRIM(INDIR)//TRIM(Cell_County), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, GEOID, Cntyidx(cell)
        Activecounty(Cntyidx(cell)) = 1
    END DO
    CLOSE (1)
    
    !Populate cell soils, zones, and location
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CellLoc), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, Soil(cell), CoefZ(cell), ROZ(cell), M2G(cell)
    END DO
    CLOSE (1)  
    
    SWDistx = 0
    !Read SWIndex
    OPEN (1, FILE = TRIM(INDIR)//TRIM(SWidxfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) idx, swdist
        SWDistx(idx) = swdist
    END DO
    CLOSE (1) 
    
    pctrch = 0.
    !get runoff loss partitioning factor
    OPEN (1, FILE = TRIM(INDIR)//TRIM(pctrchfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) izone, PctRch(izone)
    END DO
    CLOSE (1)
    
    LPM = 0.
    !Get Loss per mile parameters
    OPEN (1, FILE = TRIM(INDIR)//TRIM(ROZfile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) izone, LPM(izone)
    END DO
    CLOSE (1)
    
    ZeroRCH = 0
    IF (RCH0flg .EQ. 1) THEN
        !Determine Nulled Cells
        OPEN (1, FILE = TRIM(INDIR)//TRIM(ActCellfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, idx, ZeroRCH(cell)
        END DO
        CLOSE (1)
    END IF
    
    !Read in Call year
    !Added 1/7/2016 to control the annual input data and allow the same data to be applied to different years
    !ex: applying a fixed land use to the historical climate
    OPEN (1, FILE = TRIM(INDIR)//TRIM(CallYrFile), STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) yr, yrctrl
        IF ((yr .GE. startyr) .AND. (yr .LE. endyr)) THEN
            WBPyear(yr) = yrctrl(2)
            CNLyear(yr) = yrctrl(4)
            Miscyear(yr) = yrctrl(5)
            MIyear(yr) = yrctrl(6)
        END IF
    END DO
    CLOSE (1)
    
    !Populate CNPPID sub-canal cells
    CNPPIDcell = 0
    OPEN (1, FILE = TRIM(INDIR)//'subdist_cells.txt', STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, CNPPIDcell(cell)
    END DO
    CLOSE (1)
    
    !Populate UArea cells
    uAreaCell = 0
    OPEN (1, FILE = TRIM(INDIR)//'Uareas.txt', STATUS = 'OLD')
    READ (1, *)
    DO WHILE (.NOT. EOF(1))
        READ (1, *) cell, uAreaCell(cell)
    END DO
    CLOSE (1)
    
    ncanals = 0
    IF (cnlrflg .EQ. 1) THEN
        OPEN (2, FILE = TRIM(INDIR)//TRIM(CNLmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) idx, rchfld, rchfile
        END DO
        CLOSE (2)
        
        ncanals = idx
        ALLOCATE (CanalName(ncanals), CanalNamelen(ncanals))
        ALLOCATE (rCanalRch(ncanals, 12), cCanalRch(ncanals, ncounty, 12), cfCanalRch(ncanals, 0 : nczones, 12), rzCanalRch(ncanals, 0 : nrzones, 12), UACanalRch(ncanals, 0 : nuArea, 12))
        
        OPEN (2, FILE = TRIM(INDIR)//TRIM(CNLmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) idx, rchfld, rchfile
            CanalName(idx) = rchfld
            CanalNamelen(idx) = LEN_TRIM(CanalName(idx))
        END DO
        CLOSE (2)
        maxcanallen = MAXVAL(CanalNamelen)
    END IF
    
    nmiscr = 0
    IF (Miscrflg .EQ. 1) THEN
        idx = 0
        OPEN (2, File = TRIM(INDIR)//TRIM(MISCrmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) rchfld, rchfile
            idx = idx + 1
        END DO 
        CLOSE (2)
        
        nmiscr = idx
        ALLOCATE (MiscrName(nmiscr), MiscrNamelen(nmiscr))
        ALLOCATE (rMiscrch(nmiscr, 12), cMiscrch(nmiscr, ncounty, 12), cfMiscrch(nmiscr, nczones, 12), rzMiscrch(nmiscr, 0 : nrzones, 12), UAMiscrch(nmiscr, 0 : nuArea, 12))
        
        idx = 0
        OPEN (2, File = TRIM(INDIR)//TRIM(MISCrmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) rchfld, rchfile
            idx = idx + 1
            MiscrName(idx) = rchfld
            MiscrNamelen(idx) = LEN_TRIM(MiscrName(idx))
        END DO 
        CLOSE (2)
        maxmiscrlen = MAXVAL(MiscrNamelen)
    END IF
    
    nMunIr = 0
    IF (MIrflg .EQ. 1) THEN
        idx = 0
        OPEN (2, FILE = TRIM(INDIR)//TRIM(MIrmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) Rchfld, rchfile
            idx = idx + 1
        END DO 
        CLOSE (2)
        
        nMunIr = idx
        ALLOCATE (MunIrName(nMunIr), MunirNamelen(nMunir)) 
        ALLOCATE (rMunirch(nMunir, 12), cMunirch(nMunir, ncounty, 12), cfMunirch(nMunir, nczones, 12), rzMunirch(nMunir, 0 : nrzones, 12), UAMunirch(nMunir, 0 : nuArea, 12))
        
        idx = 0
        OPEN (2, FILE = TRIM(INDIR)//TRIM(MIrmastfile), STATUS = 'OLD')
        READ (2, *)
        DO WHILE (.NOT. EOF(2))
            READ (2, *) Rchfld, rchfile
            idx = idx + 1
            MunirName(idx) = rchfld
            MunirNamelen(idx) = LEN_TRIM(MunirName(idx))
        END DO 
        CLOSE (2)
        maxmunirlen = MAXVAL(MunirNamelen)
    END IF
    
    nMiscp = 0
    IF (miscpflg .EQ. 1) THEN
        idx = 0
        OPEN (1, FILE = TRIM(INDIR)//TRIM(MISCpmastfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) pmpfld, pmpfile
            idx = idx + 1
        END DO
        CLOSE (1)
        
        nMiscp = idx
        ALLOCATE (MiscpName(nMiscp), MiscpNamelen(nMiscp))
        ALLOCATE (rMiscpmp(nMiscp, 12), cMiscpmp(nMiscp, ncounty, 12), cfMiscpmp(nMiscp, nczones, 12), rzMiscpmp(nMiscp, 0 : nrzones, 12), UAMiscpmp(nMiscp, 0 : nuArea, 12))
        
        idx = 0
        OPEN (1, FILE = TRIM(INDIR)//TRIM(MISCpmastfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) pmpfld, pmpfile
            idx = idx + 1
            MiscpName(idx) = pmpfld
            MiscpNamelen(idx) = LEN_TRIM(MiscpName(idx))
        END DO
        CLOSE (1)
        maxmiscplen = MAXVAL(MiscpNamelen)
    END IF
    
    nMunip = 0
    IF (MIpflg .EQ. 1) THEN
        idx = 0
        OPEN (1, FILE = TRIM(INDIR)//TRIM(MIpmastfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) pmpfld, pmpfile
            idx = idx + 1
        END DO
        CLOSE (1)
        
        nMunip = idx
        ALLOCATE (MunipName(nMunip), MunipNamelen(nMunip))
        ALLOCATE (rMunipmp(nMunip, 12), cMunipmp(nMunip, ncounty, 12), cfMunipmp(nMunip, 0 : nczones, 12), rzMunipmp(nMunip, 0 : nrzones, 12), UAMunipmp(nMunip, 0 : nuArea, 12))
        
        idx = 0
        OPEN (1, FILE = TRIM(INDIR)//TRIM(MIpmastfile), STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) pmpfld, pmpfile
            idx = idx + 1
            MunipName(idx) = pmpfld
            MunipNamelen(idx) = LEN_TRIM(MunipName(idx))
        END DO
        CLOSE (1)
        maxmuniplen = MAXVAL(MunipNamelen)
    END IF
    
    CALL openreports
    WRITE (6, 600) 
    DO iyear = startyr, endyr
        !Altered 1/7/2016
        WRITE (6, 601) iyear, WBPyear(iyear), CNLyear(iyear), Miscyear(iyear), MIyear(iyear)
        WRITE (YRT, '(I4)') iyear        
        
        !Added 1/7/2016
        WRITE (YRTwbp, '(I4)') WBPyear(iyear)
        WRITE (YRTcnl, '(I4)') CNLyear(iyear)
        WRITE (YRTmisc, '(I4)') MISCyear(iyear)
        WRITE (YRTmi, '(I4)') MIyear(iyear)
        
        CALL zerovals
        
        Precip = 0.
        OPEN (1, FILE = TRIM(WBPDIR)//'Precip\PRECIP'//TRIM(YRTwbp)//'.txt', STATUS = 'OLD')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            READ (1, *) cell, yr, icrop, Precip(cell, 0 : 12)
            Precip(cell, 0) = SUM(Precip(cell, 1 : 12))
        END DO
        CLOSE (1)
        
        rCanalRch = 0.
        cCanalrch = 0.
        cfCanalrch = 0.
        rzCanalrch = 0.
        UACanalrch = 0.
        !Get Canal Recharge
        IF (Cnlrflg .EQ. 1) THEN
            OPEN (2, FILE = TRIM(INDIR)//TRIM(CNLmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2))
                READ (2, *) idx, rchfld, rchfile
        
                OPEN (1, FILE = TRIM(CNLDIR)//TRIM(rchfld)//'\'//TRIM(rchfile)//TRIM(YRTcnl)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    IF (ZeroRch(cell) .EQ. 0) THEN
                        rcanalrch(idx, :) = rcanalrch(idx, :) + monval
                        ccanalrch(idx, cntyidx(cell), :) = ccanalrch(idx, cntyidx(cell), :) + monval
                        cfcanalrch(idx, CoefZ(cell), :) = cfcanalrch(idx, CoefZ(cell), :) + monval
                        rzcanalrch(idx, ROZ(cell), :) = rzcanalrch(idx, ROZ(cell), :) + monval
                        UAcanalrch(idx, uAreaCell(cell), :) = UAcanalrch(idx, uAreaCell(cell), :) + monval
                    END IF
                END DO
                CLOSE (1)
                
            END DO
            CLOSE (2)
        END IF
        
        rmiscrch = 0.
        cmiscrch = 0.
        cfmiscrch = 0.
        rzmiscrch = 0.
        UAmiscrch = 0.
        !Miscellaneous Recharge
        IF (Miscrflg .EQ. 1) THEN
            idx = 0
            OPEN (2, FILE = TRIM(INDIR)//TRIM(MISCrmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2))
                READ (2, *) rchfld, rchfile
                idx = idx + 1
                
                OPEN (1, FILE = TRIM(MiscDIR)//TRIM(rchfld)//'\'//TRIM(rchfile)//TRIM(YRTmisc)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    IF (ZeroRch(cell) .EQ. 0) THEN
                        rmiscrch(idx, :) = rmiscrch(idx, :) + monval
                        cmiscrch(idx, cntyidx(cell), :) = cmiscrch(idx, cntyidx(cell), :) + monval
                        cfmiscrch(idx, CoefZ(cell), :) = cfmiscrch(idx, CoefZ(cell), :) + monval
                        rzmiscrch(idx, ROZ(cell), :) = rzmiscrch(idx, ROZ(cell), :) + monval
                        UAmiscrch(idx, uAreaCell(cell), :) = UAmiscrch(idx, uAreaCell(cell), :) + monval
                    END IF
                END DO
                CLOSE (1)
                
            END DO
            CLOSE (2)
        END IF
        
        rMunIrch = 0.
        cMunirch = 0.
        cfMunirch = 0.
        rzMunirch = 0.
        UAMunirch = 0.
        !Municipal and Industrial Recharge
        IF (MIrflg .EQ. 1) THEN
            idx = 0
            OPEN (2, FILE = TRIM(INDIR)//TRIM(MIrmastfile), STATUS = 'OLD')
            READ (2, *)
            DO WHILE (.NOT. EOF(2))
                READ (2, *) Rchfld, rchfile
                idx = idx + 1
                
                OPEN (1, FILE = TRIM(MIDIR)//TRIM(RCHfld)//'\'//TRIM(rchfile)//TRIM(YRTmi)//'.txt', STATUS = 'OLD')
                READ (1, *)
                DO WHILE (.NOT. EOF(1))
                    READ (1, *) cell, yr, annval, monval
                    IF (ZeroRch(cell) .EQ. 0) THEN
                        rMunirch(idx, :) = rMunIrch(idx, :) + monval
                        cMunirch(idx, cntyidx(cell), :) = cMunirch(idx, cntyidx(cell), :) + monval
                        cfmunirch(idx, CoefZ(cell), :) = cfmunirch(idx, CoefZ(cell), :) + monval
                        rzmunirch(idx, ROZ(cell), :) = rzmunirch(idx, ROZ(cell), :) + monval
                        UAmunirch(idx, uAreaCell(cell), :) = UAmunirch(idx, uAreaCell(cell), :) + monval
                    END IF
                END DO
                CLOSE (1)
            
            END DO
            CLOSE (2)
        END IF
        
        rMiscpmp = 0.
        cMiscpmp = 0.
        cfMiscpmp = 0.
        rzMiscpmp = 0.
        UAMiscpmp = 0.
        !Miscellaneous Pumping
        IF (miscpflg .EQ. 1) THEN
            idx = 0
            OPEN (1, FILE = TRIM(INDIR)//TRIM(MISCpmastfile), STATUS = 'OLD')
            READ (1, *)
            DO WHILE (.NOT. EOF(1))
                READ (1, *) pmpfld, pmpfile
                idx = idx + 1
                
                OPEN (2, FILE = TRIM(MiscDIR)//TRIM(pmpfld)//'\'//TRIM(pmpfile)//TRIM(YRTmisc)//'.txt', STATUS = 'OLD')
                READ (2, *)
                DO WHILE (.NOT. EOF(2))
                    READ (2, *) cell, yr, annval, monval
                    IF (ZeroRCH(cell) .EQ. 0) THEN
                        rMiscpmp(idx, :) = rMiscpmp(idx, :) + monval
                        cMiscpmp(idx, cntyidx(cell), :) = cMiscpmp(idx, cntyidx(cell), :) + monval
                        cfMiscpmp(idx, CoefZ(cell), :) = cfMiscpmp(idx, CoefZ(cell), :) + monval
                        rzMiscpmp(idx, ROZ(cell), :) = rzMiscpmp(idx, ROZ(cell), :) + monval
                        UAMiscpmp(idx, uAreaCell(cell), :) = UAMiscpmp(idx, uAreaCell(cell), :) + monval
                    END IF
                END DO
                CLOSE (2)
                
            END DO
            CLOSE (1)
        END IF
        
        rMunipmp = 0.
        cMunipmp = 0.
        cfMunipmp = 0.
        rzMunipmp = 0.
        UAMunipmp = 0.
        !Municipal and Industrial Pumping
        IF (MIpflg .EQ. 1) THEN
            idx = 0
            OPEN (1, FILE = TRIM(INDIR)//TRIM(MIpmastfile), STATUS = 'OLD')
            READ (1, *)
            DO WHILE (.NOT. EOF(1))
                READ (1, *) pmpfld, pmpfile
                idx = idx + 1
                
                OPEN (2, FILE = TRIM(MIDIR)//TRIM(pmpfld)//'\'//TRIM(pmpfile)//TRIM(YRTmi)//'.txt', STATUS = 'OLD')
                READ (2, *)
                DO WHILE (.NOT. EOF(2))
                    READ (2, *) cell, yr, annval, monval
                    IF (ZeroRch(cell) .EQ. 0) THEN
                        rMunipmp(idx, :) = rMunipmp(idx, :) + monval
                        cMunipmp(idx, cntyidx(cell), :) = cMunipmp(idx, cntyidx(cell), :) + monval
                        cfMunipmp(idx, CoefZ(cell), :) = cfMunipmp(idx, CoefZ(cell), :) + monval
                        rzMunipmp(idx, ROZ(cell), :) = rzMunipmp(idx, ROZ(cell), :) + monval
                        UAMunipmp(idx, uAreaCell(cell), :) = UAMunipmp(idx, uAreaCell(cell), :) + monval
                    END IF
                END DO
                CLOSE (2)
                
            END DO
            CLOSE (1)
        END IF
        
        OPEN (1, FILE = TRIM(OUTDIR)//'RAW/RAW_WSPP'//TRIM(YRT)//'.txt', STATUS = 'UNKNOWN')
        READ (1, *)
        DO WHILE (.NOT. EOF(1))
            
            READ (1, *) Cell, yr, mon, icrop, swdist, swidx, pflag, isoil, iczone, irzone, & !10
                            acs, AP, SWD, ET, RO, DP, SL, ETg, ETb, DET1, DET2, DP1, DP2, & !13
                            RO1, RO2, DP2RO, Ewat, ETtrans, NIRmET, Latseep !7
            
            IF (ZeroRCH(Cell) .EQ. 1) THEN
                GOTO 1
            END IF
            
            IF (M2G(cell) .EQ. 0.) THEN
                LF = 0.5
            ELSE
                LF = MIN(1 - EXP(-LPM(ROZ(Cell)) * M2G(cell)), 1.0)
            END IF
            
            SF = RO * (1 - LF)
            RO2DP = RO * LF * PctRch(CoefZ(cell))
            RO2ET = RO * LF * (1 - PctRch(CoefZ(cell)))
            
            rPrecip(mon, icrop, pflag, isoil) = rPrecip(mon, icrop, pflag, isoil) + Precip(cell, mon) * acs / 12
            rAcs(mon, icrop, pflag, isoil) = rAcs(mon, icrop, pflag, isoil) + acs
            rAP(mon, icrop, pflag, isoil) = rAP(mon, icrop, pflag, isoil) + AP
            rSWD(mon, icrop, pflag, isoil) = rSWD(mon, icrop, pflag, isoil) + SWD
            rET(mon, icrop, pflag, isoil) = rET(mon, icrop, pflag, isoil) + ET
            rRO(mon, icrop, pflag, isoil) = rRO(mon, icrop, pflag, isoil) + RO
            rDP(mon, icrop, pflag, isoil) = rDP(mon, icrop, pflag, isoil) + DP
            rSL(mon, icrop, pflag, isoil) = rSL(mon, icrop, pflag, isoil) + SL
            rETg(mon, icrop, pflag, isoil) = rETG(mon, icrop, pflag, isoil) + ETG
            rETb(mon, icrop, pflag, isoil) = rETb(mon, icrop, pflag, isoil) + ETb
            rDET1(mon, icrop, pflag, isoil) = rDET1(mon, icrop, pflag, isoil) + DET1
            rDET2(mon, icrop, pflag, isoil) = rDET2(mon, icrop, pflag, isoil) + DET2
            rDP1(mon, icrop, pflag, isoil) = rDP1(mon, icrop, pflag, isoil) + DP1
            rDP2(mon, icrop, pflag, isoil) = rDP2(mon, icrop, pflag, isoil) + DP2
            rRO1(mon, icrop, pflag, isoil) = rRO1(mon, icrop, pflag, isoil) + RO1
            rRO2(mon, icrop, pflag, isoil) = rRO2(mon, icrop, pflag, isoil) + RO2
            rDP2RO(mon, icrop, pflag, isoil) = rDP2RO(mon, icrop, pflag, isoil) + DP2RO 
            rETtrans(mon, icrop, pflag, isoil) = rETtrans(mon, icrop, pflag, isoil) + ETtrans
            rLatseep(mon, icrop, pflag, isoil) = rLatseep(mon, icrop, pflag, isoil) + Latseep
            rSF(mon, icrop, pflag, isoil) = rSF(mon, icrop, pflag, isoil) + SF
            rRO2DP(mon, icrop, pflag, isoil) = rRO2DP(mon, icrop, pflag, isoil) + RO2DP
            rRO2ET(mon, icrop, pflag, isoil) = rRO2ET(mon, icrop, pflag, isoil) + RO2ET
            
            cPrecip(mon, icrop, pflag, isoil, cntyidx(cell)) = cPrecip(mon, icrop, pflag, isoil, cntyidx(cell)) + Precip(cell, mon) * acs / 12
            cAcs(mon, icrop, pflag, isoil, cntyidx(cell)) = cAcs(mon, icrop, pflag, isoil, cntyidx(cell)) + acs
            cAP(mon, icrop, pflag, isoil, cntyidx(cell)) = cAP(mon, icrop, pflag, isoil, cntyidx(cell)) + AP
            cSWD(mon, icrop, pflag, isoil, cntyidx(cell)) = cSWD(mon, icrop, pflag, isoil, cntyidx(cell)) + SWD
            cET(mon, icrop, pflag, isoil, cntyidx(cell)) = cET(mon, icrop, pflag, isoil, cntyidx(cell)) + ET
            cRO(mon, icrop, pflag, isoil, cntyidx(cell)) = cRO(mon, icrop, pflag, isoil, cntyidx(cell)) + RO
            cDP(mon, icrop, pflag, isoil, cntyidx(cell)) = cDP(mon, icrop, pflag, isoil, cntyidx(cell)) + DP
            cSL(mon, icrop, pflag, isoil, cntyidx(cell)) = cSL(mon, icrop, pflag, isoil, cntyidx(cell)) + SL
            cETg(mon, icrop, pflag, isoil, cntyidx(cell)) = cETG(mon, icrop, pflag, isoil, cntyidx(cell)) + ETG
            cETb(mon, icrop, pflag, isoil, cntyidx(cell)) = cETb(mon, icrop, pflag, isoil, cntyidx(cell)) + ETb
            cDET1(mon, icrop, pflag, isoil, cntyidx(cell)) = cDET1(mon, icrop, pflag, isoil, cntyidx(cell)) + DET1
            cDET2(mon, icrop, pflag, isoil, cntyidx(cell)) = cDET2(mon, icrop, pflag, isoil, cntyidx(cell)) + DET2
            cDP1(mon, icrop, pflag, isoil, cntyidx(cell)) = cDP1(mon, icrop, pflag, isoil, cntyidx(cell)) + DP1
            cDP2(mon, icrop, pflag, isoil, cntyidx(cell)) = cDP2(mon, icrop, pflag, isoil, cntyidx(cell)) + DP2
            cRO1(mon, icrop, pflag, isoil, cntyidx(cell)) = cRO1(mon, icrop, pflag, isoil, cntyidx(cell)) + RO1
            cRO2(mon, icrop, pflag, isoil, cntyidx(cell)) = cRO2(mon, icrop, pflag, isoil, cntyidx(cell)) + RO2
            cDP2RO(mon, icrop, pflag, isoil, cntyidx(cell)) = cDP2RO(mon, icrop, pflag, isoil, cntyidx(cell)) + DP2RO 
            cETtrans(mon, icrop, pflag, isoil, cntyidx(cell)) = cETtrans(mon, icrop, pflag, isoil, cntyidx(cell)) + ETtrans
            cLatseep(mon, icrop, pflag, isoil, cntyidx(cell)) = cLatseep(mon, icrop, pflag, isoil, cntyidx(cell)) + Latseep
            cSF(mon, icrop, pflag, isoil, cntyidx(cell)) = cSF(mon, icrop, pflag, isoil, cntyidx(cell)) + SF
            cRO2DP(mon, icrop, pflag, isoil, cntyidx(cell)) = cRO2DP(mon, icrop, pflag, isoil, cntyidx(cell)) + RO2DP
            cRO2ET(mon, icrop, pflag, isoil, cntyidx(cell)) = cRO2ET(mon, icrop, pflag, isoil, cntyidx(cell)) + RO2ET
            
            cfPrecip(mon, icrop, pflag, isoil, iczone) = cfPrecip(mon, icrop, pflag, isoil, iczone) + Precip(cell, mon) * acs / 12
            cfAcs(mon, icrop, pflag, isoil, iczone) = cfAcs(mon, icrop, pflag, isoil, iczone) + acs
            cfAP(mon, icrop, pflag, isoil, iczone) = cfAP(mon, icrop, pflag, isoil, iczone) + AP
            cfSWD(mon, icrop, pflag, isoil, iczone) = cfSWD(mon, icrop, pflag, isoil, iczone) + SWD
            cfET(mon, icrop, pflag, isoil, iczone) = cfET(mon, icrop, pflag, isoil, iczone) + ET
            cfRO(mon, icrop, pflag, isoil, iczone) = cfRO(mon, icrop, pflag, isoil, iczone) + RO
            cfDP(mon, icrop, pflag, isoil, iczone) = cfDP(mon, icrop, pflag, isoil, iczone) + DP
            cfSL(mon, icrop, pflag, isoil, iczone) = cfSL(mon, icrop, pflag, isoil, iczone) + SL
            cfETg(mon, icrop, pflag, isoil, iczone) = cfETG(mon, icrop, pflag, isoil, iczone) + ETG
            cfETb(mon, icrop, pflag, isoil, iczone) = cfETb(mon, icrop, pflag, isoil, iczone) + ETb
            cfDET1(mon, icrop, pflag, isoil, iczone) = cfDET1(mon, icrop, pflag, isoil, iczone) + DET1
            cfDET2(mon, icrop, pflag, isoil, iczone) = cfDET2(mon, icrop, pflag, isoil, iczone) + DET2
            cfDP1(mon, icrop, pflag, isoil, iczone) = cfDP1(mon, icrop, pflag, isoil, iczone) + DP1
            cfDP2(mon, icrop, pflag, isoil, iczone) = cfDP2(mon, icrop, pflag, isoil, iczone) + DP2
            cfRO1(mon, icrop, pflag, isoil, iczone) = cfRO1(mon, icrop, pflag, isoil, iczone) + RO1
            cfRO2(mon, icrop, pflag, isoil, iczone) = cfRO2(mon, icrop, pflag, isoil, iczone) + RO2
            cfDP2RO(mon, icrop, pflag, isoil, iczone) = cfDP2RO(mon, icrop, pflag, isoil, iczone) + DP2RO 
            cfETtrans(mon, icrop, pflag, isoil, iczone) = cfETtrans(mon, icrop, pflag, isoil, iczone) + ETtrans
            cfLatseep(mon, icrop, pflag, isoil, iczone) = cfLatseep(mon, icrop, pflag, isoil, iczone) + Latseep
            cfSF(mon, icrop, pflag, isoil, iczone) = cfSF(mon, icrop, pflag, isoil, iczone) + SF
            cfRO2DP(mon, icrop, pflag, isoil, iczone) = cfRO2DP(mon, icrop, pflag, isoil, iczone) + RO2DP
            cfRO2ET(mon, icrop, pflag, isoil, iczone) = cfRO2ET(mon, icrop, pflag, isoil, iczone) + RO2ET
            
            rzPrecip(mon, icrop, pflag, isoil, irzone) = rzPrecip(mon, icrop, pflag, isoil, irzone) + Precip(cell, mon) * acs / 12
            rzAcs(mon, icrop, pflag, isoil, irzone) = rzAcs(mon, icrop, pflag, isoil, irzone) + acs
            rzAP(mon, icrop, pflag, isoil, irzone) = rzAP(mon, icrop, pflag, isoil, irzone) + AP
            rzSWD(mon, icrop, pflag, isoil, irzone) = rzSWD(mon, icrop, pflag, isoil, irzone) + SWD
            rzET(mon, icrop, pflag, isoil, irzone) = rzET(mon, icrop, pflag, isoil, irzone) + ET
            rzRO(mon, icrop, pflag, isoil, irzone) = rzRO(mon, icrop, pflag, isoil, irzone) + RO
            rzDP(mon, icrop, pflag, isoil, irzone) = rzDP(mon, icrop, pflag, isoil, irzone) + DP
            rzSL(mon, icrop, pflag, isoil, irzone) = rzSL(mon, icrop, pflag, isoil, irzone) + SL
            rzETg(mon, icrop, pflag, isoil, irzone) = rzETG(mon, icrop, pflag, isoil, irzone) + ETG
            rzETb(mon, icrop, pflag, isoil, irzone) = rzETb(mon, icrop, pflag, isoil, irzone) + ETb
            rzDET1(mon, icrop, pflag, isoil, irzone) = rzDET1(mon, icrop, pflag, isoil, irzone) + DET1
            rzDET2(mon, icrop, pflag, isoil, irzone) = rzDET2(mon, icrop, pflag, isoil, irzone) + DET2
            rzDP1(mon, icrop, pflag, isoil, irzone) = rzDP1(mon, icrop, pflag, isoil, irzone) + DP1
            rzDP2(mon, icrop, pflag, isoil, irzone) = rzDP2(mon, icrop, pflag, isoil, irzone) + DP2
            rzRO1(mon, icrop, pflag, isoil, irzone) = rzRO1(mon, icrop, pflag, isoil, irzone) + RO1
            rzRO2(mon, icrop, pflag, isoil, irzone) = rzRO2(mon, icrop, pflag, isoil, irzone) + RO2
            rzDP2RO(mon, icrop, pflag, isoil, irzone) = rzDP2RO(mon, icrop, pflag, isoil, irzone) + DP2RO 
            rzETtrans(mon, icrop, pflag, isoil, irzone) = rzETtrans(mon, icrop, pflag, isoil, irzone) + ETtrans
            rzLatseep(mon, icrop, pflag, isoil, irzone) = rzLatseep(mon, icrop, pflag, isoil, irzone) + Latseep
            rzSF(mon, icrop, pflag, isoil, irzone) = rzSF(mon, icrop, pflag, isoil, irzone) + SF
            rzRO2DP(mon, icrop, pflag, isoil, irzone) = rzRO2DP(mon, icrop, pflag, isoil, irzone) + RO2DP
            rzRO2ET(mon, icrop, pflag, isoil, irzone) = rzRO2ET(mon, icrop, pflag, isoil, irzone) + RO2ET
            
            swdPrecip(mon, icrop, pflag, isoil, swidx) = swdPrecip(mon, icrop, pflag, isoil, swidx) + Precip(cell, mon) * acs / 12
            swdAcs(mon, icrop, pflag, isoil, swidx) = swdAcs(mon, icrop, pflag, isoil, swidx) + acs
            swdAP(mon, icrop, pflag, isoil, swidx) = swdAP(mon, icrop, pflag, isoil, swidx) + AP
            swdSWD(mon, icrop, pflag, isoil, swidx) = swdSWD(mon, icrop, pflag, isoil, swidx) + SWD
            swdET(mon, icrop, pflag, isoil, swidx) = swdET(mon, icrop, pflag, isoil, swidx) + ET
            swdRO(mon, icrop, pflag, isoil, swidx) = swdRO(mon, icrop, pflag, isoil, swidx) + RO
            swdDP(mon, icrop, pflag, isoil, swidx) = swdDP(mon, icrop, pflag, isoil, swidx) + DP
            swdSL(mon, icrop, pflag, isoil, swidx) = swdSL(mon, icrop, pflag, isoil, swidx) + SL
            swdETg(mon, icrop, pflag, isoil, swidx) = swdETG(mon, icrop, pflag, isoil, swidx) + ETG
            swdETb(mon, icrop, pflag, isoil, swidx) = swdETb(mon, icrop, pflag, isoil, swidx) + ETb
            swdDET1(mon, icrop, pflag, isoil, swidx) = swdDET1(mon, icrop, pflag, isoil, swidx) + DET1
            swdDET2(mon, icrop, pflag, isoil, swidx) = swdDET2(mon, icrop, pflag, isoil, swidx) + DET2
            swdDP1(mon, icrop, pflag, isoil, swidx) = swdDP1(mon, icrop, pflag, isoil, swidx) + DP1
            swdDP2(mon, icrop, pflag, isoil, swidx) = swdDP2(mon, icrop, pflag, isoil, swidx) + DP2
            swdRO1(mon, icrop, pflag, isoil, swidx) = swdRO1(mon, icrop, pflag, isoil, swidx) + RO1
            swdRO2(mon, icrop, pflag, isoil, swidx) = swdRO2(mon, icrop, pflag, isoil, swidx) + RO2
            swdDP2RO(mon, icrop, pflag, isoil, swidx) = swdDP2RO(mon, icrop, pflag, isoil, swidx) + DP2RO 
            swdETtrans(mon, icrop, pflag, isoil, swidx) = swdETtrans(mon, icrop, pflag, isoil, swidx) + ETtrans
            swdLatseep(mon, icrop, pflag, isoil, swidx) = swdLatseep(mon, icrop, pflag, isoil, swidx) + Latseep
            swdSF(mon, icrop, pflag, isoil, swidx) = swdSF(mon, icrop, pflag, isoil, swidx) + SF
            swdRO2DP(mon, icrop, pflag, isoil, swidx) = swdRO2DP(mon, icrop, pflag, isoil, swidx) + RO2DP
            swdRO2ET(mon, icrop, pflag, isoil, swidx) = swdRO2ET(mon, icrop, pflag, isoil, swidx) + RO2ET
            
            IF (SWdist .EQ. 12) THEN
                cnppidPrecip(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidPrecip(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + Precip(cell, mon) * acs / 12
                cnppidAcs(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidAcs(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + acs
                cnppidAP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidAP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + AP
                cnppidSWD(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidSWD(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + SWD
                cnppidET(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidET(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + ET
                cnppidRO(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidRO(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + RO
                cnppidDP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DP
                cnppidSL(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidSL(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + SL
                cnppidETg(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidETG(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + ETG
                cnppidETb(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidETb(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + ETb
                cnppidDET1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDET1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DET1
                cnppidDET2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDET2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DET2
                cnppidDP1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDP1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DP1
                cnppidDP2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDP2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DP2
                cnppidRO1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidRO1(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + RO1
                cnppidRO2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidRO2(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + RO2
                cnppidDP2RO(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidDP2RO(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + DP2RO 
                cnppidETtrans(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidETtrans(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + ETtrans
                cnppidLatseep(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidLatseep(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + Latseep
                cnppidSF(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidSF(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + SF
                cnppidRO2DP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidRO2DP(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + RO2DP
                cnppidRO2ET(mon, icrop, pflag, isoil, CNPPIDcell(cell)) = cnppidRO2ET(mon, icrop, pflag, isoil, CNPPIDcell(cell)) + RO2ET
            ELSE
                cnppidPrecip(mon, icrop, pflag, isoil, 0) = cnppidPrecip(mon, icrop, pflag, isoil, 0) + Precip(cell, mon) * acs / 12
                cnppidAcs(mon, icrop, pflag, isoil, 0) = cnppidAcs(mon, icrop, pflag, isoil, 0) + acs
                cnppidAP(mon, icrop, pflag, isoil, 0) = cnppidAP(mon, icrop, pflag, isoil, 0) + AP
                cnppidSWD(mon, icrop, pflag, isoil, 0) = cnppidSWD(mon, icrop, pflag, isoil, 0) + SWD
                cnppidET(mon, icrop, pflag, isoil, 0) = cnppidET(mon, icrop, pflag, isoil, 0) + ET
                cnppidRO(mon, icrop, pflag, isoil, 0) = cnppidRO(mon, icrop, pflag, isoil, 0) + RO
                cnppidDP(mon, icrop, pflag, isoil, 0) = cnppidDP(mon, icrop, pflag, isoil, 0) + DP
                cnppidSL(mon, icrop, pflag, isoil, 0) = cnppidSL(mon, icrop, pflag, isoil, 0) + SL
                cnppidETg(mon, icrop, pflag, isoil, 0) = cnppidETG(mon, icrop, pflag, isoil, 0) + ETG
                cnppidETb(mon, icrop, pflag, isoil, 0) = cnppidETb(mon, icrop, pflag, isoil, 0) + ETb
                cnppidDET1(mon, icrop, pflag, isoil, 0) = cnppidDET1(mon, icrop, pflag, isoil, 0) + DET1
                cnppidDET2(mon, icrop, pflag, isoil, 0) = cnppidDET2(mon, icrop, pflag, isoil, 0) + DET2
                cnppidDP1(mon, icrop, pflag, isoil, 0) = cnppidDP1(mon, icrop, pflag, isoil, 0) + DP1
                cnppidDP2(mon, icrop, pflag, isoil, 0) = cnppidDP2(mon, icrop, pflag, isoil, 0) + DP2
                cnppidRO1(mon, icrop, pflag, isoil, 0) = cnppidRO1(mon, icrop, pflag, isoil, 0) + RO1
                cnppidRO2(mon, icrop, pflag, isoil, 0) = cnppidRO2(mon, icrop, pflag, isoil, 0) + RO2
                cnppidDP2RO(mon, icrop, pflag, isoil, 0) = cnppidDP2RO(mon, icrop, pflag, isoil, 0) + DP2RO 
                cnppidETtrans(mon, icrop, pflag, isoil, 0) = cnppidETtrans(mon, icrop, pflag, isoil, 0) + ETtrans
                cnppidLatseep(mon, icrop, pflag, isoil, 0) = cnppidLatseep(mon, icrop, pflag, isoil, 0) + Latseep
                cnppidSF(mon, icrop, pflag, isoil, 0) = cnppidSF(mon, icrop, pflag, isoil, 0) + SF
                cnppidRO2DP(mon, icrop, pflag, isoil, 0) = cnppidRO2DP(mon, icrop, pflag, isoil, 0) + RO2DP
                cnppidRO2ET(mon, icrop, pflag, isoil, 0) = cnppidRO2ET(mon, icrop, pflag, isoil, 0) + RO2ET
            END IF
            
            uAreaPrecip(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaPrecip(mon, icrop, pflag, isoil, uAreacell(cell)) + Precip(cell, mon) * acs / 12
            uAreaAcs(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaAcs(mon, icrop, pflag, isoil, uAreacell(cell)) + acs
            uAreaAP(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaAP(mon, icrop, pflag, isoil, uAreacell(cell)) + AP
            uAreaSWD(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaSWD(mon, icrop, pflag, isoil, uAreacell(cell)) + SWD
            uAreaET(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaET(mon, icrop, pflag, isoil, uAreacell(cell)) + ET
            uAreaRO(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaRO(mon, icrop, pflag, isoil, uAreacell(cell)) + RO
            uAreaDP(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDP(mon, icrop, pflag, isoil, uAreacell(cell)) + DP
            uAreaSL(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaSL(mon, icrop, pflag, isoil, uAreacell(cell)) + SL
            uAreaETg(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaETG(mon, icrop, pflag, isoil, uAreacell(cell)) + ETG
            uAreaETb(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaETb(mon, icrop, pflag, isoil, uAreacell(cell)) + ETb
            uAreaDET1(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDET1(mon, icrop, pflag, isoil, uAreacell(cell)) + DET1
            uAreaDET2(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDET2(mon, icrop, pflag, isoil, uAreacell(cell)) + DET2
            uAreaDP1(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDP1(mon, icrop, pflag, isoil, uAreacell(cell)) + DP1
            uAreaDP2(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDP2(mon, icrop, pflag, isoil, uAreacell(cell)) + DP2
            uAreaRO1(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaRO1(mon, icrop, pflag, isoil, uAreacell(cell)) + RO1
            uAreaRO2(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaRO2(mon, icrop, pflag, isoil, uAreacell(cell)) + RO2
            uAreaDP2RO(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaDP2RO(mon, icrop, pflag, isoil, uAreacell(cell)) + DP2RO 
            uAreaETtrans(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaETtrans(mon, icrop, pflag, isoil, uAreacell(cell)) + ETtrans
            uAreaLatseep(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaLatseep(mon, icrop, pflag, isoil, uAreacell(cell)) + Latseep
            uAreaSF(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaSF(mon, icrop, pflag, isoil, uAreacell(cell)) + SF
            uAreaRO2DP(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaRO2DP(mon, icrop, pflag, isoil, uAreacell(cell)) + RO2DP
            uAreaRO2ET(mon, icrop, pflag, isoil, uAreacell(cell)) = uAreaRO2ET(mon, icrop, pflag, isoil, uAreacell(cell)) + RO2ET
        
1       END DO
        
        CALL writereports
        
    END DO
    
    CALL closereport

!Added 1/7/2016
600 FORMAT (' Sim Year,  WBP Year,  CNL Year, Misc Year,   MI Year, ')
601 FORMAT (I9, 4(',', I9))
    
END PROGRAM WSPP_Report
!****************************************************************************
SUBROUTINE openreports
    USE PARAM
    IMPLICIT NONE
    
    !Regional
    OPEN (200, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (201, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (202, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (203, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (204, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (205, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (206, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (207, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (210, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (211, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (212, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (213, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (214, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (215, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (216, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (217, FILE = TRIM(OUTDIR)//'Report\Regional\Reg_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (200, 200) 
    IF (Cnlrflg .EQ. 1) WRITE (200, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (200, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (200, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (200, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (200, 234) MunipName(:)
    WRITE (200, 239)
    
    WRITE (201, 201)
    WRITE (202, 202)
    WRITE (203, 203)
    WRITE (204, 204)
    WRITE (205, 205)
    WRITE (206, 206)
    WRITE (207, 207)

    WRITE (210, 200) 
    IF (Cnlrflg .EQ. 1) WRITE (210, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (210, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (210, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (210, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (210, 234) MunipName(:)
    WRITE (210, 239)
    
    WRITE (211, 201)
    WRITE (212, 202)
    WRITE (213, 203)
    WRITE (214, 204)
    WRITE (215, 205)
    WRITE (216, 206)
    WRITE (217, 207)
    
    !County
    OPEN (300, FILE = TRIM(OUTDIR)//'Report\County\cnty_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (301, FILE = TRIM(OUTDIR)//'Report\County\cnty_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (302, FILE = TRIM(OUTDIR)//'Report\County\cnty_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (303, FILE = TRIM(OUTDIR)//'Report\County\cnty_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (304, FILE = TRIM(OUTDIR)//'Report\County\cnty_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (305, FILE = TRIM(OUTDIR)//'Report\County\cnty_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (306, FILE = TRIM(OUTDIR)//'Report\County\cnty_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (307, FILE = TRIM(OUTDIR)//'Report\County\cnty_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (310, FILE = TRIM(OUTDIR)//'Report\County\cnty_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (311, FILE = TRIM(OUTDIR)//'Report\County\cnty_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (312, FILE = TRIM(OUTDIR)//'Report\County\cnty_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (313, FILE = TRIM(OUTDIR)//'Report\County\cnty_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (314, FILE = TRIM(OUTDIR)//'Report\County\cnty_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (315, FILE = TRIM(OUTDIR)//'Report\County\cnty_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (316, FILE = TRIM(OUTDIR)//'Report\County\cnty_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (317, FILE = TRIM(OUTDIR)//'Report\County\cnty_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (300, 300) 
    IF (Cnlrflg .EQ. 1) WRITE (300, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (300, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (300, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (300, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (300, 234) MunipName(:)
    WRITE (300, 239)
    
    WRITE (301, 301)
    WRITE (302, 302)
    WRITE (303, 303)
    WRITE (304, 304)
    WRITE (305, 305)
    WRITE (306, 306)
    WRITE (307, 307)
    
    WRITE (310, 300)
    IF (Cnlrflg .EQ. 1) WRITE (310, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (310, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (310, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (310, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (310, 234) MunipName(:)
    WRITE (310, 239)
    
    WRITE (311, 301)
    WRITE (312, 302)
    WRITE (313, 303)
    WRITE (314, 304)
    WRITE (315, 305)
    WRITE (316, 306)
    WRITE (317, 307)
    
    !CoefZone
    OPEN (400, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (401, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (402, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (403, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (404, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (405, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (406, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (407, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (410, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (411, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (412, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (413, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (414, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (415, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (416, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (417, FILE = TRIM(OUTDIR)//'Report\CoefZone\coefz_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (400, 400)
    IF (Cnlrflg .EQ. 1) WRITE (400, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (400, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (400, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (400, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (400, 234) MunipName(:)
    WRITE (400, 239)
    
    WRITE (401, 401)
    WRITE (402, 402)
    WRITE (403, 403)
    WRITE (404, 404)
    WRITE (405, 405)
    WRITE (406, 406)
    WRITE (407, 407)
    
    WRITE (410, 400)
    IF (Cnlrflg .EQ. 1) WRITE (410, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (410, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (410, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (410, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (410, 234) MunipName(:)
    WRITE (410, 239)
    
    WRITE (411, 401)
    WRITE (412, 402)
    WRITE (413, 403)
    WRITE (414, 404)
    WRITE (415, 405)
    WRITE (416, 406)
    WRITE (417, 407)
    
    !RunoffZone
    OPEN (500, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (501, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (502, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (503, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (504, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (505, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (506, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (507, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (510, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (511, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (512, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (513, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (514, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (515, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (516, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (517, FILE = TRIM(OUTDIR)//'Report\ROZone\ROZ_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (500, 500)
    IF (Cnlrflg .EQ. 1) WRITE (500, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (500, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (500, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (500, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (500, 234) MunipName(:)
    WRITE (500, 239)
    
    WRITE (501, 501)
    WRITE (502, 502)
    WRITE (503, 503)
    WRITE (504, 504)
    WRITE (505, 505)
    WRITE (506, 506)
    WRITE (507, 507)
    
    WRITE (510, 500)
    IF (Cnlrflg .EQ. 1) WRITE (510, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (510, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (510, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (510, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (510, 234) MunipName(:)
    WRITE (510, 239)
    
    WRITE (511, 501)
    WRITE (512, 502)
    WRITE (513, 503)
    WRITE (514, 504)
    WRITE (515, 505)
    WRITE (516, 506)
    WRITE (517, 507)
    
    !SW District
    OPEN (600, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (601, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (602, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (603, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (604, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (605, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (606, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (607, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (610, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (611, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (612, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (613, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (614, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (615, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (616, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (617, FILE = TRIM(OUTDIR)//'Report\SWdist\SWdist_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (600, 600)
    WRITE (601, 601)
    WRITE (602, 602)
    WRITE (603, 603)
    WRITE (604, 604)
    WRITE (605, 605)
    WRITE (606, 606)
    WRITE (607, 607)
    
    WRITE (610, 600)
    WRITE (611, 601)
    WRITE (612, 602)
    WRITE (613, 603)
    WRITE (614, 604)
    WRITE (615, 605)
    WRITE (616, 606)
    WRITE (617, 607)

    !CNPPID Sub-District Area
    OPEN (700, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (701, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (702, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (703, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (704, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (705, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (706, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (707, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (710, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (711, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (712, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (713, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (714, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (715, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (716, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (717, FILE = TRIM(OUTDIR)//'Report\SWdist\SubCNPPID_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (700, 700)
    WRITE (701, 701)
    WRITE (702, 702)
    WRITE (703, 703)
    WRITE (704, 704)
    WRITE (705, 705)
    WRITE (706, 706)
    WRITE (707, 707)
    
    WRITE (710, 700)
    WRITE (711, 701)
    WRITE (712, 702)
    WRITE (713, 703)
    WRITE (714, 704)
    WRITE (715, 705)
    WRITE (716, 706)
    WRITE (717, 707)
    
    !Uareas
    OPEN (800, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Tot_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (801, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Crop_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (802, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Soil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (803, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_IrrS_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (804, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CropSoil_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (805, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CropIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (806, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_SoilIrr_Ann.txt', STATUS = 'UNKNOWN')
    OPEN (807, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CSI_Ann.txt', STATUS = 'UNKNOWN')
    
    OPEN (810, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Tot_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (811, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Crop_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (812, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_Soil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (813, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_IrrS_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (814, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CropSoil_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (815, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CropIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (816, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_SoilIrr_Mon.txt', STATUS = 'UNKNOWN')
    OPEN (817, FILE = TRIM(OUTDIR)//'Report\Uarea\Uarea_CSI_Mon.txt', STATUS = 'UNKNOWN')
    
    WRITE (800, 800) 
    IF (Cnlrflg .EQ. 1) WRITE (800, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (800, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (800, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (800, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (800, 234) MunipName(:)
    WRITE (800, 239)
    
    WRITE (801, 801)
    WRITE (802, 802)
    WRITE (803, 803)
    WRITE (804, 804)
    WRITE (805, 805)
    WRITE (806, 806)
    WRITE (807, 807)

    WRITE (810, 800) 
    IF (Cnlrflg .EQ. 1) WRITE (810, 230) Canalname(:)
    IF (Miscrflg .EQ. 1) WRITE (810, 231) MiscrName(:)
    IF (MIrflg .EQ. 1) WRITE (810, 232) MunirName(:)
    IF (miscpflg .EQ. 1) WRITE (810, 233) MiscpName(:)
    IF (MIpflg .EQ. 1) WRITE (810, 234) MunipName(:)
    WRITE (810, 239)
    
    WRITE (811, 801)
    WRITE (812, 802)
    WRITE (813, 803)
    WRITE (814, 804)
    WRITE (815, 805)
    WRITE (816, 806)
    WRITE (817, 807)
    
!Regional
200 FORMAT ('Year, Month, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET', \)
230 FORMAT (<ncanals>(', ', A<maxcanallen>), \)
231 FORMAT (<nmiscr>(', ', A<maxmiscrlen>), \)
232 FORMAT (<nMunir>(', ', A<maxmunirlen>), \)
233 FORMAT (<nMiscp>(', ', A<maxmiscplen>), \)
234 FORMAT (<nMunip>(', ', A<maxmuniplen>), \)  
239 FORMAT ()
   
201 FORMAT ('Year, Month, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
202 FORMAT ('Year, Month, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
203 FORMAT ('Year, Month, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
204 FORMAT ('Year, Month, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
205 FORMAT ('Year, Month, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
206 FORMAT ('Year, Month, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
207 FORMAT ('Year, Month, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')

!County
300 FORMAT ('Year, Month, County, County_Name, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET', \)
301 FORMAT ('Year, Month, County, County_Name, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
302 FORMAT ('Year, Month, County, County_Name, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
303 FORMAT ('Year, Month, County, County_Name, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
304 FORMAT ('Year, Month, County, County_Name, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
305 FORMAT ('Year, Month, County, County_Name, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
306 FORMAT ('Year, Month, County, County_Name, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
307 FORMAT ('Year, Month, County, County_Name, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')

!Coefficient Zone   
400 FORMAT ('Year, Month, CoefZone, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET', \)
401 FORMAT ('Year, Month, CoefZone, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
402 FORMAT ('Year, Month, CoefZone, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
403 FORMAT ('Year, Month, CoefZone, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
404 FORMAT ('Year, Month, CoefZone, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
405 FORMAT ('Year, Month, CoefZone, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
406 FORMAT ('Year, Month, CoefZone, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
407 FORMAT ('Year, Month, CoefZone, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')

!Runoff Zone
500 FORMAT ('Year, Month, ROZ, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET', \)
501 FORMAT ('Year, Month, ROZ, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
502 FORMAT ('Year, Month, ROZ, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
503 FORMAT ('Year, Month, ROZ, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
504 FORMAT ('Year, Month, ROZ, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
505 FORMAT ('Year, Month, ROZ, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
506 FORMAT ('Year, Month, ROZ, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
507 FORMAT ('Year, Month, ROZ, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
    
!SW district
600 FORMAT ('Year, Month, SWdist, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
601 FORMAT ('Year, Month, SWdist, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
602 FORMAT ('Year, Month, SWdist, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
603 FORMAT ('Year, Month, SWdist, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
604 FORMAT ('Year, Month, SWdist, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
605 FORMAT ('Year, Month, SWdist, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
606 FORMAT ('Year, Month, SWdist, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
607 FORMAT ('Year, Month, SWdist, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
    
!Sub CNPPID districts
700 FORMAT ('Year, Month, CNPPID_Subdist, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
701 FORMAT ('Year, Month, CNPPID_Subdist, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
702 FORMAT ('Year, Month, CNPPID_Subdist, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
703 FORMAT ('Year, Month, CNPPID_Subdist, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
704 FORMAT ('Year, Month, CNPPID_Subdist, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
705 FORMAT ('Year, Month, CNPPID_Subdist, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
706 FORMAT ('Year, Month, CNPPID_Subdist, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
707 FORMAT ('Year, Month, CNPPID_Subdist, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
    
!Uareas
800 FORMAT ('Year, Month, Uarea, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET', \)
801 FORMAT ('Year, Month, Uarea, Crop, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
802 FORMAT ('Year, Month, Uarea, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
803 FORMAT ('Year, Month, Uarea, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
804 FORMAT ('Year, Month, Uarea, Crop, Soil, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
805 FORMAT ('Year, Month, Uarea, Crop, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
806 FORMAT ('Year, Month, Uarea, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
807 FORMAT ('Year, Month, Uarea, Crop, Soil, IrrS, Acres, Precip, GW_Pumping, SW_Del, ET_dir, ROdir, DPdir, SL, ETg, ETb, DET1, DET2, ' &
            'DP1, DP2, RO1, RO2, DP2RO, ETtrans, Lat_Seep, Stream_Flow, RO2DP, RO2ET')
    
END SUBROUTINE openreports
!****************************************************************************
SUBROUTINE writereports
    USE PARAM
    IMPLICIT NONE

!Regional Totals
    DO imon = 0, 12
        wPrecip = SUM(rPrecip(imon,:,:,:))
        wAcs = SUM(rAcs(imon,:,:,:))
        wAP = SUM(rAP(imon,:,:,:))
        wSWD = SUM(rSWD(imon,:,:,:))
        wET = SUM(rET(imon,:,:,:))
        wRO = SUM(rRO(imon,:,:,:))
        wDP = SUM(rDP(imon,:,:,:))
        wSL = SUM(rSL(imon,:,:,:))
        wETg = SUM(rETg(imon,:,:,:))
        wETb = SUM(rETb(imon,:,:,:))
        wDET1 = SUM(rDET1(imon,:,:,:))
        wDET2 = SUM(rDET2(imon,:,:,:))
        wDP1 = SUM(rDP1(imon,:,:,:))
        wDP2 = SUM(rDP2(imon,:,:,:))
        wRO1 = SUM(rRO1(imon,:,:,:))
        wRO2 = SUM(rRO2(imon,:,:,:))
        wDP2RO = SUM(rDP2RO(imon,:,:,:))
        wETtrans = SUM(rETtrans(imon,:,:,:))
        wLatseep = SUM(rLatseep(imon,:,:,:))
        wSF = SUM(rSF(imon,:,:,:))
        wRO2DP = SUM(rRO2DP(imon,:,:,:))
        wRO2ET = SUM(rRO2ET(imon,:,:,:))

        IF (imon .EQ. 0) THEN
            WRITE (200, 210) iyear, imon, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            
            IF (cnlrflg .EQ. 1) THEN
                DO i = 1, ncanals
                    WRITE (200, 240) SUM(rcanalrch(i, :))
                END DO
            END IF
            
            IF (miscpflg .EQ. 1) THEN
                DO i = 1, nmiscr
                    WRITE (200, 240) SUM(rmiscrch(i, :))
                END DO
            END IF
            
            IF (MIrflg .EQ. 1) THEN
                DO i = 1, nmunir
                    WRITE (200, 240) SUM(rMunirch(i, :))
                END DO
            END IF
            
            IF (Miscpflg .EQ. 1) THEN
                DO i = 1, nMiscp
                    WRITE (200, 240) SUM(rMiscpmp(i, :))
                END DO
            END IF
            
            IF (MIpflg .EQ. 1) THEN
                DO i = 1, nMunip
                    WRITE (200, 240) SUM(rMunipmp(i, :))
                END DO
            END IF
            
            WRITE (200, 241)
            
        ELSE
            WRITE (210, 210) iyear, imon, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            
            IF (cnlrflg .EQ. 1) THEN
                DO i = 1, ncanals
                    WRITE (210, 240) rcanalrch(i, imon)
                END DO
            END IF
            
            IF (miscpflg .EQ. 1) THEN
                DO i = 1, nmiscr
                    WRITE (210, 240) rmiscrch(i, imon)
                END DO
            END IF
            
            IF (MIrflg .EQ. 1) THEN
                DO i = 1, nmunir
                    WRITE (210, 240) rMunirch(i, imon)
                END DO
            END IF
            
            IF (Miscpflg .EQ. 1) THEN
                DO i = 1, nMiscp
                    WRITE (210, 240) rMiscpmp(i, imon)
                END DO
            END IF
            
            IF (MIpflg .EQ. 1) THEN
                DO i = 1, nMunip
                    WRITE (210, 240) rMunipmp(i, imon)
                END DO
            END IF
            WRITE (210, 241)
        END IF
    END DO
    
!Regional Crop Totals
    DO icrop = 1, ncrops
        DO imon = 0, 12
            wPrecip = SUM(rPrecip(imon, icrop,:,:))
            wAcs = SUM(rAcs(imon, icrop,:,:))
            wAP = SUM(rAP(imon, icrop,:,:))
            wSWD = SUM(rSWD(imon, icrop,:,:))
            wET = SUM(rET(imon, icrop,:,:))
            wRO = SUM(rRO(imon, icrop,:,:))
            wDP = SUM(rDP(imon, icrop,:,:))
            wSL = SUM(rSL(imon, icrop,:,:))
            wETg = SUM(rETg(imon, icrop,:,:))
            wETb = SUM(rETb(imon, icrop,:,:))
            wDET1 = SUM(rDET1(imon, icrop,:,:))
            wDET2 = SUM(rDET2(imon, icrop,:,:))
            wDP1 = SUM(rDP1(imon, icrop,:,:))
            wDP2 = SUM(rDP2(imon, icrop,:,:))
            wRO1 = SUM(rRO1(imon, icrop,:,:))
            wRO2 = SUM(rRO2(imon, icrop,:,:))
            wDP2RO = SUM(rDP2RO(imon, icrop,:,:))
            wETtrans = SUM(rETtrans(imon, icrop,:,:))
            wLatseep = SUM(rLatseep(imon, icrop,:,:))
            wSF = SUM(rSF(imon, icrop,:,:))
            wRO2DP = SUM(rRO2DP(imon, icrop,:,:))
            wRO2ET = SUM(rRO2ET(imon, icrop,:,:))

            IF (imon .EQ. 0) THEN
                WRITE (201, 211) iyear, imon, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            ELSE
                WRITE (211, 211) iyear, imon, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            END IF
        END DO
    END DO
    
!Regional Soil Totals
    DO isoil = 1, nsoils
        DO imon = 0, 12
            wPrecip = SUM(rPrecip(imon,:,:, isoil))
            wAcs = SUM(rAcs(imon,:,:, isoil))
            wAP = SUM(rAP(imon,:,:, isoil))
            wSWD = SUM(rSWD(imon,:,:, isoil))
            wET = SUM(rET(imon,:,:, isoil))
            wRO = SUM(rRO(imon,:,:, isoil))
            wDP = SUM(rDP(imon,:,:, isoil))
            wSL = SUM(rSL(imon,:,:, isoil))
            wETg = SUM(rETg(imon,:,:, isoil))
            wETb = SUM(rETb(imon,:,:, isoil))
            wDET1 = SUM(rDET1(imon,:,:, isoil))
            wDET2 = SUM(rDET2(imon,:,:, isoil))
            wDP1 = SUM(rDP1(imon,:,:, isoil))
            wDP2 = SUM(rDP2(imon,:,:, isoil))
            wRO1 = SUM(rRO1(imon,:,:, isoil))
            wRO2 = SUM(rRO2(imon,:,:, isoil))
            wDP2RO = SUM(rDP2RO(imon,:,:, isoil))
            wETtrans = SUM(rETtrans(imon,:,:, isoil))
            wLatseep = SUM(rLatseep(imon,:,:, isoil))
            wSF = SUM(rSF(imon,:,:, isoil))
            wRO2DP = SUM(rRO2DP(imon,:,:, isoil))
            wRO2ET = SUM(rRO2ET(imon,:,:, isoil))

            IF (imon .EQ. 0) THEN
                WRITE (202, 212) iyear, imon, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            ELSE
                WRITE (212, 212) iyear, imon, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            END IF
        END DO
    END DO
    
!Regional IrrType Totals
    DO irri = 1, 4
        DO imon = 0, 12
            wPrecip = SUM(rPrecip(imon,:, irri,:))
            wAcs = SUM(rAcs(imon,:, irri,:))
            wAP = SUM(rAP(imon,:, irri,:))
            wSWD = SUM(rSWD(imon,:, irri,:))
            wET = SUM(rET(imon,:, irri,:))
            wRO = SUM(rRO(imon,:, irri,:))
            wDP = SUM(rDP(imon,:, irri,:))
            wSL = SUM(rSL(imon,:, irri,:))
            wETg = SUM(rETg(imon,:, irri,:))
            wETb = SUM(rETb(imon,:, irri,:))
            wDET1 = SUM(rDET1(imon,:, irri,:))
            wDET2 = SUM(rDET2(imon,:, irri,:))
            wDP1 = SUM(rDP1(imon,:, irri,:))
            wDP2 = SUM(rDP2(imon,:, irri,:))
            wRO1 = SUM(rRO1(imon,:, irri,:))
            wRO2 = SUM(rRO2(imon,:, irri,:))
            wDP2RO = SUM(rDP2RO(imon,:, irri,:))
            wETtrans = SUM(rETtrans(imon,:, irri,:))
            wLatseep = SUM(rLatseep(imon,:, irri,:))
            wSF = SUM(rSF(imon,:, irri,:))
            wRO2DP = SUM(rRO2DP(imon,:, irri,:))
            wRO2ET = SUM(rRO2ET(imon,:, irri,:))

            IF (imon .EQ. 0) THEN
                WRITE (203, 213) iyear, imon, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            ELSE
                WRITE (213, 213) iyear, imon, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            END IF
        END DO
    END DO
    
!Regional Crop - Soil Totals
    DO icrop = 1, ncrops
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(rPrecip(imon, icrop,:, isoil))
                wAcs = SUM(rAcs(imon, icrop,:, isoil))
                wAP = SUM(rAP(imon, icrop,:, isoil))
                wSWD = SUM(rSWD(imon, icrop,:, isoil))
                wET = SUM(rET(imon, icrop,:, isoil))
                wRO = SUM(rRO(imon, icrop,:, isoil))
                wDP = SUM(rDP(imon, icrop,:, isoil))
                wSL = SUM(rSL(imon, icrop,:, isoil))
                wETg = SUM(rETg(imon, icrop,:, isoil))
                wETb = SUM(rETb(imon, icrop,:, isoil))
                wDET1 = SUM(rDET1(imon, icrop,:, isoil))
                wDET2 = SUM(rDET2(imon, icrop,:, isoil))
                wDP1 = SUM(rDP1(imon, icrop,:, isoil))
                wDP2 = SUM(rDP2(imon, icrop,:, isoil))
                wRO1 = SUM(rRO1(imon, icrop,:, isoil))
                wRO2 = SUM(rRO2(imon, icrop,:, isoil))
                wDP2RO = SUM(rDP2RO(imon, icrop,:, isoil))
                wETtrans = SUM(rETtrans(imon, icrop,:, isoil))
                wLatseep = SUM(rLatseep(imon, icrop,:, isoil))
                wSF = SUM(rSF(imon, icrop,:, isoil))
                wRO2DP = SUM(rRO2DP(imon, icrop,:, isoil))
                wRO2ET = SUM(rRO2ET(imon, icrop,:, isoil))

                IF (imon .EQ. 0) THEN
                    WRITE (204, 214) iyear, imon, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (214, 214) iyear, imon, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!Regional Crop - IrrSource Totals
    DO icrop = 1, ncrops
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(rPrecip(imon, icrop, irri,:))
                wAcs = SUM(rAcs(imon, icrop, irri,:))
                wAP = SUM(rAP(imon, icrop, irri,:))
                wSWD = SUM(rSWD(imon, icrop, irri,:))
                wET = SUM(rET(imon, icrop, irri,:))
                wRO = SUM(rRO(imon, icrop, irri,:))
                wDP = SUM(rDP(imon, icrop, irri,:))
                wSL = SUM(rSL(imon, icrop, irri,:))
                wETg = SUM(rETg(imon, icrop, irri,:))
                wETb = SUM(rETb(imon, icrop, irri,:))
                wDET1 = SUM(rDET1(imon, icrop, irri,:))
                wDET2 = SUM(rDET2(imon, icrop, irri,:))
                wDP1 = SUM(rDP1(imon, icrop, irri,:))
                wDP2 = SUM(rDP2(imon, icrop, irri,:))
                wRO1 = SUM(rRO1(imon, icrop, irri,:))
                wRO2 = SUM(rRO2(imon, icrop, irri,:))
                wDP2RO = SUM(rDP2RO(imon, icrop, irri,:))
                wETtrans = SUM(rETtrans(imon, icrop, irri,:))
                wLatseep = SUM(rLatseep(imon, icrop, irri,:))
                wSF = SUM(rSF(imon, icrop, irri,:))
                wRO2DP = SUM(rRO2DP(imon, icrop, irri,:))
                wRO2ET = SUM(rRO2ET(imon, icrop, irri,:))

                IF (imon .EQ. 0) THEN
                    WRITE (205, 215) iyear, imon, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (215, 215) iyear, imon, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!Regional Soil - IrrSource Totals
    DO isoil = 1, nsoils
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(rPrecip(imon,:, irri, isoil))
                wAcs = SUM(rAcs(imon,:, irri, isoil))
                wAP = SUM(rAP(imon,:, irri, isoil))
                wSWD = SUM(rSWD(imon,:, irri, isoil))
                wET = SUM(rET(imon,:, irri, isoil))
                wRO = SUM(rRO(imon,:, irri, isoil))
                wDP = SUM(rDP(imon,:, irri, isoil))
                wSL = SUM(rSL(imon,:, irri, isoil))
                wETg = SUM(rETg(imon,:, irri, isoil))
                wETb = SUM(rETb(imon,:, irri, isoil))
                wDET1 = SUM(rDET1(imon,:, irri, isoil))
                wDET2 = SUM(rDET2(imon,:, irri, isoil))
                wDP1 = SUM(rDP1(imon,:, irri, isoil))
                wDP2 = SUM(rDP2(imon,:, irri, isoil))
                wRO1 = SUM(rRO1(imon,:, irri, isoil))
                wRO2 = SUM(rRO2(imon,:, irri, isoil))
                wDP2RO = SUM(rDP2RO(imon,:, irri, isoil))
                wETtrans = SUM(rETtrans(imon,:, irri, isoil))
                wLatseep = SUM(rLatseep(imon,:, irri, isoil))
                wSF = SUM(rSF(imon,:, irri, isoil))
                wRO2DP = SUM(rRO2DP(imon,:, irri, isoil))
                wRO2ET = SUM(rRO2ET(imon,:, irri, isoil))

                IF (imon .EQ. 0) THEN
                    WRITE (206, 216) iyear, imon, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (216, 216) iyear, imon, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!Regional Crop - Soil - IrrSource Totals
    DO icrop = 1, ncrops
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = rPrecip(imon, icrop, irri, isoil)
                    wAcs = rAcs(imon, icrop, irri, isoil)
                    wAP = rAP(imon, icrop, irri, isoil)
                    wSWD = rSWD(imon, icrop, irri, isoil)
                    wET = rET(imon, icrop, irri, isoil)
                    wRO = rRO(imon, icrop, irri, isoil)
                    wDP = rDP(imon, icrop, irri, isoil)
                    wSL = rSL(imon, icrop, irri, isoil)
                    wETg = rETg(imon, icrop, irri, isoil)
                    wETb = rETb(imon, icrop, irri, isoil)
                    wDET1 = rDET1(imon, icrop, irri, isoil)
                    wDET2 = rDET2(imon, icrop, irri, isoil)
                    wDP1 = rDP1(imon, icrop, irri, isoil)
                    wDP2 = rDP2(imon, icrop, irri, isoil)
                    wRO1 = rRO1(imon, icrop, irri, isoil)
                    wRO2 = rRO2(imon, icrop, irri, isoil)
                    wDP2RO = rDP2RO(imon, icrop, irri, isoil)
                    wETtrans = rETtrans(imon, icrop, irri, isoil)
                    wLatseep = rLatseep(imon, icrop, irri, isoil)
                    wSF = rSF(imon, icrop, irri, isoil)
                    wRO2DP = rRO2DP(imon, icrop, irri, isoil)
                    wRO2ET = rRO2ET(imon, icrop, irri, isoil)

                    IF (imon .EQ. 0) THEN
                        WRITE (207, 217) iyear, imon, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (217, 217) iyear, imon, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO

!County Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO imon = 0, 12
                wPrecip = SUM(cPrecip(imon,:,:,:, icounty))
                wAcs = SUM(cAcs(imon,:,:,:, icounty))
                wAP = SUM(cAP(imon,:,:,:, icounty))
                wSWD = SUM(cSWD(imon,:,:,:, icounty))
                wET = SUM(cET(imon,:,:,:, icounty))
                wRO = SUM(cRO(imon,:,:,:, icounty))
                wDP = SUM(cDP(imon,:,:,:, icounty))
                wSL = SUM(cSL(imon,:,:,:, icounty))
                wETg = SUM(cETg(imon,:,:,:, icounty))
                wETb = SUM(cETb(imon,:,:,:, icounty))
                wDET1 = SUM(cDET1(imon,:,:,:, icounty))
                wDET2 = SUM(cDET2(imon,:,:,:, icounty))
                wDP1 = SUM(cDP1(imon,:,:,:, icounty))
                wDP2 = SUM(cDP2(imon,:,:,:, icounty))
                wRO1 = SUM(cRO1(imon,:,:,:, icounty))
                wRO2 = SUM(cRO2(imon,:,:,:, icounty))
                wDP2RO = SUM(cDP2RO(imon,:,:,:, icounty))
                wETtrans = SUM(cETtrans(imon,:,:,:, icounty))
                wLatseep = SUM(cLatseep(imon,:,:,:, icounty))
                wSF = SUM(cSF(imon,:,:,:, icounty))
                wRO2DP = SUM(cRO2DP(imon,:,:,:, icounty))
                wRO2ET = SUM(cRO2ET(imon,:,:,:, icounty))
        
                IF (imon .EQ. 0) THEN
                    WRITE (300, 310) iyear, imon, icounty, CountyName(icounty), wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    
                    IF (cnlrflg .EQ. 1) THEN
                        DO i = 1, ncanals
                            WRITE (300, 240) SUM(ccanalrch(i, icounty, :))
                        END DO
                    END IF
            
                    IF (miscpflg .EQ. 1) THEN
                        DO i = 1, nmiscr
                            WRITE (300, 240) SUM(cmiscrch(i, icounty, :))
                        END DO
                    END IF
            
                    IF (MIrflg .EQ. 1) THEN
                        DO i = 1, nmunir
                            WRITE (300, 240) SUM(cMunirch(i, icounty, :))
                        END DO
                    END IF
            
                    IF (Miscpflg .EQ. 1) THEN
                        DO i = 1, nMiscp
                            WRITE (300, 240) SUM(cMiscpmp(i, icounty, :))
                        END DO
                    END IF
            
                    IF (MIpflg .EQ. 1) THEN
                        DO i = 1, nMunip
                            WRITE (300, 240) SUM(cMunipmp(i, icounty, :))
                        END DO
                    END IF
            
                    WRITE (300, 241)                    
                    
                ELSE
                    
                    WRITE (310, 310) iyear, imon, icounty, CountyName(icounty), wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    
                    IF (cnlrflg .EQ. 1) THEN
                        DO i = 1, ncanals
                            WRITE (310, 240) ccanalrch(i, icounty, imon)
                        END DO
                    END IF
            
                    IF (miscpflg .EQ. 1) THEN
                        DO i = 1, nmiscr
                            WRITE (310, 240) cmiscrch(i, icounty, imon)
                        END DO
                    END IF
            
                    IF (MIrflg .EQ. 1) THEN
                        DO i = 1, nmunir
                            WRITE (310, 240) cMunirch(i, icounty, imon)
                        END DO
                    END IF
            
                    IF (Miscpflg .EQ. 1) THEN
                        DO i = 1, nMiscp
                            WRITE (310, 240) cMiscpmp(i, icounty, imon)
                        END DO
                    END IF
            
                    IF (MIpflg .EQ. 1) THEN
                        DO i = 1, nMunip
                            WRITE (310, 240) cMunipmp(i, icounty, imon)
                        END DO
                    END IF
            
                    WRITE (310, 241)
                    
                END IF
            END DO
        END IF
    END DO

!County Crop Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO icrop = 1, ncrops
                DO imon = 0, 12
                    wPrecip = SUM(cPrecip(imon, icrop,:,:, icounty))
                    wAcs = SUM(cAcs(imon, icrop,:,:, icounty))
                    wAP = SUM(cAP(imon, icrop,:,:, icounty))
                    wSWD = SUM(cSWD(imon, icrop,:,:, icounty))
                    wET = SUM(cET(imon, icrop,:,:, icounty))
                    wRO = SUM(cRO(imon, icrop,:,:, icounty))
                    wDP = SUM(cDP(imon, icrop,:,:, icounty))
                    wSL = SUM(cSL(imon, icrop,:,:, icounty))
                    wETg = SUM(cETg(imon, icrop,:,:, icounty))
                    wETb = SUM(cETb(imon, icrop,:,:, icounty))
                    wDET1 = SUM(cDET1(imon, icrop,:,:, icounty))
                    wDET2 = SUM(cDET2(imon, icrop,:,:, icounty))
                    wDP1 = SUM(cDP1(imon, icrop,:,:, icounty))
                    wDP2 = SUM(cDP2(imon, icrop,:,:, icounty))
                    wRO1 = SUM(cRO1(imon, icrop,:,:, icounty))
                    wRO2 = SUM(cRO2(imon, icrop,:,:, icounty))
                    wDP2RO = SUM(cDP2RO(imon, icrop,:,:, icounty))
                    wETtrans = SUM(cETtrans(imon, icrop,:,:, icounty))
                    wLatseep = SUM(cLatseep(imon, icrop,:,:, icounty))
                    wSF = SUM(cSF(imon, icrop,:,:, icounty))
                    wRO2DP = SUM(cRO2DP(imon, icrop,:,:, icounty))
                    wRO2ET = SUM(cRO2ET(imon, icrop,:,:, icounty))
    
                    IF (imon .EQ. 0) THEN
                        WRITE (301, 311) iyear, imon, icounty, CountyName(icounty), icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (311, 311) iyear, imon, icounty, CountyName(icounty), icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END IF
    END DO
    
!County Soil Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(cPrecip(imon,:,:, isoil, icounty))
                    wAcs = SUM(cAcs(imon,:,:, isoil, icounty))
                    wAP = SUM(cAP(imon,:,:, isoil, icounty))
                    wSWD = SUM(cSWD(imon,:,:, isoil, icounty))
                    wET = SUM(cET(imon,:,:, isoil, icounty))
                    wRO = SUM(cRO(imon,:,:, isoil, icounty))
                    wDP = SUM(cDP(imon,:,:, isoil, icounty))
                    wSL = SUM(cSL(imon,:,:, isoil, icounty))
                    wETg = SUM(cETg(imon,:,:, isoil, icounty))
                    wETb = SUM(cETb(imon,:,:, isoil, icounty))
                    wDET1 = SUM(cDET1(imon,:,:, isoil, icounty))
                    wDET2 = SUM(cDET2(imon,:,:, isoil, icounty))
                    wDP1 = SUM(cDP1(imon,:,:, isoil, icounty))
                    wDP2 = SUM(cDP2(imon,:,:, isoil, icounty))
                    wRO1 = SUM(cRO1(imon,:,:, isoil, icounty))
                    wRO2 = SUM(cRO2(imon,:,:, isoil, icounty))
                    wDP2RO = SUM(cDP2RO(imon,:,:, isoil, icounty))
                    wETtrans = SUM(cETtrans(imon,:,:, isoil, icounty))
                    wLatseep = SUM(cLatseep(imon,:,:, isoil, icounty))
                    wSF = SUM(cSF(imon,:,:, isoil, icounty))
                    wRO2DP = SUM(cRO2DP(imon,:,:, isoil, icounty))
                    wRO2ET = SUM(cRO2ET(imon,:,:, isoil, icounty))
    
                    IF (imon .EQ. 0) THEN
                        WRITE (302, 312) iyear, imon, icounty, CountyName(icounty), isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (312, 312) iyear, imon, icounty, CountyName(icounty), isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END IF
    END DO
    
!County IrrType Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(cPrecip(imon,:, irri,:, icounty))
                    wAcs = SUM(cAcs(imon,:, irri,:, icounty))
                    wAP = SUM(cAP(imon,:, irri,:, icounty))
                    wSWD = SUM(cSWD(imon,:, irri,:, icounty))
                    wET = SUM(cET(imon,:, irri,:, icounty))
                    wRO = SUM(cRO(imon,:, irri,:, icounty))
                    wDP = SUM(cDP(imon,:, irri,:, icounty))
                    wSL = SUM(cSL(imon,:, irri,:, icounty))
                    wETg = SUM(cETg(imon,:, irri,:, icounty))
                    wETb = SUM(cETb(imon,:, irri,:, icounty))
                    wDET1 = SUM(cDET1(imon,:, irri,:, icounty))
                    wDET2 = SUM(cDET2(imon,:, irri,:, icounty))
                    wDP1 = SUM(cDP1(imon,:, irri,:, icounty))
                    wDP2 = SUM(cDP2(imon,:, irri,:, icounty))
                    wRO1 = SUM(cRO1(imon,:, irri,:, icounty))
                    wRO2 = SUM(cRO2(imon,:, irri,:, icounty))
                    wDP2RO = SUM(cDP2RO(imon,:, irri,:, icounty))
                    wETtrans = SUM(cETtrans(imon,:, irri,:, icounty))
                    wLatseep = SUM(cLatseep(imon,:, irri,:, icounty))
                    wSF = SUM(cSF(imon,:, irri,:, icounty))
                    wRO2DP = SUM(cRO2DP(imon,:, irri,:, icounty))
                    wRO2ET = SUM(cRO2ET(imon,:, irri,:, icounty))
        
                    IF (imon .EQ. 0) THEN
                        WRITE (303, 313) iyear, imon, icounty, CountyName(icounty), irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE    
                        WRITE (313, 313) iyear, imon, icounty, CountyName(icounty), irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END IF
    END DO
    
!County Crop - Soil Totals
     DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO icrop = 1, ncrops
                DO isoil = 1, nsoils
                    DO imon = 0, 12
                        wPrecip = SUM(cPrecip(imon, icrop,:, isoil, icounty))
                        wAcs = SUM(cAcs(imon, icrop,:, isoil, icounty))
                        wAP = SUM(cAP(imon, icrop,:, isoil, icounty))
                        wSWD = SUM(cSWD(imon, icrop,:, isoil, icounty))
                        wET = SUM(cET(imon, icrop,:, isoil, icounty))
                        wRO = SUM(cRO(imon, icrop,:, isoil, icounty))
                        wDP = SUM(cDP(imon, icrop,:, isoil, icounty))
                        wSL = SUM(cSL(imon, icrop,:, isoil, icounty))
                        wETg = SUM(cETg(imon, icrop,:, isoil, icounty))
                        wETb = SUM(cETb(imon, icrop,:, isoil, icounty))
                        wDET1 = SUM(cDET1(imon, icrop,:, isoil, icounty))
                        wDET2 = SUM(cDET2(imon, icrop,:, isoil, icounty))
                        wDP1 = SUM(cDP1(imon, icrop,:, isoil, icounty))
                        wDP2 = SUM(cDP2(imon, icrop,:, isoil, icounty))
                        wRO1 = SUM(cRO1(imon, icrop,:, isoil, icounty))
                        wRO2 = SUM(cRO2(imon, icrop,:, isoil, icounty))
                        wDP2RO = SUM(cDP2RO(imon, icrop,:, isoil, icounty))
                        wETtrans = SUM(cETtrans(imon, icrop,:, isoil, icounty))
                        wLatseep = SUM(cLatseep(imon, icrop,:, isoil, icounty))
                        wSF = SUM(cSF(imon, icrop,:, isoil, icounty))
                        wRO2DP = SUM(cRO2DP(imon, icrop,:, isoil, icounty))
                        wRO2ET = SUM(cRO2ET(imon, icrop,:, isoil, icounty))

                        IF (imon .EQ. 0) THEN
                            WRITE (304, 314) iyear, imon, icounty, CountyName(icounty), icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (314, 314) iyear, imon, icounty, CountyName(icounty), icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END IF
    END DO
    
!County Crop - IrrSource Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO icrop = 1, ncrops
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = SUM(cPrecip(imon, icrop, irri,:, icounty))
                        wAcs = SUM(cAcs(imon, icrop, irri,:, icounty))
                        wAP = SUM(cAP(imon, icrop, irri,:, icounty))
                        wSWD = SUM(cSWD(imon, icrop, irri,:, icounty))
                        wET = SUM(cET(imon, icrop, irri,:, icounty))
                        wRO = SUM(cRO(imon, icrop, irri,:, icounty))
                        wDP = SUM(cDP(imon, icrop, irri,:, icounty))
                        wSL = SUM(cSL(imon, icrop, irri,:, icounty))
                        wETg = SUM(cETg(imon, icrop, irri,:, icounty))
                        wETb = SUM(cETb(imon, icrop, irri,:, icounty))
                        wDET1 = SUM(cDET1(imon, icrop, irri,:, icounty))
                        wDET2 = SUM(cDET2(imon, icrop, irri,:, icounty))
                        wDP1 = SUM(cDP1(imon, icrop, irri,:, icounty))
                        wDP2 = SUM(cDP2(imon, icrop, irri,:, icounty))
                        wRO1 = SUM(cRO1(imon, icrop, irri,:, icounty))
                        wRO2 = SUM(cRO2(imon, icrop, irri,:, icounty))
                        wDP2RO = SUM(cDP2RO(imon, icrop, irri,:, icounty))
                        wETtrans = SUM(cETtrans(imon, icrop, irri,:, icounty))
                        wLatseep = SUM(cLatseep(imon, icrop, irri,:, icounty))
                        wSF = SUM(cSF(imon, icrop, irri,:, icounty))
                        wRO2DP = SUM(cRO2DP(imon, icrop, irri,:, icounty))
                        wRO2ET = SUM(cRO2ET(imon, icrop, irri,:, icounty))
    
                        IF (imon .EQ. 0) THEN
                            WRITE (305, 315) iyear, imon, icounty, CountyName(icounty), icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (315, 315) iyear, imon, icounty, CountyName(icounty), icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END IF
    END DO
    
!County Soil - IrrSource Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = SUM(cPrecip(imon,:, irri, isoil, icounty))
                        wAcs = SUM(cAcs(imon,:, irri, isoil, icounty))
                        wAP = SUM(cAP(imon,:, irri, isoil, icounty))
                        wSWD = SUM(cSWD(imon,:, irri, isoil, icounty))
                        wET = SUM(cET(imon,:, irri, isoil, icounty))
                        wRO = SUM(cRO(imon,:, irri, isoil, icounty))
                        wDP = SUM(cDP(imon,:, irri, isoil, icounty))
                        wSL = SUM(cSL(imon,:, irri, isoil, icounty))
                        wETg = SUM(cETg(imon,:, irri, isoil, icounty))
                        wETb = SUM(cETb(imon,:, irri, isoil, icounty))
                        wDET1 = SUM(cDET1(imon,:, irri, isoil, icounty))
                        wDET2 = SUM(cDET2(imon,:, irri, isoil, icounty))
                        wDP1 = SUM(cDP1(imon,:, irri, isoil, icounty))
                        wDP2 = SUM(cDP2(imon,:, irri, isoil, icounty))
                        wRO1 = SUM(cRO1(imon,:, irri, isoil, icounty))
                        wRO2 = SUM(cRO2(imon,:, irri, isoil, icounty))
                        wDP2RO = SUM(cDP2RO(imon,:, irri, isoil, icounty))
                        wETtrans = SUM(cETtrans(imon,:, irri, isoil, icounty))
                        wLatseep = SUM(cLatseep(imon,:, irri, isoil, icounty))
                        wSF = SUM(cSF(imon,:, irri, isoil, icounty))
                        wRO2DP = SUM(cRO2DP(imon,:, irri, isoil, icounty))
                        wRO2ET = SUM(cRO2ET(imon,:, irri, isoil, icounty))
    
                        IF (imon .EQ. 0) THEN
                            WRITE (306, 316) iyear, imon, icounty, CountyName(icounty), isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (316, 316) iyear, imon, icounty, CountyName(icounty), isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END IF
    END DO
    
!County Crop - Soil - IrrSource Totals
    DO icounty = 1, ncounty
        IF (Activecounty(icounty) .EQ. 1) THEN
            DO icrop = 1, ncrops
                DO isoil = 1, nsoils
                    DO irri = 1, 4
                        DO imon = 0, 12
                            wPrecip = cPrecip(imon, icrop, irri, isoil, icounty)
                            wAcs = cAcs(imon, icrop, irri, isoil, icounty)
                            wAP = cAP(imon, icrop, irri, isoil, icounty)
                            wSWD = cSWD(imon, icrop, irri, isoil, icounty)
                            wET = cET(imon, icrop, irri, isoil, icounty)
                            wRO = cRO(imon, icrop, irri, isoil, icounty)
                            wDP = cDP(imon, icrop, irri, isoil, icounty)
                            wSL = cSL(imon, icrop, irri, isoil, icounty)
                            wETg = cETg(imon, icrop, irri, isoil, icounty)
                            wETb = cETb(imon, icrop, irri, isoil, icounty)
                            wDET1 = cDET1(imon, icrop, irri, isoil, icounty)
                            wDET2 = cDET2(imon, icrop, irri, isoil, icounty)
                            wDP1 = cDP1(imon, icrop, irri, isoil, icounty)
                            wDP2 = cDP2(imon, icrop, irri, isoil, icounty)
                            wRO1 = cRO1(imon, icrop, irri, isoil, icounty)
                            wRO2 = cRO2(imon, icrop, irri, isoil, icounty)
                            wDP2RO = cDP2RO(imon, icrop, irri, isoil, icounty)
                            wETtrans = cETtrans(imon, icrop, irri, isoil, icounty)
                            wLatseep = cLatseep(imon, icrop, irri, isoil, icounty)
                            wSF = cSF(imon, icrop, irri, isoil, icounty)
                            wRO2DP = cRO2DP(imon, icrop, irri, isoil, icounty)
                            wRO2ET = cRO2ET(imon, icrop, irri, isoil, icounty)
        
                            IF (imon .EQ. 0) THEN
                                WRITE (307, 317) iyear, imon, icounty, CountyName(icounty), icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                            ELSE
                                WRITE (317, 317) iyear, imon, icounty, CountyName(icounty), icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                            END IF
                        END DO
                    END DO
                END DO
            END DO
        END IF
    END DO

!CoefZone Totals
    DO izone = 0, nczones
        DO imon = 0, 12
            wPrecip = SUM(cfPrecip(imon,:,:,:, izone))
            wAcs = SUM(cfAcs(imon,:,:,:, izone))
            wAP = SUM(cfAP(imon,:,:,:, izone))
            wSWD = SUM(cfSWD(imon,:,:,:, izone))
            wET = SUM(cfET(imon,:,:,:, izone))
            wRO = SUM(cfRO(imon,:,:,:, izone))
            wDP = SUM(cfDP(imon,:,:,:, izone))
            wSL = SUM(cfSL(imon,:,:,:, izone))
            wETg = SUM(cfETg(imon,:,:,:, izone))
            wETb = SUM(cfETb(imon,:,:,:, izone))
            wDET1 = SUM(cfDET1(imon,:,:,:, izone))
            wDET2 = SUM(cfDET2(imon,:,:,:, izone))
            wDP1 = SUM(cfDP1(imon,:,:,:, izone))
            wDP2 = SUM(cfDP2(imon,:,:,:, izone))
            wRO1 = SUM(cfRO1(imon,:,:,:, izone))
            wRO2 = SUM(cfRO2(imon,:,:,:, izone))
            wDP2RO = SUM(cfDP2RO(imon,:,:,:, izone))
            wETtrans = SUM(cfETtrans(imon,:,:,:, izone))
            wLatseep = SUM(cfLatseep(imon,:,:,:, izone))
            wSF = SUM(cfSF(imon,:,:,:, izone))
            wRO2DP = SUM(cfRO2DP(imon,:,:,:, izone))
            wRO2ET = SUM(cfRO2ET(imon,:,:,:, izone))
    
            IF (imon .EQ. 0) THEN
                WRITE (400, 410) iyear, imon, izone, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (400, 240) SUM(cfcanalrch(i, izone, :))
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (400, 240) SUM(cfmiscrch(i, izone, :))
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (400, 240) SUM(cfMunirch(i, izone, :))
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (400, 240) SUM(cfMiscpmp(i, izone, :))
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (400, 240) SUM(cfMunipmp(i, izone, :))
                    END DO
                END IF
            
                WRITE (400, 241)
                
            ELSE
                WRITE (410, 410) iyear, imon, izone, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (410, 240) cfcanalrch(i, izone, imon)
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (410, 240) cfmiscrch(i, izone, imon)
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (410, 240) cfMunirch(i, izone, imon)
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (410, 240) cfMiscpmp(i, izone, imon)
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (410, 240) cfMunipmp(i, izone, imon)
                    END DO
                END IF
            
                WRITE (410, 241)
                
            END IF
        END DO
    END DO

!CoefZone Crop Totals
    DO izone = 0, nczones
        DO icrop = 1, ncrops
            DO imon = 0, 12
                wPrecip = SUM(cfPrecip(imon, icrop,:,:, izone))
                wAcs = SUM(cfAcs(imon, icrop,:,:, izone))
                wAP = SUM(cfAP(imon, icrop,:,:, izone))
                wSWD = SUM(cfSWD(imon, icrop,:,:, izone))
                wET = SUM(cfET(imon, icrop,:,:, izone))
                wRO = SUM(cfRO(imon, icrop,:,:, izone))
                wDP = SUM(cfDP(imon, icrop,:,:, izone))
                wSL = SUM(cfSL(imon, icrop,:,:, izone))
                wETg = SUM(cfETg(imon, icrop,:,:, izone))
                wETb = SUM(cfETb(imon, icrop,:,:, izone))
                wDET1 = SUM(cfDET1(imon, icrop,:,:, izone))
                wDET2 = SUM(cfDET2(imon, icrop,:,:, izone))
                wDP1 = SUM(cfDP1(imon, icrop,:,:, izone))
                wDP2 = SUM(cfDP2(imon, icrop,:,:, izone))
                wRO1 = SUM(cfRO1(imon, icrop,:,:, izone))
                wRO2 = SUM(cfRO2(imon, icrop,:,:, izone))
                wDP2RO = SUM(cfDP2RO(imon, icrop,:,:, izone))
                wETtrans = SUM(cfETtrans(imon, icrop,:,:, izone))
                wLatseep = SUM(cfLatseep(imon, icrop,:,:, izone))
                wSF = SUM(cfSF(imon, icrop,:,:, izone))
                wRO2DP = SUM(cfRO2DP(imon, icrop,:,:, izone))
                wRO2ET = SUM(cfRO2ET(imon, icrop,:,:, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (401, 411) iyear, imon, izone, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (411, 411) iyear, imon, izone, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!CoefZone Soil Totals
    DO izone = 0, nczones
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(cfPrecip(imon,:,:, isoil, izone))
                wAcs = SUM(cfAcs(imon,:,:, isoil, izone))
                wAP = SUM(cfAP(imon,:,:, isoil, izone))
                wSWD = SUM(cfSWD(imon,:,:, isoil, izone))
                wET = SUM(cfET(imon,:,:, isoil, izone))
                wRO = SUM(cfRO(imon,:,:, isoil, izone))
                wDP = SUM(cfDP(imon,:,:, isoil, izone))
                wSL = SUM(cfSL(imon,:,:, isoil, izone))
                wETg = SUM(cfETg(imon,:,:, isoil, izone))
                wETb = SUM(cfETb(imon,:,:, isoil, izone))
                wDET1 = SUM(cfDET1(imon,:,:, isoil, izone))
                wDET2 = SUM(cfDET2(imon,:,:, isoil, izone))
                wDP1 = SUM(cfDP1(imon,:,:, isoil, izone))
                wDP2 = SUM(cfDP2(imon,:,:, isoil, izone))
                wRO1 = SUM(cfRO1(imon,:,:, isoil, izone))
                wRO2 = SUM(cfRO2(imon,:,:, isoil, izone))
                wDP2RO = SUM(cfDP2RO(imon,:,:, isoil, izone))
                wETtrans = SUM(cfETtrans(imon,:,:, isoil, izone))
                wLatseep = SUM(cfLatseep(imon,:,:, isoil, izone))
                wSF = SUM(cfSF(imon,:,:, isoil, izone))
                wRO2DP = SUM(cfRO2DP(imon,:,:, isoil, izone))
                wRO2ET = SUM(cfRO2ET(imon,:,:, isoil, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (402, 412) iyear, imon, izone, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (412, 412) iyear, imon, izone, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!CoefZone IrrType Totals
    DO izone = 0, nczones
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(cfPrecip(imon,:, irri,:, izone))
                wAcs = SUM(cfAcs(imon,:, irri,:, izone))
                wAP = SUM(cfAP(imon,:, irri,:, izone))
                wSWD = SUM(cfSWD(imon,:, irri,:, izone))
                wET = SUM(cfET(imon,:, irri,:, izone))
                wRO = SUM(cfRO(imon,:, irri,:, izone))
                wDP = SUM(cfDP(imon,:, irri,:, izone))
                wSL = SUM(cfSL(imon,:, irri,:, izone))
                wETg = SUM(cfETg(imon,:, irri,:, izone))
                wETb = SUM(cfETb(imon,:, irri,:, izone))
                wDET1 = SUM(cfDET1(imon,:, irri,:, izone))
                wDET2 = SUM(cfDET2(imon,:, irri,:, izone))
                wDP1 = SUM(cfDP1(imon,:, irri,:, izone))
                wDP2 = SUM(cfDP2(imon,:, irri,:, izone))
                wRO1 = SUM(cfRO1(imon,:, irri,:, izone))
                wRO2 = SUM(cfRO2(imon,:, irri,:, izone))
                wDP2RO = SUM(cfDP2RO(imon,:, irri,:, izone))
                wETtrans = SUM(cfETtrans(imon,:, irri,:, izone))
                wLatseep = SUM(cfLatseep(imon,:, irri,:, izone))
                wSF = SUM(cfSF(imon,:, irri,:, izone))
                wRO2DP = SUM(cfRO2DP(imon,:, irri,:, izone))
                wRO2ET = SUM(cfRO2ET(imon,:, irri,:, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (403, 413) iyear, imon, izone, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (413, 413) iyear, imon, izone, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!CoefZone Crop - Soil Totals
    DO izone = 0, nczones
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(cfPrecip(imon, icrop,:, isoil, izone))
                    wAcs = SUM(cfAcs(imon, icrop,:, isoil, izone))
                    wAP = SUM(cfAP(imon, icrop,:, isoil, izone))
                    wSWD = SUM(cfSWD(imon, icrop,:, isoil, izone))
                    wET = SUM(cfET(imon, icrop,:, isoil, izone))
                    wRO = SUM(cfRO(imon, icrop,:, isoil, izone))
                    wDP = SUM(cfDP(imon, icrop,:, isoil, izone))
                    wSL = SUM(cfSL(imon, icrop,:, isoil, izone))
                    wETg = SUM(cfETg(imon, icrop,:, isoil, izone))
                    wETb = SUM(cfETb(imon, icrop,:, isoil, izone))
                    wDET1 = SUM(cfDET1(imon, icrop,:, isoil, izone))
                    wDET2 = SUM(cfDET2(imon, icrop,:, isoil, izone))
                    wDP1 = SUM(cfDP1(imon, icrop,:, isoil, izone))
                    wDP2 = SUM(cfDP2(imon, icrop,:, isoil, izone))
                    wRO1 = SUM(cfRO1(imon, icrop,:, isoil, izone))
                    wRO2 = SUM(cfRO2(imon, icrop,:, isoil, izone))
                    wDP2RO = SUM(cfDP2RO(imon, icrop,:, isoil, izone))
                    wETtrans = SUM(cfETtrans(imon, icrop,:, isoil, izone))
                    wLatseep = SUM(cfLatseep(imon, icrop,:, isoil, izone))
                    wSF = SUM(cfSF(imon, icrop,:, isoil, izone))
                    wRO2DP = SUM(cfRO2DP(imon, icrop,:, isoil, izone))
                    wRO2ET = SUM(cfRO2ET(imon, icrop,:, isoil, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (404, 414) iyear, imon, izone, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (414, 414) iyear, imon, izone, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!CoefZone Crop - IrrSource Totals
    DO izone = 0, nczones
        DO icrop = 1, ncrops
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(cfPrecip(imon, icrop, irri,:, izone))
                    wAcs = SUM(cfAcs(imon, icrop, irri,:, izone))
                    wAP = SUM(cfAP(imon, icrop, irri,:, izone))
                    wSWD = SUM(cfSWD(imon, icrop, irri,:, izone))
                    wET = SUM(cfET(imon, icrop, irri,:, izone))
                    wRO = SUM(cfRO(imon, icrop, irri,:, izone))
                    wDP = SUM(cfDP(imon, icrop, irri,:, izone))
                    wSL = SUM(cfSL(imon, icrop, irri,:, izone))
                    wETg = SUM(cfETg(imon, icrop, irri,:, izone))
                    wETb = SUM(cfETb(imon, icrop, irri,:, izone))
                    wDET1 = SUM(cfDET1(imon, icrop, irri,:, izone))
                    wDET2 = SUM(cfDET2(imon, icrop, irri,:, izone))
                    wDP1 = SUM(cfDP1(imon, icrop, irri,:, izone))
                    wDP2 = SUM(cfDP2(imon, icrop, irri,:, izone))
                    wRO1 = SUM(cfRO1(imon, icrop, irri,:, izone))
                    wRO2 = SUM(cfRO2(imon, icrop, irri,:, izone))
                    wDP2RO = SUM(cfDP2RO(imon, icrop, irri,:, izone))
                    wETtrans = SUM(cfETtrans(imon, icrop, irri,:, izone))
                    wLatseep = SUM(cfLatseep(imon, icrop, irri,:, izone))
                    wSF = SUM(cfSF(imon, icrop, irri,:, izone))
                    wRO2DP = SUM(cfRO2DP(imon, icrop, irri,:, izone))
                    wRO2ET = SUM(cfRO2ET(imon, icrop, irri,:, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (405, 415) iyear, imon, izone, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (415, 415) iyear, imon, izone, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!CoefZone Soil - IrrSource Totals
    DO izone = 0, nczones
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(cfPrecip(imon,:, irri, isoil, izone))
                    wAcs = SUM(cfAcs(imon,:, irri, isoil, izone))
                    wAP = SUM(cfAP(imon,:, irri, isoil, izone))
                    wSWD = SUM(cfSWD(imon,:, irri, isoil, izone))
                    wET = SUM(cfET(imon,:, irri, isoil, izone))
                    wRO = SUM(cfRO(imon,:, irri, isoil, izone))
                    wDP = SUM(cfDP(imon,:, irri, isoil, izone))
                    wSL = SUM(cfSL(imon,:, irri, isoil, izone))
                    wETg = SUM(cfETg(imon,:, irri, isoil, izone))
                    wETb = SUM(cfETb(imon,:, irri, isoil, izone))
                    wDET1 = SUM(cfDET1(imon,:, irri, isoil, izone))
                    wDET2 = SUM(cfDET2(imon,:, irri, isoil, izone))
                    wDP1 = SUM(cfDP1(imon,:, irri, isoil, izone))
                    wDP2 = SUM(cfDP2(imon,:, irri, isoil, izone))
                    wRO1 = SUM(cfRO1(imon,:, irri, isoil, izone))
                    wRO2 = SUM(cfRO2(imon,:, irri, isoil, izone))
                    wDP2RO = SUM(cfDP2RO(imon,:, irri, isoil, izone))
                    wETtrans = SUM(cfETtrans(imon,:, irri, isoil, izone))
                    wLatseep = SUM(cfLatseep(imon,:, irri, isoil, izone))
                    wSF = SUM(cfSF(imon,:, irri, isoil, izone))
                    wRO2DP = SUM(cfRO2DP(imon,:, irri, isoil, izone))
                    wRO2ET = SUM(cfRO2ET(imon,:, irri, isoil, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (406, 416) iyear, imon, izone, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (416, 416) iyear, imon, izone, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!CoefZone Crop - Soil - IrrSource Totals
    DO izone = 0, nczones
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = cfPrecip(imon, icrop, irri, isoil, izone)
                        wAcs = cfAcs(imon, icrop, irri, isoil, izone)
                        wAP = cfAP(imon, icrop, irri, isoil, izone)
                        wSWD = cfSWD(imon, icrop, irri, isoil, izone)
                        wET = cfET(imon, icrop, irri, isoil, izone)
                        wRO = cfRO(imon, icrop, irri, isoil, izone)
                        wDP = cfDP(imon, icrop, irri, isoil, izone)
                        wSL = cfSL(imon, icrop, irri, isoil, izone)
                        wETg = cfETg(imon, icrop, irri, isoil, izone)
                        wETb = cfETb(imon, icrop, irri, isoil, izone)
                        wDET1 = cfDET1(imon, icrop, irri, isoil, izone)
                        wDET2 = cfDET2(imon, icrop, irri, isoil, izone)
                        wDP1 = cfDP1(imon, icrop, irri, isoil, izone)
                        wDP2 = cfDP2(imon, icrop, irri, isoil, izone)
                        wRO1 = cfRO1(imon, icrop, irri, isoil, izone)
                        wRO2 = cfRO2(imon, icrop, irri, isoil, izone)
                        wDP2RO = cfDP2RO(imon, icrop, irri, isoil, izone)
                        wETtrans = cfETtrans(imon, icrop, irri, isoil, izone)
                        wLatseep = cfLatseep(imon, icrop, irri, isoil, izone)
                        wSF = cfSF(imon, icrop, irri, isoil, izone)
                        wRO2DP = cfRO2DP(imon, icrop, irri, isoil, izone)
                        wRO2ET = cfRO2ET(imon, icrop, irri, isoil, izone)
    
                        IF (imon .EQ. 0) THEN
                            WRITE (407, 417) iyear, imon, izone, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (417, 417) iyear, imon, izone, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END DO
    END DO

!ROzone Totals
    DO izone = 0, nrzones
        DO imon = 0, 12
            wPrecip = SUM(rzPrecip(imon,:,:,:, izone))
            wAcs = SUM(rzAcs(imon,:,:,:, izone))
            wAP = SUM(rzAP(imon,:,:,:, izone))
            wSWD = SUM(rzSWD(imon,:,:,:, izone))
            wET = SUM(rzET(imon,:,:,:, izone))
            wRO = SUM(rzRO(imon,:,:,:, izone))
            wDP = SUM(rzDP(imon,:,:,:, izone))
            wSL = SUM(rzSL(imon,:,:,:, izone))
            wETg = SUM(rzETg(imon,:,:,:, izone))
            wETb = SUM(rzETb(imon,:,:,:, izone))
            wDET1 = SUM(rzDET1(imon,:,:,:, izone))
            wDET2 = SUM(rzDET2(imon,:,:,:, izone))
            wDP1 = SUM(rzDP1(imon,:,:,:, izone))
            wDP2 = SUM(rzDP2(imon,:,:,:, izone))
            wRO1 = SUM(rzRO1(imon,:,:,:, izone))
            wRO2 = SUM(rzRO2(imon,:,:,:, izone))
            wDP2RO = SUM(rzDP2RO(imon,:,:,:, izone))
            wETtrans = SUM(rzETtrans(imon,:,:,:, izone))
            wLatseep = SUM(rzLatseep(imon,:,:,:, izone))
            wSF = SUM(rzSF(imon,:,:,:, izone))
            wRO2DP = SUM(rzRO2DP(imon,:,:,:, izone))
            wRO2ET = SUM(rzRO2ET(imon,:,:,:, izone))
    
            IF (imon .EQ. 0) THEN
                WRITE (500, 510) iyear, imon, izone, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (500, 240) SUM(rzcanalrch(i, izone, :))
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (500, 240) SUM(rzmiscrch(i, izone, :))
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (500, 240) SUM(rzMunirch(i, izone, :))
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (500, 240) SUM(rzMiscpmp(i, izone, :))
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (500, 240) SUM(rzMunipmp(i, izone, :))
                    END DO
                END IF
            
                WRITE (500, 241)
            
            ELSE
                WRITE (510, 510) iyear, imon, izone, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (510, 240) rzcanalrch(i, izone, imon)
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (510, 240) rzmiscrch(i, izone, imon)
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (510, 240) rzMunirch(i, izone, imon)
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (510, 240) rzMiscpmp(i, izone, imon)
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (510, 240) rzMunipmp(i, izone, imon)
                    END DO
                END IF
            
                WRITE (510, 241)
                
            END IF
        END DO
    END DO
    
!ROzone Crop Totals
    DO izone = 0, nrzones
        DO icrop = 1, ncrops
            DO imon = 0, 12
                wPrecip = SUM(rzPrecip(imon, icrop,:,:, izone))
                wAcs = SUM(rzAcs(imon, icrop,:,:, izone))
                wAP = SUM(rzAP(imon, icrop,:,:, izone))
                wSWD = SUM(rzSWD(imon, icrop,:,:, izone))
                wET = SUM(rzET(imon, icrop,:,:, izone))
                wRO = SUM(rzRO(imon, icrop,:,:, izone))
                wDP = SUM(rzDP(imon, icrop,:,:, izone))
                wSL = SUM(rzSL(imon, icrop,:,:, izone))
                wETg = SUM(rzETg(imon, icrop,:,:, izone))
                wETb = SUM(rzETb(imon, icrop,:,:, izone))
                wDET1 = SUM(rzDET1(imon, icrop,:,:, izone))
                wDET2 = SUM(rzDET2(imon, icrop,:,:, izone))
                wDP1 = SUM(rzDP1(imon, icrop,:,:, izone))
                wDP2 = SUM(rzDP2(imon, icrop,:,:, izone))
                wRO1 = SUM(rzRO1(imon, icrop,:,:, izone))
                wRO2 = SUM(rzRO2(imon, icrop,:,:, izone))
                wDP2RO = SUM(rzDP2RO(imon, icrop,:,:, izone))
                wETtrans = SUM(rzETtrans(imon, icrop,:,:, izone))
                wLatseep = SUM(rzLatseep(imon, icrop,:,:, izone))
                wSF = SUM(rzSF(imon, icrop,:,:, izone))
                wRO2DP = SUM(rzRO2DP(imon, icrop,:,:, izone))
                wRO2ET = SUM(rzRO2ET(imon, icrop,:,:, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (501, 511) iyear, imon, izone, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (511, 511) iyear, imon, izone, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!ROzone Soil Totals
    DO izone = 0, nrzones
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(rzPrecip(imon,:,:, isoil, izone))
                wAcs = SUM(rzAcs(imon,:,:, isoil, izone))
                wAP = SUM(rzAP(imon,:,:, isoil, izone))
                wSWD = SUM(rzSWD(imon,:,:, isoil, izone))
                wET = SUM(rzET(imon,:,:, isoil, izone))
                wRO = SUM(rzRO(imon,:,:, isoil, izone))
                wDP = SUM(rzDP(imon,:,:, isoil, izone))
                wSL = SUM(rzSL(imon,:,:, isoil, izone))
                wETg = SUM(rzETg(imon,:,:, isoil, izone))
                wETb = SUM(rzETb(imon,:,:, isoil, izone))
                wDET1 = SUM(rzDET1(imon,:,:, isoil, izone))
                wDET2 = SUM(rzDET2(imon,:,:, isoil, izone))
                wDP1 = SUM(rzDP1(imon,:,:, isoil, izone))
                wDP2 = SUM(rzDP2(imon,:,:, isoil, izone))
                wRO1 = SUM(rzRO1(imon,:,:, isoil, izone))
                wRO2 = SUM(rzRO2(imon,:,:, isoil, izone))
                wDP2RO = SUM(rzDP2RO(imon,:,:, isoil, izone))
                wETtrans = SUM(rzETtrans(imon,:,:, isoil, izone))
                wLatseep = SUM(rzLatseep(imon,:,:, isoil, izone))
                wSF = SUM(rzSF(imon,:,:, isoil, izone))
                wRO2DP = SUM(rzRO2DP(imon,:,:, isoil, izone))
                wRO2ET = SUM(rzRO2ET(imon,:,:, isoil, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (502, 512) iyear, imon, izone, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (512, 512) iyear, imon, izone, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!ROzone IrrType Totals
    DO izone = 0, nrzones
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(rzPrecip(imon,:, irri,:, izone))
                wAcs = SUM(rzAcs(imon,:, irri,:, izone))
                wAP = SUM(rzAP(imon,:, irri,:, izone))
                wSWD = SUM(rzSWD(imon,:, irri,:, izone))
                wET = SUM(rzET(imon,:, irri,:, izone))
                wRO = SUM(rzRO(imon,:, irri,:, izone))
                wDP = SUM(rzDP(imon,:, irri,:, izone))
                wSL = SUM(rzSL(imon,:, irri,:, izone))
                wETg = SUM(rzETg(imon,:, irri,:, izone))
                wETb = SUM(rzETb(imon,:, irri,:, izone))
                wDET1 = SUM(rzDET1(imon,:, irri,:, izone))
                wDET2 = SUM(rzDET2(imon,:, irri,:, izone))
                wDP1 = SUM(rzDP1(imon,:, irri,:, izone))
                wDP2 = SUM(rzDP2(imon,:, irri,:, izone))
                wRO1 = SUM(rzRO1(imon,:, irri,:, izone))
                wRO2 = SUM(rzRO2(imon,:, irri,:, izone))
                wDP2RO = SUM(rzDP2RO(imon,:, irri,:, izone))
                wETtrans = SUM(rzETtrans(imon,:, irri,:, izone))
                wLatseep = SUM(rzLatseep(imon,:, irri,:, izone))
                wSF = SUM(rzSF(imon,:, irri,:, izone))
                wRO2DP = SUM(rzRO2DP(imon,:, irri,:, izone))
                wRO2ET = SUM(rzRO2ET(imon,:, irri,:, izone))
    
                IF (imon .EQ. 0) THEN
                    WRITE (503, 513) iyear, imon, izone, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (513, 513) iyear, imon, izone, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!ROzone Crop - Soil Totals
    DO izone = 0, nrzones
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(rzPrecip(imon, icrop,:, isoil, izone))
                    wAcs = SUM(rzAcs(imon, icrop,:, isoil, izone))
                    wAP = SUM(rzAP(imon, icrop,:, isoil, izone))
                    wSWD = SUM(rzSWD(imon, icrop,:, isoil, izone))
                    wET = SUM(rzET(imon, icrop,:, isoil, izone))
                    wRO = SUM(rzRO(imon, icrop,:, isoil, izone))
                    wDP = SUM(rzDP(imon, icrop,:, isoil, izone))
                    wSL = SUM(rzSL(imon, icrop,:, isoil, izone))
                    wETg = SUM(rzETg(imon, icrop,:, isoil, izone))
                    wETb = SUM(rzETb(imon, icrop,:, isoil, izone))
                    wDET1 = SUM(rzDET1(imon, icrop,:, isoil, izone))
                    wDET2 = SUM(rzDET2(imon, icrop,:, isoil, izone))
                    wDP1 = SUM(rzDP1(imon, icrop,:, isoil, izone))
                    wDP2 = SUM(rzDP2(imon, icrop,:, isoil, izone))
                    wRO1 = SUM(rzRO1(imon, icrop,:, isoil, izone))
                    wRO2 = SUM(rzRO2(imon, icrop,:, isoil, izone))
                    wDP2RO = SUM(rzDP2RO(imon, icrop,:, isoil, izone))
                    wETtrans = SUM(rzETtrans(imon, icrop,:, isoil, izone))
                    wLatseep = SUM(rzLatseep(imon, icrop,:, isoil, izone))
                    wSF = SUM(rzSF(imon, icrop,:, isoil, izone))
                    wRO2DP = SUM(rzRO2DP(imon, icrop,:, isoil, izone))
                    wRO2ET = SUM(rzRO2ET(imon, icrop,:, isoil, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (504, 514) iyear, imon, izone, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (514, 514) iyear, imon, izone, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Crop - IrrSource Totals
    DO izone = 0, nrzones
        DO icrop = 1, ncrops
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(rzPrecip(imon, icrop, irri,:, izone))
                    wAcs = SUM(rzAcs(imon, icrop, irri,:, izone))
                    wAP = SUM(rzAP(imon, icrop, irri,:, izone))
                    wSWD = SUM(rzSWD(imon, icrop, irri,:, izone))
                    wET = SUM(rzET(imon, icrop, irri,:, izone))
                    wRO = SUM(rzRO(imon, icrop, irri,:, izone))
                    wDP = SUM(rzDP(imon, icrop, irri,:, izone))
                    wSL = SUM(rzSL(imon, icrop, irri,:, izone))
                    wETg = SUM(rzETg(imon, icrop, irri,:, izone))
                    wETb = SUM(rzETb(imon, icrop, irri,:, izone))
                    wDET1 = SUM(rzDET1(imon, icrop, irri,:, izone))
                    wDET2 = SUM(rzDET2(imon, icrop, irri,:, izone))
                    wDP1 = SUM(rzDP1(imon, icrop, irri,:, izone))
                    wDP2 = SUM(rzDP2(imon, icrop, irri,:, izone))
                    wRO1 = SUM(rzRO1(imon, icrop, irri,:, izone))
                    wRO2 = SUM(rzRO2(imon, icrop, irri,:, izone))
                    wDP2RO = SUM(rzDP2RO(imon, icrop, irri,:, izone))
                    wETtrans = SUM(rzETtrans(imon, icrop, irri,:, izone))
                    wLatseep = SUM(rzLatseep(imon, icrop, irri,:, izone))
                    wSF = SUM(rzSF(imon, icrop, irri,:, izone))
                    wRO2DP = SUM(rzRO2DP(imon, icrop, irri,:, izone))
                    wRO2ET = SUM(rzRO2ET(imon, icrop, irri,:, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (505, 515) iyear, imon, izone, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (515, 515) iyear, imon, izone, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Soil - IrrSource Totals
    DO izone = 0, nrzones
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(rzPrecip(imon,:, irri, isoil, izone))
                    wAcs = SUM(rzAcs(imon,:, irri, isoil, izone))
                    wAP = SUM(rzAP(imon,:, irri, isoil, izone))
                    wSWD = SUM(rzSWD(imon,:, irri, isoil, izone))
                    wET = SUM(rzET(imon,:, irri, isoil, izone))
                    wRO = SUM(rzRO(imon,:, irri, isoil, izone))
                    wDP = SUM(rzDP(imon,:, irri, isoil, izone))
                    wSL = SUM(rzSL(imon,:, irri, isoil, izone))
                    wETg = SUM(rzETg(imon,:, irri, isoil, izone))
                    wETb = SUM(rzETb(imon,:, irri, isoil, izone))
                    wDET1 = SUM(rzDET1(imon,:, irri, isoil, izone))
                    wDET2 = SUM(rzDET2(imon,:, irri, isoil, izone))
                    wDP1 = SUM(rzDP1(imon,:, irri, isoil, izone))
                    wDP2 = SUM(rzDP2(imon,:, irri, isoil, izone))
                    wRO1 = SUM(rzRO1(imon,:, irri, isoil, izone))
                    wRO2 = SUM(rzRO2(imon,:, irri, isoil, izone))
                    wDP2RO = SUM(rzDP2RO(imon,:, irri, isoil, izone))
                    wETtrans = SUM(rzETtrans(imon,:, irri, isoil, izone))
                    wLatseep = SUM(rzLatseep(imon,:, irri, isoil, izone))
                    wSF = SUM(rzSF(imon,:, irri, isoil, izone))
                    wRO2DP = SUM(rzRO2DP(imon,:, irri, isoil, izone))
                    wRO2ET = SUM(rzRO2ET(imon,:, irri, isoil, izone))

                    IF (imon .EQ. 0) THEN
                        WRITE (506, 516) iyear, imon, izone, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (516, 516) iyear, imon, izone, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Crop - Soil - IrrSource Totals
    DO izone = 0, nrzones
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = rzPrecip(imon, icrop, irri, isoil, izone)
                        wAcs = rzAcs(imon, icrop, irri, isoil, izone)
                        wAP = rzAP(imon, icrop, irri, isoil, izone)
                        wSWD = rzSWD(imon, icrop, irri, isoil, izone)
                        wET = rzET(imon, icrop, irri, isoil, izone)
                        wRO = rzRO(imon, icrop, irri, isoil, izone)
                        wDP = rzDP(imon, icrop, irri, isoil, izone)
                        wSL = rzSL(imon, icrop, irri, isoil, izone)
                        wETg = rzETg(imon, icrop, irri, isoil, izone)
                        wETb = rzETb(imon, icrop, irri, isoil, izone)
                        wDET1 = rzDET1(imon, icrop, irri, isoil, izone)
                        wDET2 = rzDET2(imon, icrop, irri, isoil, izone)
                        wDP1 = rzDP1(imon, icrop, irri, isoil, izone)
                        wDP2 = rzDP2(imon, icrop, irri, isoil, izone)
                        wRO1 = rzRO1(imon, icrop, irri, isoil, izone)
                        wRO2 = rzRO2(imon, icrop, irri, isoil, izone)
                        wDP2RO = rzDP2RO(imon, icrop, irri, isoil, izone)
                        wETtrans = rzETtrans(imon, icrop, irri, isoil, izone)
                        wLatseep = rzLatseep(imon, icrop, irri, isoil, izone)
                        wSF = rzSF(imon, icrop, irri, isoil, izone)
                        wRO2DP = rzRO2DP(imon, icrop, irri, isoil, izone)
                        wRO2ET = rzRO2ET(imon, icrop, irri, isoil, izone)
    
                        IF (imon .EQ. 0) THEN
                            WRITE (507, 517) iyear, imon, izone, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (517, 517) iyear, imon, izone, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END DO
    END DO

!SW district Totals
    DO irrd = 0, nirrdist
        DO imon = 0, 12
            wPrecip = SUM(swdPrecip(imon,:,:,:, irrd))
            wAcs = SUM(swdAcs(imon,:,:,:, irrd))
            wAP = SUM(swdAP(imon,:,:,:, irrd))
            wSWD = SUM(swdSWD(imon,:,:,:, irrd))
            wET = SUM(swdET(imon,:,:,:, irrd))
            wRO = SUM(swdRO(imon,:,:,:, irrd))
            wDP = SUM(swdDP(imon,:,:,:, irrd))
            wSL = SUM(swdSL(imon,:,:,:, irrd))
            wETg = SUM(swdETg(imon,:,:,:, irrd))
            wETb = SUM(swdETb(imon,:,:,:, irrd))
            wDET1 = SUM(swdDET1(imon,:,:,:, irrd))
            wDET2 = SUM(swdDET2(imon,:,:,:, irrd))
            wDP1 = SUM(swdDP1(imon,:,:,:, irrd))
            wDP2 = SUM(swdDP2(imon,:,:,:, irrd))
            wRO1 = SUM(swdRO1(imon,:,:,:, irrd))
            wRO2 = SUM(swdRO2(imon,:,:,:, irrd))
            wDP2RO = SUM(swdDP2RO(imon,:,:,:, irrd))
            wETtrans = SUM(swdETtrans(imon,:,:,:, irrd))
            wLatseep = SUM(swdLatseep(imon,:,:,:, irrd))
            wSF = SUM(swdSF(imon,:,:,:, irrd))
            wRO2DP = SUM(swdRO2DP(imon,:,:,:, irrd))
            wRO2ET = SUM(swdRO2ET(imon,:,:,:, irrd))
    
            IF (imon .EQ. 0) THEN
                WRITE (600, 610) iyear, imon, SWDistx(irrd), wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            ELSE
                WRITE (610, 610) iyear, imon, SWDistx(irrd), wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            END IF
        END DO
    END DO
    
!swdistrict Crop Totals
    DO irrd = 0, nirrdist
        DO icrop = 1, ncrops
            DO imon = 0, 12
                wPrecip = SUM(swdPrecip(imon, icrop,:,:, irrd))
                wAcs = SUM(swdAcs(imon, icrop,:,:, irrd))
                wAP = SUM(swdAP(imon, icrop,:,:, irrd))
                wSWD = SUM(swdSWD(imon, icrop,:,:, irrd))
                wET = SUM(swdET(imon, icrop,:,:, irrd))
                wRO = SUM(swdRO(imon, icrop,:,:, irrd))
                wDP = SUM(swdDP(imon, icrop,:,:, irrd))
                wSL = SUM(swdSL(imon, icrop,:,:, irrd))
                wETg = SUM(swdETg(imon, icrop,:,:, irrd))
                wETb = SUM(swdETb(imon, icrop,:,:, irrd))
                wDET1 = SUM(swdDET1(imon, icrop,:,:, irrd))
                wDET2 = SUM(swdDET2(imon, icrop,:,:, irrd))
                wDP1 = SUM(swdDP1(imon, icrop,:,:, irrd))
                wDP2 = SUM(swdDP2(imon, icrop,:,:, irrd))
                wRO1 = SUM(swdRO1(imon, icrop,:,:, irrd))
                wRO2 = SUM(swdRO2(imon, icrop,:,:, irrd))
                wDP2RO = SUM(swdDP2RO(imon, icrop,:,:, irrd))
                wETtrans = SUM(swdETtrans(imon, icrop,:,:, irrd))
                wLatseep = SUM(swdLatseep(imon, icrop,:,:, irrd))
                wSF = SUM(swdSF(imon, icrop,:,:, irrd))
                wRO2DP = SUM(swdRO2DP(imon, icrop,:,:, irrd))
                wRO2ET = SUM(swdRO2ET(imon, icrop,:,:, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (601, 611) iyear, imon, SWDistx(irrd), icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (611, 611) iyear, imon, SWDistx(irrd), icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW district Soil Totals
    DO irrd = 0, nirrdist
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(swdPrecip(imon,:,:, isoil, irrd))
                wAcs = SUM(swdAcs(imon,:,:, isoil, irrd))
                wAP = SUM(swdAP(imon,:,:, isoil, irrd))
                wSWD = SUM(swdSWD(imon,:,:, isoil, irrd))
                wET = SUM(swdET(imon,:,:, isoil, irrd))
                wRO = SUM(swdRO(imon,:,:, isoil, irrd))
                wDP = SUM(swdDP(imon,:,:, isoil, irrd))
                wSL = SUM(swdSL(imon,:,:, isoil, irrd))
                wETg = SUM(swdETg(imon,:,:, isoil, irrd))
                wETb = SUM(swdETb(imon,:,:, isoil, irrd))
                wDET1 = SUM(swdDET1(imon,:,:, isoil, irrd))
                wDET2 = SUM(swdDET2(imon,:,:, isoil, irrd))
                wDP1 = SUM(swdDP1(imon,:,:, isoil, irrd))
                wDP2 = SUM(swdDP2(imon,:,:, isoil, irrd))
                wRO1 = SUM(swdRO1(imon,:,:, isoil, irrd))
                wRO2 = SUM(swdRO2(imon,:,:, isoil, irrd))
                wDP2RO = SUM(swdDP2RO(imon,:,:, isoil, irrd))
                wETtrans = SUM(swdETtrans(imon,:,:, isoil, irrd))
                wLatseep = SUM(swdLatseep(imon,:,:, isoil, irrd))
                wSF = SUM(swdSF(imon,:,:, isoil, irrd))
                wRO2DP = SUM(swdRO2DP(imon,:,:, isoil, irrd))
                wRO2ET = SUM(swdRO2ET(imon,:,:, isoil, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (602, 612) iyear, imon, SWDistx(irrd), isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (612, 612) iyear, imon, SWDistx(irrd), isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW district IrrType Totals
    DO irrd = 0, nirrdist
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(swdPrecip(imon,:, irri,:, irrd))
                wAcs = SUM(swdAcs(imon,:, irri,:, irrd))
                wAP = SUM(swdAP(imon,:, irri,:, irrd))
                wSWD = SUM(swdSWD(imon,:, irri,:, irrd))
                wET = SUM(swdET(imon,:, irri,:, irrd))
                wRO = SUM(swdRO(imon,:, irri,:, irrd))
                wDP = SUM(swdDP(imon,:, irri,:, irrd))
                wSL = SUM(swdSL(imon,:, irri,:, irrd))
                wETg = SUM(swdETg(imon,:, irri,:, irrd))
                wETb = SUM(swdETb(imon,:, irri,:, irrd))
                wDET1 = SUM(swdDET1(imon,:, irri,:, irrd))
                wDET2 = SUM(swdDET2(imon,:, irri,:, irrd))
                wDP1 = SUM(swdDP1(imon,:, irri,:, irrd))
                wDP2 = SUM(swdDP2(imon,:, irri,:, irrd))
                wRO1 = SUM(swdRO1(imon,:, irri,:, irrd))
                wRO2 = SUM(swdRO2(imon,:, irri,:, irrd))
                wDP2RO = SUM(swdDP2RO(imon,:, irri,:, irrd))
                wETtrans = SUM(swdETtrans(imon,:, irri,:, irrd))
                wLatseep = SUM(swdLatseep(imon,:, irri,:, irrd))
                wSF = SUM(swdSF(imon,:, irri,:, irrd))
                wRO2DP = SUM(swdRO2DP(imon,:, irri,:, irrd))
                wRO2ET = SUM(swdRO2ET(imon,:, irri,:, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (603, 613) iyear, imon, SWDistx(irrd), irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (613, 613) iyear, imon, SWDistx(irrd), irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW District Crop - Soil Totals
    DO irrd = 0, nirrdist
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(swdPrecip(imon, icrop,:, isoil, irrd))
                    wAcs = SUM(swdAcs(imon, icrop,:, isoil, irrd))
                    wAP = SUM(swdAP(imon, icrop,:, isoil, irrd))
                    wSWD = SUM(swdSWD(imon, icrop,:, isoil, irrd))
                    wET = SUM(swdET(imon, icrop,:, isoil, irrd))
                    wRO = SUM(swdRO(imon, icrop,:, isoil, irrd))
                    wDP = SUM(swdDP(imon, icrop,:, isoil, irrd))
                    wSL = SUM(swdSL(imon, icrop,:, isoil, irrd))
                    wETg = SUM(swdETg(imon, icrop,:, isoil, irrd))
                    wETb = SUM(swdETb(imon, icrop,:, isoil, irrd))
                    wDET1 = SUM(swdDET1(imon, icrop,:, isoil, irrd))
                    wDET2 = SUM(swdDET2(imon, icrop,:, isoil, irrd))
                    wDP1 = SUM(swdDP1(imon, icrop,:, isoil, irrd))
                    wDP2 = SUM(swdDP2(imon, icrop,:, isoil, irrd))
                    wRO1 = SUM(swdRO1(imon, icrop,:, isoil, irrd))
                    wRO2 = SUM(swdRO2(imon, icrop,:, isoil, irrd))
                    wDP2RO = SUM(swdDP2RO(imon, icrop,:, isoil, irrd))
                    wETtrans = SUM(swdETtrans(imon, icrop,:, isoil, irrd))
                    wLatseep = SUM(swdLatseep(imon, icrop,:, isoil, irrd))
                    wSF = SUM(swdSF(imon, icrop,:, isoil, irrd))
                    wRO2DP = SUM(swdRO2DP(imon, icrop,:, isoil, irrd))
                    wRO2ET = SUM(swdRO2ET(imon, icrop,:, isoil, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (604, 614) iyear, imon, SWDistx(irrd), icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (614, 614) iyear, imon, SWDistx(irrd), icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW District Crop - IrrSource Totals
    DO irrd = 0, nirrdist
        DO icrop = 1, ncrops
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(swdPrecip(imon, icrop, irri,:, irrd))
                    wAcs = SUM(swdAcs(imon, icrop, irri,:, irrd))
                    wAP = SUM(swdAP(imon, icrop, irri,:, irrd))
                    wSWD = SUM(swdSWD(imon, icrop, irri,:, irrd))
                    wET = SUM(swdET(imon, icrop, irri,:, irrd))
                    wRO = SUM(swdRO(imon, icrop, irri,:, irrd))
                    wDP = SUM(swdDP(imon, icrop, irri,:, irrd))
                    wSL = SUM(swdSL(imon, icrop, irri,:, irrd))
                    wETg = SUM(swdETg(imon, icrop, irri,:, irrd))
                    wETb = SUM(swdETb(imon, icrop, irri,:, irrd))
                    wDET1 = SUM(swdDET1(imon, icrop, irri,:, irrd))
                    wDET2 = SUM(swdDET2(imon, icrop, irri,:, irrd))
                    wDP1 = SUM(swdDP1(imon, icrop, irri,:, irrd))
                    wDP2 = SUM(swdDP2(imon, icrop, irri,:, irrd))
                    wRO1 = SUM(swdRO1(imon, icrop, irri,:, irrd))
                    wRO2 = SUM(swdRO2(imon, icrop, irri,:, irrd))
                    wDP2RO = SUM(swdDP2RO(imon, icrop, irri,:, irrd))
                    wETtrans = SUM(swdETtrans(imon, icrop, irri,:, irrd))
                    wLatseep = SUM(swdLatseep(imon, icrop, irri,:, irrd))
                    wSF = SUM(swdSF(imon, icrop, irri,:, irrd))
                    wRO2DP = SUM(swdRO2DP(imon, icrop, irri,:, irrd))
                    wRO2ET = SUM(swdRO2ET(imon, icrop, irri,:, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (605, 615) iyear, imon, SWDistx(irrd), icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (615, 615) iyear, imon, SWDistx(irrd), icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW district Soil - IrrSource Totals
    DO irrd = 0, nirrdist
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(swdPrecip(imon,:, irri, isoil, irrd))
                    wAcs = SUM(swdAcs(imon,:, irri, isoil, irrd))
                    wAP = SUM(swdAP(imon,:, irri, isoil, irrd))
                    wSWD = SUM(swdSWD(imon,:, irri, isoil, irrd))
                    wET = SUM(swdET(imon,:, irri, isoil, irrd))
                    wRO = SUM(swdRO(imon,:, irri, isoil, irrd))
                    wDP = SUM(swdDP(imon,:, irri, isoil, irrd))
                    wSL = SUM(swdSL(imon,:, irri, isoil, irrd))
                    wETg = SUM(swdETg(imon,:, irri, isoil, irrd))
                    wETb = SUM(swdETb(imon,:, irri, isoil, irrd))
                    wDET1 = SUM(swdDET1(imon,:, irri, isoil, irrd))
                    wDET2 = SUM(swdDET2(imon,:, irri, isoil, irrd))
                    wDP1 = SUM(swdDP1(imon,:, irri, isoil, irrd))
                    wDP2 = SUM(swdDP2(imon,:, irri, isoil, irrd))
                    wRO1 = SUM(swdRO1(imon,:, irri, isoil, irrd))
                    wRO2 = SUM(swdRO2(imon,:, irri, isoil, irrd))
                    wDP2RO = SUM(swdDP2RO(imon,:, irri, isoil, irrd))
                    wETtrans = SUM(swdETtrans(imon,:, irri, isoil, irrd))
                    wLatseep = SUM(swdLatseep(imon,:, irri, isoil, irrd))
                    wSF = SUM(swdSF(imon,:, irri, isoil, irrd))
                    wRO2DP = SUM(swdRO2DP(imon,:, irri, isoil, irrd))
                    wRO2ET = SUM(swdRO2ET(imon,:, irri, isoil, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (606, 616) iyear, imon, SWDistx(irrd), isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (616, 616) iyear, imon, SWDistx(irrd), isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW District Crop - Soil - IrrSource Totals
    DO irrd = 0, nirrdist
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = swdPrecip(imon, icrop, irri, isoil, irrd)
                        wAcs = swdAcs(imon, icrop, irri, isoil, irrd)
                        wAP = swdAP(imon, icrop, irri, isoil, irrd)
                        wSWD = swdSWD(imon, icrop, irri, isoil, irrd)
                        wET = swdET(imon, icrop, irri, isoil, irrd)
                        wRO = swdRO(imon, icrop, irri, isoil, irrd)
                        wDP = swdDP(imon, icrop, irri, isoil, irrd)
                        wSL = swdSL(imon, icrop, irri, isoil, irrd)
                        wETg = swdETg(imon, icrop, irri, isoil, irrd)
                        wETb = swdETb(imon, icrop, irri, isoil, irrd)
                        wDET1 = swdDET1(imon, icrop, irri, isoil, irrd)
                        wDET2 = swdDET2(imon, icrop, irri, isoil, irrd)
                        wDP1 = swdDP1(imon, icrop, irri, isoil, irrd)
                        wDP2 = swdDP2(imon, icrop, irri, isoil, irrd)
                        wRO1 = swdRO1(imon, icrop, irri, isoil, irrd)
                        wRO2 = swdRO2(imon, icrop, irri, isoil, irrd)
                        wDP2RO = swdDP2RO(imon, icrop, irri, isoil, irrd)
                        wETtrans = swdETtrans(imon, icrop, irri, isoil, irrd)
                        wLatseep = swdLatseep(imon, icrop, irri, isoil, irrd)
                        wSF = swdSF(imon, icrop, irri, isoil, irrd)
                        wRO2DP = swdRO2DP(imon, icrop, irri, isoil, irrd)
                        wRO2ET = swdRO2ET(imon, icrop, irri, isoil, irrd)
    
                        IF (imon .EQ. 0) THEN
                            WRITE (607, 617) iyear, imon, SWDistx(irrd), icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (617, 617) iyear, imon, SWDistx(irrd), icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END DO
    END DO
    
!CNPPID sub district Totals
    DO irrd = 0, subcnppid
        DO imon = 0, 12
            wPrecip = SUM(cnppidPrecip(imon,:,:,:, irrd))
            wAcs = SUM(cnppidAcs(imon,:,:,:, irrd))
            wAP = SUM(cnppidAP(imon,:,:,:, irrd))
            wSWD = SUM(cnppidSWD(imon,:,:,:, irrd))
            wET = SUM(cnppidET(imon,:,:,:, irrd))
            wRO = SUM(cnppidRO(imon,:,:,:, irrd))
            wDP = SUM(cnppidDP(imon,:,:,:, irrd))
            wSL = SUM(cnppidSL(imon,:,:,:, irrd))
            wETg = SUM(cnppidETg(imon,:,:,:, irrd))
            wETb = SUM(cnppidETb(imon,:,:,:, irrd))
            wDET1 = SUM(cnppidDET1(imon,:,:,:, irrd))
            wDET2 = SUM(cnppidDET2(imon,:,:,:, irrd))
            wDP1 = SUM(cnppidDP1(imon,:,:,:, irrd))
            wDP2 = SUM(cnppidDP2(imon,:,:,:, irrd))
            wRO1 = SUM(cnppidRO1(imon,:,:,:, irrd))
            wRO2 = SUM(cnppidRO2(imon,:,:,:, irrd))
            wDP2RO = SUM(cnppidDP2RO(imon,:,:,:, irrd))
            wETtrans = SUM(cnppidETtrans(imon,:,:,:, irrd))
            wLatseep = SUM(cnppidLatseep(imon,:,:,:, irrd))
            wSF = SUM(cnppidSF(imon,:,:,:, irrd))
            wRO2DP = SUM(cnppidRO2DP(imon,:,:,:, irrd))
            wRO2ET = SUM(cnppidRO2ET(imon,:,:,:, irrd))
    
            IF (imon .EQ. 0) THEN
                WRITE (700, 710) iyear, imon, irrd, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            ELSE
                WRITE (710, 710) iyear, imon, irrd, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
            END IF
        END DO
    END DO
    
!swdistrict Crop Totals
    DO irrd = 0, subcnppid
        DO icrop = 1, ncrops
            DO imon = 0, 12
                wPrecip = SUM(cnppidPrecip(imon, icrop,:,:, irrd))
                wAcs = SUM(cnppidAcs(imon, icrop,:,:, irrd))
                wAP = SUM(cnppidAP(imon, icrop,:,:, irrd))
                wSWD = SUM(cnppidSWD(imon, icrop,:,:, irrd))
                wET = SUM(cnppidET(imon, icrop,:,:, irrd))
                wRO = SUM(cnppidRO(imon, icrop,:,:, irrd))
                wDP = SUM(cnppidDP(imon, icrop,:,:, irrd))
                wSL = SUM(cnppidSL(imon, icrop,:,:, irrd))
                wETg = SUM(cnppidETg(imon, icrop,:,:, irrd))
                wETb = SUM(cnppidETb(imon, icrop,:,:, irrd))
                wDET1 = SUM(cnppidDET1(imon, icrop,:,:, irrd))
                wDET2 = SUM(cnppidDET2(imon, icrop,:,:, irrd))
                wDP1 = SUM(cnppidDP1(imon, icrop,:,:, irrd))
                wDP2 = SUM(cnppidDP2(imon, icrop,:,:, irrd))
                wRO1 = SUM(cnppidRO1(imon, icrop,:,:, irrd))
                wRO2 = SUM(cnppidRO2(imon, icrop,:,:, irrd))
                wDP2RO = SUM(cnppidDP2RO(imon, icrop,:,:, irrd))
                wETtrans = SUM(cnppidETtrans(imon, icrop,:,:, irrd))
                wLatseep = SUM(cnppidLatseep(imon, icrop,:,:, irrd))
                wSF = SUM(cnppidSF(imon, icrop,:,:, irrd))
                wRO2DP = SUM(cnppidRO2DP(imon, icrop,:,:, irrd))
                wRO2ET = SUM(cnppidRO2ET(imon, icrop,:,:, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (701, 711) iyear, imon, irrd, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (711, 711) iyear, imon, irrd, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW district Soil Totals
    DO irrd = 0, subcnppid
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(cnppidPrecip(imon,:,:, isoil, irrd))
                wAcs = SUM(cnppidAcs(imon,:,:, isoil, irrd))
                wAP = SUM(cnppidAP(imon,:,:, isoil, irrd))
                wSWD = SUM(cnppidSWD(imon,:,:, isoil, irrd))
                wET = SUM(cnppidET(imon,:,:, isoil, irrd))
                wRO = SUM(cnppidRO(imon,:,:, isoil, irrd))
                wDP = SUM(cnppidDP(imon,:,:, isoil, irrd))
                wSL = SUM(cnppidSL(imon,:,:, isoil, irrd))
                wETg = SUM(cnppidETg(imon,:,:, isoil, irrd))
                wETb = SUM(cnppidETb(imon,:,:, isoil, irrd))
                wDET1 = SUM(cnppidDET1(imon,:,:, isoil, irrd))
                wDET2 = SUM(cnppidDET2(imon,:,:, isoil, irrd))
                wDP1 = SUM(cnppidDP1(imon,:,:, isoil, irrd))
                wDP2 = SUM(cnppidDP2(imon,:,:, isoil, irrd))
                wRO1 = SUM(cnppidRO1(imon,:,:, isoil, irrd))
                wRO2 = SUM(cnppidRO2(imon,:,:, isoil, irrd))
                wDP2RO = SUM(cnppidDP2RO(imon,:,:, isoil, irrd))
                wETtrans = SUM(cnppidETtrans(imon,:,:, isoil, irrd))
                wLatseep = SUM(cnppidLatseep(imon,:,:, isoil, irrd))
                wSF = SUM(cnppidSF(imon,:,:, isoil, irrd))
                wRO2DP = SUM(cnppidRO2DP(imon,:,:, isoil, irrd))
                wRO2ET = SUM(cnppidRO2ET(imon,:,:, isoil, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (702, 712) iyear, imon, irrd, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (712, 712) iyear, imon, irrd, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW district IrrType Totals
    DO irrd = 0, subcnppid
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(cnppidPrecip(imon,:, irri,:, irrd))
                wAcs = SUM(cnppidAcs(imon,:, irri,:, irrd))
                wAP = SUM(cnppidAP(imon,:, irri,:, irrd))
                wSWD = SUM(cnppidSWD(imon,:, irri,:, irrd))
                wET = SUM(cnppidET(imon,:, irri,:, irrd))
                wRO = SUM(cnppidRO(imon,:, irri,:, irrd))
                wDP = SUM(cnppidDP(imon,:, irri,:, irrd))
                wSL = SUM(cnppidSL(imon,:, irri,:, irrd))
                wETg = SUM(cnppidETg(imon,:, irri,:, irrd))
                wETb = SUM(cnppidETb(imon,:, irri,:, irrd))
                wDET1 = SUM(cnppidDET1(imon,:, irri,:, irrd))
                wDET2 = SUM(cnppidDET2(imon,:, irri,:, irrd))
                wDP1 = SUM(cnppidDP1(imon,:, irri,:, irrd))
                wDP2 = SUM(cnppidDP2(imon,:, irri,:, irrd))
                wRO1 = SUM(cnppidRO1(imon,:, irri,:, irrd))
                wRO2 = SUM(cnppidRO2(imon,:, irri,:, irrd))
                wDP2RO = SUM(cnppidDP2RO(imon,:, irri,:, irrd))
                wETtrans = SUM(cnppidETtrans(imon,:, irri,:, irrd))
                wLatseep = SUM(cnppidLatseep(imon,:, irri,:, irrd))
                wSF = SUM(cnppidSF(imon,:, irri,:, irrd))
                wRO2DP = SUM(cnppidRO2DP(imon,:, irri,:, irrd))
                wRO2ET = SUM(cnppidRO2ET(imon,:, irri,:, irrd))
    
                IF (imon .EQ. 0) THEN
                    WRITE (703, 713) iyear, imon, irrd, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (713, 713) iyear, imon, irrd, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!SW District Crop - Soil Totals
    DO irrd = 0, subcnppid
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(cnppidPrecip(imon, icrop,:, isoil, irrd))
                    wAcs = SUM(cnppidAcs(imon, icrop,:, isoil, irrd))
                    wAP = SUM(cnppidAP(imon, icrop,:, isoil, irrd))
                    wSWD = SUM(cnppidSWD(imon, icrop,:, isoil, irrd))
                    wET = SUM(cnppidET(imon, icrop,:, isoil, irrd))
                    wRO = SUM(cnppidRO(imon, icrop,:, isoil, irrd))
                    wDP = SUM(cnppidDP(imon, icrop,:, isoil, irrd))
                    wSL = SUM(cnppidSL(imon, icrop,:, isoil, irrd))
                    wETg = SUM(cnppidETg(imon, icrop,:, isoil, irrd))
                    wETb = SUM(cnppidETb(imon, icrop,:, isoil, irrd))
                    wDET1 = SUM(cnppidDET1(imon, icrop,:, isoil, irrd))
                    wDET2 = SUM(cnppidDET2(imon, icrop,:, isoil, irrd))
                    wDP1 = SUM(cnppidDP1(imon, icrop,:, isoil, irrd))
                    wDP2 = SUM(cnppidDP2(imon, icrop,:, isoil, irrd))
                    wRO1 = SUM(cnppidRO1(imon, icrop,:, isoil, irrd))
                    wRO2 = SUM(cnppidRO2(imon, icrop,:, isoil, irrd))
                    wDP2RO = SUM(cnppidDP2RO(imon, icrop,:, isoil, irrd))
                    wETtrans = SUM(cnppidETtrans(imon, icrop,:, isoil, irrd))
                    wLatseep = SUM(cnppidLatseep(imon, icrop,:, isoil, irrd))
                    wSF = SUM(cnppidSF(imon, icrop,:, isoil, irrd))
                    wRO2DP = SUM(cnppidRO2DP(imon, icrop,:, isoil, irrd))
                    wRO2ET = SUM(cnppidRO2ET(imon, icrop,:, isoil, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (704, 714) iyear, imon, irrd, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (714, 714) iyear, imon, irrd, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW District Crop - IrrSource Totals
    DO irrd = 0, subcnppid
        DO icrop = 1, ncrops
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(cnppidPrecip(imon, icrop, irri,:, irrd))
                    wAcs = SUM(cnppidAcs(imon, icrop, irri,:, irrd))
                    wAP = SUM(cnppidAP(imon, icrop, irri,:, irrd))
                    wSWD = SUM(cnppidSWD(imon, icrop, irri,:, irrd))
                    wET = SUM(cnppidET(imon, icrop, irri,:, irrd))
                    wRO = SUM(cnppidRO(imon, icrop, irri,:, irrd))
                    wDP = SUM(cnppidDP(imon, icrop, irri,:, irrd))
                    wSL = SUM(cnppidSL(imon, icrop, irri,:, irrd))
                    wETg = SUM(cnppidETg(imon, icrop, irri,:, irrd))
                    wETb = SUM(cnppidETb(imon, icrop, irri,:, irrd))
                    wDET1 = SUM(cnppidDET1(imon, icrop, irri,:, irrd))
                    wDET2 = SUM(cnppidDET2(imon, icrop, irri,:, irrd))
                    wDP1 = SUM(cnppidDP1(imon, icrop, irri,:, irrd))
                    wDP2 = SUM(cnppidDP2(imon, icrop, irri,:, irrd))
                    wRO1 = SUM(cnppidRO1(imon, icrop, irri,:, irrd))
                    wRO2 = SUM(cnppidRO2(imon, icrop, irri,:, irrd))
                    wDP2RO = SUM(cnppidDP2RO(imon, icrop, irri,:, irrd))
                    wETtrans = SUM(cnppidETtrans(imon, icrop, irri,:, irrd))
                    wLatseep = SUM(cnppidLatseep(imon, icrop, irri,:, irrd))
                    wSF = SUM(cnppidSF(imon, icrop, irri,:, irrd))
                    wRO2DP = SUM(cnppidRO2DP(imon, icrop, irri,:, irrd))
                    wRO2ET = SUM(cnppidRO2ET(imon, icrop, irri,:, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (705, 715) iyear, imon, irrd, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (715, 715) iyear, imon, irrd, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW district Soil - IrrSource Totals
    DO irrd = 0, subcnppid
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(cnppidPrecip(imon,:, irri, isoil, irrd))
                    wAcs = SUM(cnppidAcs(imon,:, irri, isoil, irrd))
                    wAP = SUM(cnppidAP(imon,:, irri, isoil, irrd))
                    wSWD = SUM(cnppidSWD(imon,:, irri, isoil, irrd))
                    wET = SUM(cnppidET(imon,:, irri, isoil, irrd))
                    wRO = SUM(cnppidRO(imon,:, irri, isoil, irrd))
                    wDP = SUM(cnppidDP(imon,:, irri, isoil, irrd))
                    wSL = SUM(cnppidSL(imon,:, irri, isoil, irrd))
                    wETg = SUM(cnppidETg(imon,:, irri, isoil, irrd))
                    wETb = SUM(cnppidETb(imon,:, irri, isoil, irrd))
                    wDET1 = SUM(cnppidDET1(imon,:, irri, isoil, irrd))
                    wDET2 = SUM(cnppidDET2(imon,:, irri, isoil, irrd))
                    wDP1 = SUM(cnppidDP1(imon,:, irri, isoil, irrd))
                    wDP2 = SUM(cnppidDP2(imon,:, irri, isoil, irrd))
                    wRO1 = SUM(cnppidRO1(imon,:, irri, isoil, irrd))
                    wRO2 = SUM(cnppidRO2(imon,:, irri, isoil, irrd))
                    wDP2RO = SUM(cnppidDP2RO(imon,:, irri, isoil, irrd))
                    wETtrans = SUM(cnppidETtrans(imon,:, irri, isoil, irrd))
                    wLatseep = SUM(cnppidLatseep(imon,:, irri, isoil, irrd))
                    wSF = SUM(cnppidSF(imon,:, irri, isoil, irrd))
                    wRO2DP = SUM(cnppidRO2DP(imon,:, irri, isoil, irrd))
                    wRO2ET = SUM(swdRO2ET(imon,:, irri, isoil, irrd))

                    IF (imon .EQ. 0) THEN
                        WRITE (706, 716) iyear, imon, irrd, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (716, 716) iyear, imon, irrd, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!SW District Crop - Soil - IrrSource Totals
    DO irrd = 0, subcnppid
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = cnppidPrecip(imon, icrop, irri, isoil, irrd)
                        wAcs = cnppidAcs(imon, icrop, irri, isoil, irrd)
                        wAP = cnppidAP(imon, icrop, irri, isoil, irrd)
                        wSWD = cnppidSWD(imon, icrop, irri, isoil, irrd)
                        wET = cnppidET(imon, icrop, irri, isoil, irrd)
                        wRO = cnppidRO(imon, icrop, irri, isoil, irrd)
                        wDP = cnppidDP(imon, icrop, irri, isoil, irrd)
                        wSL = cnppidSL(imon, icrop, irri, isoil, irrd)
                        wETg = cnppidETg(imon, icrop, irri, isoil, irrd)
                        wETb = cnppidETb(imon, icrop, irri, isoil, irrd)
                        wDET1 = cnppidDET1(imon, icrop, irri, isoil, irrd)
                        wDET2 = cnppidDET2(imon, icrop, irri, isoil, irrd)
                        wDP1 = cnppidDP1(imon, icrop, irri, isoil, irrd)
                        wDP2 = cnppidDP2(imon, icrop, irri, isoil, irrd)
                        wRO1 = cnppidRO1(imon, icrop, irri, isoil, irrd)
                        wRO2 = cnppidRO2(imon, icrop, irri, isoil, irrd)
                        wDP2RO = cnppidDP2RO(imon, icrop, irri, isoil, irrd)
                        wETtrans = cnppidETtrans(imon, icrop, irri, isoil, irrd)
                        wLatseep = cnppidLatseep(imon, icrop, irri, isoil, irrd)
                        wSF = cnppidSF(imon, icrop, irri, isoil, irrd)
                        wRO2DP = cnppidRO2DP(imon, icrop, irri, isoil, irrd)
                        wRO2ET = cnppidRO2ET(imon, icrop, irri, isoil, irrd)
    
                        IF (imon .EQ. 0) THEN
                            WRITE (707, 717) iyear, imon, irrd, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (717, 717) iyear, imon, irrd, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END DO
    END DO    
    
!Uarea Totals
    DO iUarea = 0, nuArea
        DO imon = 0, 12
            wPrecip = SUM(uAreaPrecip(imon,:,:,:, iuArea))
            wAcs = SUM(uAreaAcs(imon,:,:,:, iuArea))
            wAP = SUM(uAreaAP(imon,:,:,:, iuArea))
            wSWD = SUM(uAreaSWD(imon,:,:,:, iuArea))
            wET = SUM(uAreaET(imon,:,:,:, iuArea))
            wRO = SUM(uAreaRO(imon,:,:,:, iuArea))
            wDP = SUM(uAreaDP(imon,:,:,:, iuArea))
            wSL = SUM(uAreaSL(imon,:,:,:, iuArea))
            wETg = SUM(uAreaETg(imon,:,:,:, iuArea))
            wETb = SUM(uAreaETb(imon,:,:,:, iuArea))
            wDET1 = SUM(uAreaDET1(imon,:,:,:, iuArea))
            wDET2 = SUM(uAreaDET2(imon,:,:,:, iuArea))
            wDP1 = SUM(uAreaDP1(imon,:,:,:, iuArea))
            wDP2 = SUM(uAreaDP2(imon,:,:,:, iuArea))
            wRO1 = SUM(uAreaRO1(imon,:,:,:, iuArea))
            wRO2 = SUM(uAreaRO2(imon,:,:,:, iuArea))
            wDP2RO = SUM(uAreaDP2RO(imon,:,:,:, iuArea))
            wETtrans = SUM(uAreaETtrans(imon,:,:,:, iuArea))
            wLatseep = SUM(uAreaLatseep(imon,:,:,:, iuArea))
            wSF = SUM(uAreaSF(imon,:,:,:, iuArea))
            wRO2DP = SUM(uAreaRO2DP(imon,:,:,:, iuArea))
            wRO2ET = SUM(uAreaRO2ET(imon,:,:,:, iuArea))
    
            IF (imon .EQ. 0) THEN
                WRITE (800, 810) iyear, imon, iuArea, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (800, 240) SUM(UAcanalrch(i, iuArea, :))
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (800, 240) SUM(UAmiscrch(i, iuArea, :))
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (800, 240) SUM(UAMunirch(i, iuArea, :))
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (800, 240) SUM(UAMiscpmp(i, iuArea, :))
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (800, 240) SUM(UAMunipmp(i, iuArea, :))
                    END DO
                END IF
            
                WRITE (800, 241)
            
            ELSE
                WRITE (810, 810) iyear, imon, iuArea, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                    wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                
                IF (cnlrflg .EQ. 1) THEN
                    DO i = 1, ncanals
                        WRITE (810, 240) UAcanalrch(i, iuArea, imon)
                    END DO
                END IF
            
                IF (miscpflg .EQ. 1) THEN
                    DO i = 1, nmiscr
                        WRITE (810, 240) UAmiscrch(i, iuArea, imon)
                    END DO
                END IF
            
                IF (MIrflg .EQ. 1) THEN
                    DO i = 1, nmunir
                        WRITE (810, 240) UAMunirch(i, iuArea, imon)
                    END DO
                END IF
            
                IF (Miscpflg .EQ. 1) THEN
                    DO i = 1, nMiscp
                        WRITE (810, 240) UAMiscpmp(i, iuArea, imon)
                    END DO
                END IF
            
                IF (MIpflg .EQ. 1) THEN
                    DO i = 1, nMunip
                        WRITE (810, 240) UAMunipmp(i, iuArea, imon)
                    END DO
                END IF
            
                WRITE (810, 241)
                
            END IF
        END DO
    END DO    
    
!Uarea Crop Totals
    DO iuArea = 0, nuArea
        DO icrop = 1, ncrops
            DO imon = 0, 12
                wPrecip = SUM(uAreaPrecip(imon, icrop,:,:, iuArea))
                wAcs = SUM(uAreaAcs(imon, icrop,:,:, iuArea))
                wAP = SUM(uAreaAP(imon, icrop,:,:, iuArea))
                wSWD = SUM(uAreaSWD(imon, icrop,:,:, iuArea))
                wET = SUM(uAreaET(imon, icrop,:,:, iuArea))
                wRO = SUM(uAreaRO(imon, icrop,:,:, iuArea))
                wDP = SUM(uAreaDP(imon, icrop,:,:, iuArea))
                wSL = SUM(uAreaSL(imon, icrop,:,:, iuArea))
                wETg = SUM(uAreaETg(imon, icrop,:,:, iuArea))
                wETb = SUM(uAreaETb(imon, icrop,:,:, iuArea))
                wDET1 = SUM(uAreaDET1(imon, icrop,:,:, iuArea))
                wDET2 = SUM(uAreaDET2(imon, icrop,:,:, iuArea))
                wDP1 = SUM(uAreaDP1(imon, icrop,:,:, iuArea))
                wDP2 = SUM(uAreaDP2(imon, icrop,:,:, iuArea))
                wRO1 = SUM(uAreaRO1(imon, icrop,:,:, iuArea))
                wRO2 = SUM(uAreaRO2(imon, icrop,:,:, iuArea))
                wDP2RO = SUM(uAreaDP2RO(imon, icrop,:,:, iuArea))
                wETtrans = SUM(uAreaETtrans(imon, icrop,:,:, iuArea))
                wLatseep = SUM(uAreaLatseep(imon, icrop,:,:, iuArea))
                wSF = SUM(uAreaSF(imon, icrop,:,:, iuArea))
                wRO2DP = SUM(uAreaRO2DP(imon, icrop,:,:, iuArea))
                wRO2ET = SUM(uAreaRO2ET(imon, icrop,:,:, iuArea))
    
                IF (imon .EQ. 0) THEN
                    WRITE (801, 811) iyear, imon, iuArea, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (811, 811) iyear, imon, iuArea, icrop, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO   
    
!Uarea Soil Totals
    DO iuArea = 0, nuArea
        DO isoil = 1, nsoils
            DO imon = 0, 12
                wPrecip = SUM(uAreaPrecip(imon,:,:, isoil, iuArea))
                wAcs = SUM(uAreaAcs(imon,:,:, isoil, iuArea))
                wAP = SUM(uAreaAP(imon,:,:, isoil, iuArea))
                wSWD = SUM(uAreaSWD(imon,:,:, isoil, iuArea))
                wET = SUM(uAreaET(imon,:,:, isoil, iuArea))
                wRO = SUM(uAreaRO(imon,:,:, isoil, iuArea))
                wDP = SUM(uAreaDP(imon,:,:, isoil, iuArea))
                wSL = SUM(uAreaSL(imon,:,:, isoil, iuArea))
                wETg = SUM(uAreaETg(imon,:,:, isoil, iuArea))
                wETb = SUM(uAreaETb(imon,:,:, isoil, iuArea))
                wDET1 = SUM(uAreaDET1(imon,:,:, isoil, iuArea))
                wDET2 = SUM(uAreaDET2(imon,:,:, isoil, iuArea))
                wDP1 = SUM(uAreaDP1(imon,:,:, isoil, iuArea))
                wDP2 = SUM(uAreaDP2(imon,:,:, isoil, iuArea))
                wRO1 = SUM(uAreaRO1(imon,:,:, isoil, iuArea))
                wRO2 = SUM(uAreaRO2(imon,:,:, isoil, iuArea))
                wDP2RO = SUM(uAreaDP2RO(imon,:,:, isoil, iuArea))
                wETtrans = SUM(uAreaETtrans(imon,:,:, isoil, iuArea))
                wLatseep = SUM(uAreaLatseep(imon,:,:, isoil, iuArea))
                wSF = SUM(uAreaSF(imon,:,:, isoil, iuArea))
                wRO2DP = SUM(uAreaRO2DP(imon,:,:, isoil, iuArea))
                wRO2ET = SUM(uAreaRO2ET(imon,:,:, isoil, iuArea))
    
                IF (imon .EQ. 0) THEN
                    WRITE (802, 812) iyear, imon, iuArea, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (812, 812) iyear, imon, iuArea, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO

!Uarea IrrType Totals
    DO iuArea = 0, nuArea
        DO irri = 1, 4
            DO imon = 0, 12
                wPrecip = SUM(uAreaPrecip(imon,:, irri,:, iuArea))
                wAcs = SUM(uAreaAcs(imon,:, irri,:, iuArea))
                wAP = SUM(uAreaAP(imon,:, irri,:, iuArea))
                wSWD = SUM(uAreaSWD(imon,:, irri,:, iuArea))
                wET = SUM(uAreaET(imon,:, irri,:, iuArea))
                wRO = SUM(uAreaRO(imon,:, irri,:, iuArea))
                wDP = SUM(uAreaDP(imon,:, irri,:, iuArea))
                wSL = SUM(uAreaSL(imon,:, irri,:, iuArea))
                wETg = SUM(uAreaETg(imon,:, irri,:, iuArea))
                wETb = SUM(uAreaETb(imon,:, irri,:, iuArea))
                wDET1 = SUM(uAreaDET1(imon,:, irri,:, iuArea))
                wDET2 = SUM(uAreaDET2(imon,:, irri,:, iuArea))
                wDP1 = SUM(uAreaDP1(imon,:, irri,:, iuArea))
                wDP2 = SUM(uAreaDP2(imon,:, irri,:, iuArea))
                wRO1 = SUM(uAreaRO1(imon,:, irri,:, iuArea))
                wRO2 = SUM(uAreaRO2(imon,:, irri,:, iuArea))
                wDP2RO = SUM(uAreaDP2RO(imon,:, irri,:, iuArea))
                wETtrans = SUM(uAreaETtrans(imon,:, irri,:, iuArea))
                wLatseep = SUM(uAreaLatseep(imon,:, irri,:, iuArea))
                wSF = SUM(uAreaSF(imon,:, irri,:, iuArea))
                wRO2DP = SUM(uAreaRO2DP(imon,:, irri,:, iuArea))
                wRO2ET = SUM(uAreaRO2ET(imon,:, irri,:, iuArea))
    
                IF (imon .EQ. 0) THEN
                    WRITE (803, 813) iyear, imon, iuArea, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                ELSE
                    WRITE (813, 813) iyear, imon, iuArea, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                        wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                END IF
            END DO
        END DO
    END DO
    
!Uarea Crop - Soil Totals
    DO iuArea = 0, nuArea
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO imon = 0, 12
                    wPrecip = SUM(uAreaPrecip(imon, icrop,:, isoil, iuArea))
                    wAcs = SUM(uAreaAcs(imon, icrop,:, isoil, iuArea))
                    wAP = SUM(uAreaAP(imon, icrop,:, isoil, iuArea))
                    wSWD = SUM(uAreaSWD(imon, icrop,:, isoil, iuArea))
                    wET = SUM(uAreaET(imon, icrop,:, isoil, iuArea))
                    wRO = SUM(uAreaRO(imon, icrop,:, isoil, iuArea))
                    wDP = SUM(uAreaDP(imon, icrop,:, isoil, iuArea))
                    wSL = SUM(uAreaSL(imon, icrop,:, isoil, iuArea))
                    wETg = SUM(uAreaETg(imon, icrop,:, isoil, iuArea))
                    wETb = SUM(uAreaETb(imon, icrop,:, isoil, iuArea))
                    wDET1 = SUM(uAreaDET1(imon, icrop,:, isoil, iuArea))
                    wDET2 = SUM(uAreaDET2(imon, icrop,:, isoil, iuArea))
                    wDP1 = SUM(uAreaDP1(imon, icrop,:, isoil, iuArea))
                    wDP2 = SUM(uAreaDP2(imon, icrop,:, isoil, iuArea))
                    wRO1 = SUM(uAreaRO1(imon, icrop,:, isoil, iuArea))
                    wRO2 = SUM(uAreaRO2(imon, icrop,:, isoil, iuArea))
                    wDP2RO = SUM(uAreaDP2RO(imon, icrop,:, isoil, iuArea))
                    wETtrans = SUM(uAreaETtrans(imon, icrop,:, isoil, iuArea))
                    wLatseep = SUM(uAreaLatseep(imon, icrop,:, isoil, iuArea))
                    wSF = SUM(uAreaSF(imon, icrop,:, isoil, iuArea))
                    wRO2DP = SUM(uAreaRO2DP(imon, icrop,:, isoil, iuArea))
                    wRO2ET = SUM(uAreaRO2ET(imon, icrop,:, isoil, iuArea))

                    IF (imon .EQ. 0) THEN
                        WRITE (804, 814) iyear, imon, iuArea, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (814, 814) iyear, imon, iuArea, icrop, isoil, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Crop - IrrSource Totals
    DO iuArea = 0, nuArea
        DO icrop = 1, ncrops
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(uAreaPrecip(imon, icrop, irri,:, iuArea))
                    wAcs = SUM(uAreaAcs(imon, icrop, irri,:, iuArea))
                    wAP = SUM(uAreaAP(imon, icrop, irri,:, iuArea))
                    wSWD = SUM(uAreaSWD(imon, icrop, irri,:, iuArea))
                    wET = SUM(uAreaET(imon, icrop, irri,:, iuArea))
                    wRO = SUM(uAreaRO(imon, icrop, irri,:, iuArea))
                    wDP = SUM(uAreaDP(imon, icrop, irri,:, iuArea))
                    wSL = SUM(uAreaSL(imon, icrop, irri,:, iuArea))
                    wETg = SUM(uAreaETg(imon, icrop, irri,:, iuArea))
                    wETb = SUM(uAreaETb(imon, icrop, irri,:, iuArea))
                    wDET1 = SUM(uAreaDET1(imon, icrop, irri,:, iuArea))
                    wDET2 = SUM(uAreaDET2(imon, icrop, irri,:, iuArea))
                    wDP1 = SUM(uAreaDP1(imon, icrop, irri,:, iuArea))
                    wDP2 = SUM(uAreaDP2(imon, icrop, irri,:, iuArea))
                    wRO1 = SUM(uAreaRO1(imon, icrop, irri,:, iuArea))
                    wRO2 = SUM(uAreaRO2(imon, icrop, irri,:, iuArea))
                    wDP2RO = SUM(uAreaDP2RO(imon, icrop, irri,:, iuArea))
                    wETtrans = SUM(uAreaETtrans(imon, icrop, irri,:, iuArea))
                    wLatseep = SUM(uAreaLatseep(imon, icrop, irri,:, iuArea))
                    wSF = SUM(uAreaSF(imon, icrop, irri,:, iuArea))
                    wRO2DP = SUM(uAreaRO2DP(imon, icrop, irri,:, iuArea))
                    wRO2ET = SUM(uAreaRO2ET(imon, icrop, irri,:, iuArea))

                    IF (imon .EQ. 0) THEN
                        WRITE (805, 815) iyear, imon, iuArea, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (815, 815) iyear, imon, iuArea, icrop, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Soil - IrrSource Totals
    DO iuArea = 0, nuArea
        DO isoil = 1, nsoils
            DO irri = 1, 4
                DO imon = 0, 12
                    wPrecip = SUM(uAreaPrecip(imon,:, irri, isoil, iuArea))
                    wAcs = SUM(uAreaAcs(imon,:, irri, isoil, iuArea))
                    wAP = SUM(uAreaAP(imon,:, irri, isoil, iuArea))
                    wSWD = SUM(uAreaSWD(imon,:, irri, isoil, iuArea))
                    wET = SUM(uAreaET(imon,:, irri, isoil, iuArea))
                    wRO = SUM(uAreaRO(imon,:, irri, isoil, iuArea))
                    wDP = SUM(uAreaDP(imon,:, irri, isoil, iuArea))
                    wSL = SUM(uAreaSL(imon,:, irri, isoil, iuArea))
                    wETg = SUM(uAreaETg(imon,:, irri, isoil, iuArea))
                    wETb = SUM(uAreaETb(imon,:, irri, isoil, iuArea))
                    wDET1 = SUM(uAreaDET1(imon,:, irri, isoil, iuArea))
                    wDET2 = SUM(uAreaDET2(imon,:, irri, isoil, iuArea))
                    wDP1 = SUM(uAreaDP1(imon,:, irri, isoil, iuArea))
                    wDP2 = SUM(uAreaDP2(imon,:, irri, isoil, iuArea))
                    wRO1 = SUM(uAreaRO1(imon,:, irri, isoil, iuArea))
                    wRO2 = SUM(uAreaRO2(imon,:, irri, isoil, iuArea))
                    wDP2RO = SUM(uAreaDP2RO(imon,:, irri, isoil, iuArea))
                    wETtrans = SUM(uAreaETtrans(imon,:, irri, isoil, iuArea))
                    wLatseep = SUM(uAreaLatseep(imon,:, irri, isoil, iuArea))
                    wSF = SUM(uAreaSF(imon,:, irri, isoil, iuArea))
                    wRO2DP = SUM(uAreaRO2DP(imon,:, irri, isoil, iuArea))
                    wRO2ET = SUM(uAreaRO2ET(imon,:, irri, isoil, iuArea))

                    IF (imon .EQ. 0) THEN
                        WRITE (806, 816) iyear, imon, iuArea, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    ELSE
                        WRITE (816, 816) iyear, imon, iuArea, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                            wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                    END IF
                END DO
            END DO
        END DO
    END DO
    
!ROzone Crop - Soil - IrrSource Totals
    DO iuArea = 0, nuArea
        DO icrop = 1, ncrops
            DO isoil = 1, nsoils
                DO irri = 1, 4
                    DO imon = 0, 12
                        wPrecip = uAreaPrecip(imon, icrop, irri, isoil, iuArea)
                        wAcs = uAreaAcs(imon, icrop, irri, isoil, iuArea)
                        wAP = uAreaAP(imon, icrop, irri, isoil, iuArea)
                        wSWD = uAreaSWD(imon, icrop, irri, isoil, iuArea)
                        wET = uAreaET(imon, icrop, irri, isoil, iuArea)
                        wRO = uAreaRO(imon, icrop, irri, isoil, iuArea)
                        wDP = uAreaDP(imon, icrop, irri, isoil, iuArea)
                        wSL = uAreaSL(imon, icrop, irri, isoil, iuArea)
                        wETg = uAreaETg(imon, icrop, irri, isoil, iuArea)
                        wETb = uAreaETb(imon, icrop, irri, isoil, iuArea)
                        wDET1 = uAreaDET1(imon, icrop, irri, isoil, iuArea)
                        wDET2 = uAreaDET2(imon, icrop, irri, isoil, iuArea)
                        wDP1 = uAreaDP1(imon, icrop, irri, isoil, iuArea)
                        wDP2 = uAreaDP2(imon, icrop, irri, isoil, iuArea)
                        wRO1 = uAreaRO1(imon, icrop, irri, isoil, iuArea)
                        wRO2 = uAreaRO2(imon, icrop, irri, isoil, iuArea)
                        wDP2RO = uAreaDP2RO(imon, icrop, irri, isoil, iuArea)
                        wETtrans = uAreaETtrans(imon, icrop, irri, isoil, iuArea)
                        wLatseep = uAreaLatseep(imon, icrop, irri, isoil, iuArea)
                        wSF = uAreaSF(imon, icrop, irri, isoil, iuArea)
                        wRO2DP = uAreaRO2DP(imon, icrop, irri, isoil, iuArea)
                        wRO2ET = uAreaRO2ET(imon, icrop, irri, isoil, iuArea)
    
                        IF (imon .EQ. 0) THEN
                            WRITE (807, 817) iyear, imon, iuArea, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        ELSE
                            WRITE (817, 817) iyear, imon, iuArea, icrop, isoil, irri, wAcs, wPrecip, wAP, wSWD, wET, wRO, wDP, wSL, wETg, WETb, wDET1, wDET2, &
                                                wDP1, wDP2, wRO1, wRO2, wDP2RO, wETtrans, wLatseep, wSF, wRO2DP, wRO2ET
                        END IF
                    END DO
                END DO
            END DO
        END DO
    END DO
    
210 FORMAT (I5, ',', I3, 22(',', F14.2), \)
    
240 FORMAT (',', F14.2, \)
241 FORMAT ()
    
211 FORMAT (I5, 2(',', I3), 22(',', F14.2))
212 FORMAT (I5, 2(',', I3), 22(',', F14.2))
213 FORMAT (I5, 2(',', I3), 22(',', F14.2))
214 FORMAT (I5, 3(',', I3), 22(',', F14.2))
215 FORMAT (I5, 3(',', I3), 22(',', F14.2))
216 FORMAT (I5, 3(',', I3), 22(',', F14.2))
217 FORMAT (I5, 4(',', I3), 22(',', F14.2))

310 FORMAT (I5, 2(',', I3), ',', A14, 22(',', F14.2), \)
    
311 FORMAT (I5, 2(',', I3), ',', A14, ',', I3, 22(',', F14.2))
312 FORMAT (I5, 2(',', I3), ',', A14, ',', I3, 22(',', F14.2))
313 FORMAT (I5, 2(',', I3), ',', A14, ',', I3, 22(',', F14.2))
314 FORMAT (I5, 2(',', I3), ',', A14, 2(',', I3), 22(',', F14.2))
315 FORMAT (I5, 2(',', I3), ',', A14, 2(',', I3), 22(',', F14.2))
316 FORMAT (I5, 2(',', I3), ',', A14, 2(',', I3), 22(',', F14.2))
317 FORMAT (I5, 2(',', I3), ',', A14, 3(',', I3), 22(',', F14.2))
       
410 FORMAT (I5, 2(',', I3), 22(',', F14.2), \)
411 FORMAT (I5, 3(',', I3), 22(',', F14.2))
412 FORMAT (I5, 3(',', I3), 22(',', F14.2))
413 FORMAT (I5, 3(',', I3), 22(',', F14.2))    
414 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
415 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
416 FORMAT (I5, 4(',', I3), 22(',', F14.2))
417 FORMAT (I5, 5(',', I3), 22(',', F14.2))

510 FORMAT (I5, 2(',', I3), 22(',', F14.2), \)
511 FORMAT (I5, 3(',', I3), 22(',', F14.2))
512 FORMAT (I5, 3(',', I3), 22(',', F14.2))
513 FORMAT (I5, 3(',', I3), 22(',', F14.2))    
514 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
515 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
516 FORMAT (I5, 4(',', I3), 22(',', F14.2))
517 FORMAT (I5, 5(',', I3), 22(',', F14.2))
    
610 FORMAT (I5, 2(',', I3), 22(',', F14.2))
611 FORMAT (I5, 3(',', I3), 22(',', F14.2))
612 FORMAT (I5, 3(',', I3), 22(',', F14.2))
613 FORMAT (I5, 3(',', I3), 22(',', F14.2))    
614 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
615 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
616 FORMAT (I5, 4(',', I3), 22(',', F14.2))
617 FORMAT (I5, 5(',', I3), 22(',', F14.2))
    
710 FORMAT (I5, 2(',', I3), 22(',', F14.2))
711 FORMAT (I5, 3(',', I3), 22(',', F14.2))
712 FORMAT (I5, 3(',', I3), 22(',', F14.2))
713 FORMAT (I5, 3(',', I3), 22(',', F14.2))    
714 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
715 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
716 FORMAT (I5, 4(',', I3), 22(',', F14.2))
717 FORMAT (I5, 5(',', I3), 22(',', F14.2))
    
810 FORMAT (I5, 2(',', I3), 22(',', F14.2), \)
811 FORMAT (I5, 3(',', I3), 22(',', F14.2))
812 FORMAT (I5, 3(',', I3), 22(',', F14.2))
813 FORMAT (I5, 3(',', I3), 22(',', F14.2))    
814 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
815 FORMAT (I5, 4(',', I3), 22(',', F14.2))    
816 FORMAT (I5, 4(',', I3), 22(',', F14.2))
817 FORMAT (I5, 5(',', I3), 22(',', F14.2))
    
END SUBROUTINE writereports
!****************************************************************************
SUBROUTINE closereport
    USE PARAM
    IMPLICIT NONE
    
    DO i = 1, 8
        CLOSE (199 + i)
        CLOSE (209 + i)
        CLOSE (299 + i)
        CLOSE (309 + i)
        CLOSE (399 + i)
        CLOSE (409 + i)
        CLOSE (499 + i)
        CLOSE (509 + i)
        CLOSE (599 + i)
        CLOSE (609 + i)
        CLOSE (699 + i)
        CLOSE (709 + i)
        CLOSE (799 + i)
        CLOSE (809 + i)
    END DO
    
END SUBROUTINE closereport
!****************************************************************************
SUBROUTINE zerovals
    USE PARAM
    IMPLICIT NONE

    rPrecip = 0.
    rAcs = 0.
    rAP = 0.
    rSWD = 0.
    rET = 0.
    rRO = 0.
    rDP = 0.
    rSL = 0.
    rETg = 0.
    rETb = 0.
    rDET1 = 0.
    rDET2 = 0.
    rDP1 = 0.
    rDP2 = 0.
    rRO1 = 0.
    rRO2 = 0.
    rDP2RO = 0.
    rETtrans = 0.
    rLatseep = 0.
    rSF = 0.
    rRO2DP = 0.
    rRO2ET = 0.
    
    cPrecip = 0.
    cAcs = 0.
    cAP = 0.
    cSWD = 0.
    cET = 0.
    cRO = 0.
    cDP = 0.
    cSL = 0.
    cETg = 0.
    cETb = 0.
    cDET1 = 0.
    cDET2 = 0.
    cDP1 = 0.
    cDP2 = 0.
    cRO1 = 0.
    cRO2 = 0.
    cDP2RO = 0.
    cETtrans = 0.
    cLatseep = 0.
    cSF = 0.
    cRO2DP = 0.
    cRO2ET = 0.
    
    cfPrecip = 0.
    cfAcs = 0.
    cfAP = 0.
    cfSWD = 0.
    cfET = 0.
    cfRO = 0.
    cfDP = 0.
    cfSL = 0.
    cfETg = 0.
    cfETb = 0.
    cfDET1 = 0.
    cfDET2 = 0.
    cfDP1 = 0.
    cfDP2 = 0.
    cfRO1 = 0.
    cfRO2 = 0.
    cfDP2RO = 0.
    cfETtrans = 0.
    cfLatseep = 0.
    cfSF = 0.
    cfRO2DP = 0.
    cfRO2ET = 0.
    
    rzPrecip = 0.
    rzAcs = 0.
    rzAP = 0.
    rzSWD = 0.
    rzET = 0.
    rzRO = 0.
    rzDP = 0.
    rzSL = 0.
    rzETg = 0.
    rzETb = 0.
    rzDET1 = 0.
    rzDET2 = 0.
    rzDP1 = 0.
    rzDP2 = 0.
    rzRO1 = 0.
    rzRO2 = 0.
    rzDP2RO = 0.
    rzETtrans = 0.
    rzLatseep = 0.
    rzSF = 0.
    rzRO2DP = 0.
    rzRO2ET = 0.
    
    swdPrecip = 0.
    swdAcs = 0.
    swdAP = 0.
    swdSWD = 0.
    swdET = 0.
    swdRO = 0.
    swdDP = 0.
    swdSL = 0.
    swdETg = 0.
    swdETb = 0.
    swdDET1 = 0.
    swdDET2 = 0.
    swdDP1 = 0.
    swdDP2 = 0.
    swdRO1 = 0.
    swdRO2 = 0.
    swdDP2RO = 0.
    swdETtrans = 0.
    swdLatseep = 0.
    swdSF = 0.
    swdRO2DP = 0.
    swdRO2ET = 0.
    
    cnppidPrecip = 0.
    cnppidAcs = 0.
    cnppidAP = 0.
    cnppidSWD = 0.
    cnppidET = 0.
    cnppidRO = 0.
    cnppidDP = 0.
    cnppidSL = 0.
    cnppidETg = 0.
    cnppidETb = 0.
    cnppidDET1 = 0.
    cnppidDET2 = 0.
    cnppidDP1 = 0.
    cnppidDP2 = 0.
    cnppidRO1 = 0.
    cnppidRO2 = 0.
    cnppidDP2RO = 0.
    cnppidETtrans = 0.
    cnppidLatseep = 0.
    cnppidSF = 0.
    cnppidRO2DP = 0.
    cnppidRO2ET = 0.
    
    uAreaPrecip = 0.
    uAreaAcs = 0.
    uAreaAP = 0.
    uAreaSWD = 0.
    uAreaET = 0.
    uAreaRO = 0.
    uAreaDP = 0.
    uAreaSL = 0.
    uAreaETg = 0.
    uAreaETb = 0.
    uAreaDET1 = 0.
    uAreaDET2 = 0.
    uAreaDP1 = 0.
    uAreaDP2 = 0.
    uAreaRO1 = 0.
    uAreaRO2 = 0.
    uAreaDP2RO = 0.
    uAreaETtrans = 0.
    uAreaLatseep = 0.
    uAreaSF = 0.
    uAreaRO2DP = 0.
    uAreaRO2ET = 0.


END SUBROUTINE zerovals
!****************************************************************************